(**********************************************************************
  intel Native Signal Processing Library (Version 3) Interface Unit

  Author:	Robert Rossmair
		Wagelwitz, Rodaer Str. 6
		04688 Mutzschen
		Germany

  e-mail:	Robert.Rossmair@t-online.de

  Version:	01-Mar-1998

  History:
	01-Mar-1998:
		Boris Sabanin pointed me to another oversight:
		Some constants defined in nspcvrt.h (NSP_Round..
		NSP_OvfErr) are in octal notation. This has not
		been taken into account in prior versions.
		I fixed NSP.pas in this respect.
	23-Feb-1998:
		Another hint from Boris Sabanin caused me to replace
		all occurences of "Result" in inline assembler code
		by "@Result" to make it work with {$X-} ("extended
		syntax off" compiler directive).
	11-FEB-1998: Thanks to a hint from Boris Sabanin, a calling
		convention incompatibility between Delphi and NSP.DLL
		has been detected. Due to that incompatibility, with
		version 17-SEP-1997 all functions having SCplx type
		results will return wrong values. Work-arounds using
		assembler code wrappers have been implemented.
		In addition, I corrected some oversights.
	17-SEP-1997: First release

  This file imports all of the 872 procedures and functions exported
  by NSP.DLL. It is an Object Pascal translation of the NSP library
  header files.

  Use, modification and distribution is allowed without limitation,
  warranty, or liability of any kind.
 **********************************************************************)
unit NSP;

(*==== NSP Library Control Symbols  =========================================

nsp_UsesVector:     vector/scalar Zero,Set,Copy,Add,Sub,Mpy,Div,Conj
		    b<,Inv>Tresh<1,2>,bAbs<1,2>,bAdd<2,3>,bExp<1,2>,bLn<1,2>,
		    bMpy<2,3>, bSqr<1,2>, bSqrt<1,2>, Max, Mean, Min, StDev
		    bConj<,Extend><1,2>, bConjFlip2
		    b<Mu,A>LawToLin, bLinTo<Mu,A>Law
		    AutoCorr<Biased,Unbiased>, CrossCorr
nsp_UsesConversion: bImag, bReal, bCplxTo2Real, b2RealToCplx,
		    b<,r>Mag, b<,rPhase>,
		    bFloatTo<Int,Fix,S31Fix,S15Fix,S7Fix,S1516Fix>,
		    b<Int,Fix,S31Fix,S15Fix,S7Fix,S1516Fix>ToFloat,
		    b<,r>CartToPolar, b<,r>PolarToCart
nsp_UsesConvolution:Conv
		    Conv2D
		    Filter2D
nsp_UsesTransform:  Dft
		    Fft<,Nip>, <Real,Ccs>Fftl<,Nip>, MpyRC<Perm,Pack><2,3>
		    <Real,Ccs><,2>Fft<,Nip>
		    Goertz
nsp_UsesFir:        struct FirState,FirInit<,Mr>,FirFree,Fir<Get,Set><Taps,Dlyl>
		    struct Fir<Taps|Dly>State, <Up,Down>Sample,
		    FirlInit<,Mr,Dlyl>, Firl, Firl<Get,Set><Taps,Dlyl>
nsp_UsesIir:        struct Iir<Tap,Dly>State, IirlInit<,Bq,Dlyl>, Iirl
		    struct Iir<Tap,Dly>State, IirInit<,Bq>, Iir, IirFree
nsp_UsesLms:        struct Lms<Tap,Dly>State, LmslInit<,Mr,Dlyl>,
		    Lmsl<Get,Set><Step,Leak>, Lmsl<,Na>
nsp_UsesMisc:       BitRevTbl,BitRev,
		    TwdTbl
nsp_UsesSampleGen:  struct ToneState, ToneInit, Tone
		    struct TrnglState, TrnglInit, Trngl
		    bRandUni, RandUni, RandUniInit
		    bRandGauss, RandGauss, RandGausInit
nsp_UsesWin:        Win<Bartlett,Hann,Hamming,Blackman<,Std,Opt>,Kaiser>

(*==== NSP Library Parts ==================================================*)

{ $IFDEF nsp_UsesAll}
{$DEFINE nsp_UsesConvolution}
{$DEFINE nsp_UsesConversion}
{$DEFINE nsp_UsesFir}
{$DEFINE nsp_UsesIir}
{$DEFINE nsp_UsesLms}
{$DEFINE nsp_UsesMisc}
{$DEFINE nsp_UsesSampleGen}
{$DEFINE nsp_UsesTransform}
{$DEFINE nsp_UsesVector}
{$DEFINE nsp_UsesWin}
{ $ENDIF}

interface

uses Windows;

const
  NSP_EPS  = 1.0E-12;
  NSP_PI   = 3.14159265358979324; (* Standard C not support M_PI     *)
  NSP_2PI  = 6.28318530717958648;
  NSP_PI_2 = 1.57079632679489662;
  NSP_PI_4 = 0.785398163397448310;

// Fourier Transformation Constants -> TNspFourierTrOptions
(*
  NSP_Forw		= 1;
  NSP_Inv		= 2;
  NSP_Init		= 4;
  NSP_NoScale		= 8;
  NSP_NoBitRev		= 16;
  NSP_InBitRev		= 32;
  NSP_OutBitRev		= 64;
  NSP_OutRCPack		= 128;
  NSP_OutRCPerm		= 256;
  NSP_InRCPack		= 512;
  NSP_InRCPerm		= 1024;
  NSP_UseMMX		= 4096;
  NSP_UseFastMMX	= 8192;
  NSP_RADIX		= 16384;
*)
// NSPErrStatus,NSPErrMode Values Definition

  NSP_StsOk            =  0;	// everything is ok
  NSP_StsBackTrace     = -1;	// pseudo error for back trace
  NSP_StsError         = -2;	// unknown /unspecified error
  NSP_StsInternal      = -3;	// internal error (bad state)
  NSP_StsNoMem         = -4;	// out of memory
  NSP_StsBadArg        = -5;	// function arg/param is bad
  NSP_StsBadFunc       = -6;	// unsupported function
  NSP_StsNoConv        = -7;	// iter. didn't converge
  NSP_StsAutoTrace     = -8;	// Tracing through nsptrace.h

  NSP_ErrModeLeaf     = 0;	// Print error and exit program
  NSP_ErrModeParent   = 1;	// Print error and continue
  NSP_ErrModeSilent   = 2;	// Don't print and continue

// Flags for scaleMode parameter of nsp?Func(...,scaleMode,scaleFactor)

  NSP_NO_SCALE        = $0000;	// ignore factor
  NSP_FIXED_SCALE     = $0002;	// fixed factor value
  NSP_AUTO_SCALE      = $0004;	// detect factor value

  NSP_OVERFLOW        = $0000;	// wrapround
  NSP_SATURATE        = $0001;	// saturate

  XBUFLEN = 32;

type
  Short = Smallint;
  Long	= Longint;
  Float = Single;

  PSCplx = ^TSCplx;
  TSCplx = record
    re: float;
    im:	float;
  end;

  PDCplx = ^TDCplx;
  TDCplx = record
    re: Double;
    im: Double;
  end;

  PWCplx = ^TWCplx;
  TWCplx = record
    re: Short;
    im: Short;
  end;

  PICplx = ^TICplx;
  TICplx = record
    re: Integer;
    im: Integer;
  end;

  ArrayIndex = 0..99999999;	// 0..10**8-1

  TByteArray = array[ArrayIndex] of Byte;	PByteArray = ^TByteArray;
  TShortArray = array[ArrayIndex] of Short;	PShortArray = ^TShortArray;
  TIntArray = array[ArrayIndex] of Integer;	PIntArray = ^TIntArray;
  TLongArray = array[ArrayIndex] of Long;	PLongArray = ^TLongArray;
  TFloatArray = array[ArrayIndex] of Float;	PFloatArray = ^TFloatArray;
  TDoubleArray = array[ArrayIndex] of Double;	PDoubleArray = ^TDoubleArray;
  TWCplxArray = array[ArrayIndex] of TWCplx;	PWCplxArray = ^TWCplxArray;
  TSCplxArray = array[ArrayIndex] of TSCplx;	PSCplxArray = ^TSCplxArray;
  TDCplxArray = array[ArrayIndex] of TDCplx;	PDCplxArray = ^TDCplxArray;
  TICplxArray = array[ArrayIndex] of TICplx;	PICplxArray = ^TICplxArray;

  PNSPLibVersion = ^TNspLibVersion;
  TNspLibVersion = record
    major: Integer;		// e.g. 3                          */
    minor: Integer;		// e.g. 00                         */
    build: Integer;		// e.g. 11                         */
    Name: PChar;		// e.g. "nspp6l.lib","nspm5.dll"   */
    Version: PChar;		// e.g. "v3.00"                    */
    InternalVersion: PChar;	// e.g. "[3.00.11, 07/12/96]"      */
    BuildDate: PChar;		// e.g. "Jun 1 96"                 */
    CallConv: PChar;		// e.g. "DLL","Microsoft","Borland"*/
  end;

  NSPStatus = Integer;
  TNspErrorCallBack = function(
	Status: NSPStatus;
	Funcname: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): Integer cdecl;

  TNspFourierTrOptions = set of (
	nspForw,
	nspInv,
	nspInit,
	nspNoScale,
	nspNoBitRev,
	nspInBitRev,
	nspOutBitRev,
	nspOutRCPack,
	nspOutRCPerm,
	nspInRCPack,
	nspInRCPerm,
	nspUseMMX,
	nspUseFastMMX,
	nspRADIX);

// Single Frequency DFT structures.                                        */

  TNspSGoertzState = record
    freq: Float;
    cosV: Float;
    sinV: Float;
    cpcV: Float;
    prev2re: Float;
    prev2im: Float;
    prev1re: Float;
    prev1im: Float;
  end;
  TNspCGoertzState = TNspSGoertzState;
  TNspWGoertzState = TNspSGoertzState;
  TNspVGoertzState = TNspSGoertzState;

  TNspDGoertzState = record
    freq: Double;
    cosV: Double;
    sinV: Double;
    cpcV: Double;
    prev2re: Double;
    prev2im: Double;
    prev1re: Double;
    prev1im: Double;
  end;
  TNspZGoertzState = TNspDGoertzState;

  TNSPSRandUniState = record
    Low, Muler: Float;		// Lower value and scale multiplier */
    Seed2, Carry: UInt;		// Variables of the algorithm       */
    seed1: array[0..2] of UInt;	// Variables of the algorithm       */
    QuickSeed: Integer;
  end;

  TNSPDRandUniState = record
    Low, Muler: Double;		// Lower value and scale multiplier */
    Seed2, Carry: UInt;		// Variables of the algorithm       */
    seed1: array[0..2] of UInt;	// Variables of the algorithm       */
    QuickSeed: Integer;
  end;
(*
   A new state structure of the pseudo random numbers
   generator has two seed values (m_seed) now. Generator
   based on multiplication uses m_seed values. Generator
   based on subtraction uses d_seed and carry.
*)
  TNSPWRandUniState = record
    Low, Muler: Float;		// Lower value and scale multiplier */
    Center, Pwr2: Integer;
    M_seed: array[0..1] of UInt;
    D_seed: array[0..2] of UInt;
    Carry: UInt;
    QuickSeed: Integer;
    Current: Integer;
    XBuf: array[0..XBUFLEN-1] of Short;
  end;

  TNSPCRandUniState = TNSPSRandUniState;
  TNSPZRandUniState = TNSPDRandUniState;
  TNSPVRandUniState = TNSPWRandUniState;

  TNSPSRandGausState = record
    Mean, StDev: Float;		// Mean and standard deviation values */
    ExtraValue: Float;		// Value computed early               */
    UseExtra: Integer;		// Flag to use extra value            */
    Seed2, Carry: UInt;		// Variables of the algorithm         */
    Seed1: array[0..2] of UInt;	// Variables of the algorithm         */
    QuickSeed: UInt;
  end;

  TNSPDRandGausState = record
    Mean, StDev: Double;	// Mean and standard deviation values */
    ExtraValue: Double;		// Value computed early               */
    UseExtra: Integer;		// Flag to use extra value            */
    Seed2, Carry: UInt;		// Variables of the algorithm         */
    Seed1: array[0..2] of UInt;	// Variables of the algorithm         */
    QuickSeed: UInt;
  end;

  TNSPWRandGausState = record
    UseExtra: Integer;		// Flag to use extra value            */
    Seed2, Carry: UInt;		// Variables of the algorithm         */
    Seed1: array[0..2] of UInt;	// Variables of the algorithm         */
    Mean, StDev: Short;		// Mean and standard deviation values */
    ExtraValue: Short;		// Value computed early               */
    QuickSeed: UInt;
  end;

  TNSPCRandGausState = TNSPSRandGausState;
  TNSPZRandGausState = TNSPDRandGausState;
  TNSPVRandGausState = TNSPWRandGausState;

//--- Tone structures -----------------------------------------------------*/

  TNSPSToneState = record
    cosBase: Double;
    cosCurr: Double;
    cosPrev: Double;
    mag: Float;
    rfreq: Float;
    phase: Float;
    isInit: array[0..3] of Char;
  end;

  TNSPCToneState = record
    cosBase: Double;
    cosCurr: TDCplx;
    cosPrev: TDCplx;
    mag: Float;
    rfreq: Float;
    phase: Float;
    isInit: array[0..3] of Char;
  end;

  TNSPDToneState = record
    cosBase: Double;
    cosCurr: Double;
    cosPrev: Double;
    mag:   Double;
    rfreq: Double;
    phase: Double;
    isInit: array[0..3] of Char;
  end;

  TNSPZToneState = record
    cosBase: Double;
    cosCurr: TDCplx;
    cosPrev: TDCplx;
    mag:   Double;
    rfreq: Double;
    phase: Double;
    isInit: array[0..3] of Char;
  end;

  TNSPWToneState = record
    dummy: array[0..31] of Double;
  end;

  TNSPVToneState = record
    dummy: array[0..31] of Double;
  end;

//--- Triangle structures -------------------------------------------------*/

  TNSPSTrnglState = record
    mag: Float;
    step: Float;
    step1: Float;
    step2: Float;
    st12: Float;
    st21: Float;
    last: Float;
  end;
  TNSPWTrnglState = TNSPSTrnglState;

  TNSPCTrnglState = record
    mag: Float;
    step: TSCplx;
    step1: Float;
    step2: Float;
    st12: Float;
    st21: Float;
    last: TSCplx;
  end;
  TNSPVTrnglState = TNSPCTrnglState;

  TNSPDTrnglState = record
    mag: Double;
    step: Double;
    step1: Double;
    step2: Double;
    st12: Double;
    st21: Double;
    last: Double;
  end;
  TNSPITrnglState = TNSPDTrnglState;

  TNSPZTrnglState = record
    mag: Double;
    step: TDCplx;
    step1: Double;
    step2: Double;
    st12: Double;
    st21: Double;
    last: TDCplx;
  end;
  TNSPJTrnglState = TNSPZTrnglState;

//  NSP_DegToRad(deg) ((deg)/180.0 * NSP_PI)

function nspSDegToRad(deg: float): float;
function nspDDegToRad(deg: double): double;

//  MAX(a,b) (((a) > (b)) ? (a) : (b))
//  MIN(a,b) (((a) < (b)) ? (a) : (b))

// nsperror.h
function nspGetLibVersion: PNSPLibVersion; cdecl;
function nspGetErrStatus: NSPStatus; cdecl;
procedure nspSetErrStatus(Status: NSPStatus); cdecl;
function nspGetErrMode: Integer; cdecl;
procedure nspSetErrMode(Mode: Integer); cdecl;
function nspError(
	Status: NSPStatus;
	Func: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; cdecl;
function nspErrorStr(Status: NSPStatus): PChar; cdecl;
function nspNulDevReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; cdecl;
function nspStdErrReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; cdecl;
function nspGuiBoxReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; cdecl;
function nspRedirectError(nspErrorFunc: TNspErrorCallBack): TNspErrorCallBack; cdecl;
// nspalloc.h
function nspMalloc(Length: Integer): Pointer; cdecl;
function nspsMalloc(Length: Integer): PFloatArray; cdecl;
function nspdMalloc(Length: Integer): PDoubleArray; cdecl;
function nspcMalloc(Length: Integer): PSCplxArray; cdecl;
function nspzMalloc(Length: Integer): PDCplxArray; cdecl;
function nspwMalloc(Length: Integer): PShortArray; cdecl;
function nspvMalloc(Length: Integer): PWCplxArray; cdecl;
function nspiMalloc(Length: Integer): PLongArray; cdecl;
function nspjMalloc(Length: Integer): PICplxArray; cdecl;
procedure nspFree(P: Pointer); cdecl;

{$IFDEF nsp_UsesConvolution}
//
//  RCS:
//      $Source: nsp30/include/nspconv.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 10:57:14 $
//
//  Purpose: NSP Convolution

(*-------------------------------------------------------------------------*)
(*         Conv                                                            *)
(*                                                                         *)
(* Performs finite, linear convolution of two sequences.                   *)

procedure nspsConv(x: PFloatArray; xLen: Integer; h: PFloatArray; hLen: Integer; y: PFloatArray); cdecl;
procedure nspcConv(x: PSCplxArray; xLen: Integer; h: PSCplxArray; hLen: Integer; y: PSCplxArray); cdecl;
procedure nspdConv(x: PDoubleArray; xLen: Integer; h: PDoubleArray; hLen: Integer; y: PDoubleArray); cdecl;
procedure nspzConv(x: PDCplxArray; xLen: Integer; h: PDCplxArray; hLen: Integer; y: PDCplxArray); cdecl;
procedure nspscConv(x: PFloatArray; xLen: Integer; h: PSCplxArray; hLen: Integer; y: PSCplxArray); cdecl;
procedure nspcsConv(x: PSCplxArray; xLen: Integer; h: PFloatArray; hLen: Integer; y: PSCplxArray); cdecl;
procedure nspdzConv(x: PDoubleArray; xLen: Integer; h: PDCplxArray; hLen: Integer; y: PDCplxArray); cdecl;
procedure nspzdConv(x: PDCplxArray; xLen: Integer; h: PDoubleArray; hLen: Integer; y: PDCplxArray); cdecl;
procedure nspwConv(x: PShortArray; xLen: Integer; h: PShortArray; hLen: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nspcnv2d.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 10:57:04 $
//
//  Purpose: NSP 2D Filtering Functions

(*
 * Conv2D
 *
 * Perform finite, linear convolution of two two-dimensional signals.
 *)

procedure nspsConv2D(x: PFloatArray; xCols, xRows: Integer; h: PFloatArray; hCols, hRows: Integer; y: PFloatArray); cdecl;
procedure nspdConv2D(x: PDoubleArray; xCols, xRows: Integer; h: PDoubleArray; hCols, hRows: Integer; y: PDoubleArray); cdecl;
procedure nspwConv2D(x: PShortArray; xCols, xRows: Integer; h: PShortArray; hCols, hRows: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nspfir2.h $
//      $Revision: 1.1 $
//      $Date: 1995/08/01 11:04:38 $
//
//  Purpose: NSP20 Filter2D

procedure nspsFilter2D(x: PFloatArray; xCols, xRows: Integer; h: PFloatArray; hCols, hRows: Integer; y: PFloatArray); cdecl;
procedure nspdFilter2D(x: PDoubleArray; xCols, xRows: Integer; h: PDoubleArray; hCols, hRows: Integer; y: PDoubleArray); cdecl;
procedure nspwFilter2D(x: PShortArray; xCols, xRows: Integer; h: PShortArray; hCols, hRows: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
{$ENDIF}
{$IFDEF nsp_UsesConversion}
//
//  RCS:
//      $Source: nsp30/include/nspcvrt.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 10:58:00 $
//
//  Purpose: Data Convertion Functions.

//-------Flags to determine the transformation-----------------------------*/
const
  NSP_Noflags   = $0000;	// Specifies an absence of all flags       */

  NSP_Round     = $0080;	// Specifies that floating-point values    */
				// should be rounded to the nearest integer*/
  NSP_TruncZero = $0100;	// Specifies that floating-point values    */
				// should be truncated toward zero         */
  NSP_TruncNeg  = $0200;	// Specifies that floating-point values    */
				// should be truncated toward negative     */
				// infinity                                */
  NSP_TruncPos  = $0400;	// Specifies that floating-point values    */
				// should be truncated toward positive     */
				// infinity                                */
  NSP_Unsigned  = $0800;	// Specifies that integer values are       */
				// unsigned                                */
  NSP_Clip      = $1000;	// Specifies that floating-point values    */
				// outside the allowable integer range     */
				// are saturated to maximum(minimum)       */
				// integer value                           */
  NSP_OvfErr    = $2000;	// Specifies that an overflow error should */
				// be signaled with a call to nspError()   */

//-------------------------------------------------------------------------*/
// Maximum possible phase value for short function, corresponds to NSP_PI  */

  NSP_CVRT_MAXPHASE = 16383;

//-------------------------------------------------------------------------*/
//                        Short functions                                  */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nspvbReal                                                             */
// DESCRIPTION:                                                            */
//   Return the real part of the coplex vector                             */
// PARAMETERS:                                                             */
//   src       - an input complex vector;                                  */
//   dst       - an output vector to store the real part;                  */
//   len       - a length of the arrays;                                   */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//   These errors are registered only if NSP_DEBUG is defined              */

procedure nspvbReal(Src: PWCplxArray; Dst: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspvbImag                                                             */
// DESCRIPTION:                                                            */
//   Return the imaginary part of the coplex vector                        */
// PARAMETERS:                                                             */
//   src       - an input complex vector;                                  */
//   dst       - an output vector to store the imaginary part;             */
//   len       - a length of the arrays.                                   */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure nspvbImag(Src: PWCplxArray; Dst: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspvbMag                                                              */
// DESCRIPTION:                                                            */
//   Compute the magnitude of complex vector elements.                     */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   mag       - an output vector to store the magnitude components;       */
//   len       - a length of the arrays.                                   */
// ALGORITHM:                                                              */
//   Computation is perfomed using standart C mathematical functions.      */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspvbMag(Src: PWCplxArray; Mag: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspvbPhase                                                            */
// DESCRIPTION:                                                            */
//   Compute the phase (in radians) of complex vector elements.            */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   phase     - an output vector to store the phase components;           */
//   len       - a length of the arrays.                                   */
// ALGORITHM:                                                              */
//   Computation is perfomed using standart C mathematical functions.      */
//   Phase of (0.0, 0.0) is assumed to be 0.0.                             */
//   Phase values are in the range (-Pi, Pi].                              */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspvbPhase(Src: PWCplxArray; Phase: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspwbrMag                                                             */
// DESCRIPTION:                                                            */
//   Compute the magnitude of complex data formed as two real vectors.     */
// PARAMETERS:                                                             */
//   srcReal   - an input vector containing a real part of complex data    */
//   srcImag   - an input vector containing an imag. part of complex data  */
//   mag       - an output vector to store the magnitude components;       */
//   len       - a length of the arrays.                                   */
// ALGORITHM:                                                              */
//   Computation is perfomed using standart C mathematical functions.      */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspwbrMag(srcReal, srcImag: PShortArray; Mag: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspwbrPhase                                                           */
// DESCRIPTION:                                                            */
//   Compute the phase of complex data formed as two real vectors.         */
// PARAMETERS:                                                             */
//   srcReal   - an input vector containing a real part of complex data    */
//   srcImag   - an input vector containing an imag. part of complex data  */
//   phase     - an output vector to store the magnitude components;       */
//   len       - a length of the arrays.                                   */
// ALGORITHM:                                                              */
//   Computation is perfomed using standart C mathematical functions.      */
//   Phase of (0.0, 0.0) is assumed to be 0.0.                             */
//   Phase values are in the range (-Pi, Pi].                              */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspwbrPhase(srcReal, srcImag: PShortArray; Phase: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspvbCplxTo2Real                                                      */
// DESCRIPTION:                                                            */
//   Return the real and imaginary parts of the coplex vector              */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   dstReal   - an output vector to store the real part                   */
//   dstImag   - an output vector to store the imaginary part;             */
//   len       - a length of the arrays.                                   */
// ERRORS:                                                                 */
//    1) Some of pointers to input or output data are NULL                 */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspvbCplxTo2Real(Src: PWCplxArray; DstReal, DstImag: PShortArray; len: Integer); cdecl;

// FUNCTION:                                                               */
//   nspvb2RealToCplx                                                      */
// DESCRIPTION:                                                            */
//   Construct complex vector from real and imaginary components           */
// PARAMETERS:                                                             */
//   srcReal   - an input real component. May be NULL - a real part of the */
//               output will be zero;                                      */
//   srcImag   - an input imaginary component. May be NULL - an imaginary  */
//               part of the output will be zero;                          */
//   dst       - an output complex vector;                                 */
//   len       - a length of the arrays.                                   */
// ERRORS:                                                                 */
//    1) The pointer to output data is NULL                                */
//    2) The length of the arrays is less or equal zero                    */
//  These errors are registered only if NSP_DEBUG is defined               */

procedure  nspvb2RealToCplx(srcReal, srcImag: PShortArray; Dst: PWCplxArray; len: Integer); cdecl;

//-------------------------------------------------------------------------*/
//             bFloatToInt, bIntToFloat                                    */
//                                                                         */
//Vector floating-point to integer and integer to floating-point conversion*/
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<s,d>bFloatToInt                                                   */
// DESCRIPTION:                                                            */
//   Converts the floating-point data in the input array and stores        */
//   the results in the output array.                                      */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//   wordSize  - specifies the size of an integer in bits,                 */
//                     and must be 8, 16 or 32;                            */
//   flags     - specifies the sort of the conversion and consists of the  */
//               bitwise-OR of flags, defined in the beginnig of this file */
//                                                                         */

procedure nspsbFloatToInt(Src: PFloatArray; var dst; len, wordSize, Flags: Integer); cdecl;
procedure nspdbFloatToInt(Src: PDoubleArray; var dst; len, wordSize, Flags: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>bIntToFloat                                                   */
// DESCRIPTION:                                                            */
//   Converts the integer data in the input array and stores the results   */
//   in the output array                                                   */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//   wordSize  - specifies the size of an integer in bits,                 */
//                     and must be 8, 16 or 32;                            */
//   flags     - specifies that integer values are unsigned or signed.     */
//                                                                         */

procedure nspsbIntToFloat(const src; Dst: PFloatArray; len, wordSize, flags: Integer); cdecl;
procedure nspdbIntToFloat(const src; Dst: PDoubleArray; len, wordSize, flags: Integer); cdecl;

//-------------------------------------------------------------------------*/
//             bFloatToFix, bFixToFloat                                    */
//                                                                         */
// Vector floating-point to fixed-point and                                */
//            fixed-point to floating-point conversion                     */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<s,d>bFloatToFix                                                   */
// DESCRIPTION:                                                            */
//   Converts the floating-point data in the input array and stores        */
//   the results in the output array.                                      */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//   wordSize  - specifies the size of an fix-point in bits,               */
//                     and must be 8, 16 or 32;                            */
//   fractBits - specifies the number of fractional bits                   */
//   flags     - specifies the sort of the conversion and consists of the  */
//               bitwise-OR of flags, defined in the beginnig of this file */
//                                                                         */

procedure nspsbFloatToFix(Src: PFloatArray; var dst; len, wordSize, fractBits, flags: Integer); cdecl;
procedure nspdbFloatToFix(Src: PDoubleArray; var dst; len, wordSize, fractBits, flags: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>bFixToFloat                                                   */
// DESCRIPTION:                                                            */
//   Converts the fixed-point data in the input array and stores the       */
//       results in the output array                                       */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//   wordSize  - specifies the size of an integer in bits,                 */
//                     and must be 8, 16 or 32;                            */
//   fractBits - specifies the number of fractional bits                   */
//   flags     - specifies that fixed-point values are unsigned or signed  */
//                                                                         */

procedure nspsbFixToFloat(const src; Dst: PFloatArray; len, wordSize, fractBits, flags: Integer); cdecl;
procedure nspdbFixToFloat(const src; Dst: PDoubleArray; len, wordSize, fractBits, flags: Integer); cdecl;

//-------------------------------------------------------------------------*/
// bFloatToS31Fix, bFloatToS15Fix, bFloatToS7Fix, bFloatToS1516Fix,        */
// bS31FixToFloat, bS15FixToFloat, bS7FixToFloat, bS1516FixToFloat         */
// Vector floating-point to fixed-point of format S.31, S.15, S.7, S15.16  */
//                and fixed-point of format S.31, S.15, S.7, S15.16        */
//                to floating-point conversion                             */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<s,d>bFloatToS31Fix                                                */
//   nsp<s,d>bFloatToS15Fix                                                */
//   nsp<s,d>bFloatToS7Fix                                                 */
//   nsp<s,d>bFloatToS1516Fix                                              */
// DESCRIPTION:                                                            */
//   Converts the floating-point data in the input array and stores        */
//   the results in the output array.                                      */
//   nsp?bFloatToS31Fix() to format of S.31 (a sign bit,31 fractional bits)*/
//   nsp?bFloatToS15Fix() to format of S.15 (a sign bit,15 fractional bits)*/
//   nsp?bFloatToS7Fix()  to format of S.7  (a sign bit,7  fractional bits)*/
//   nsp?bFloatToS1516Fix() to format of S15.16                            */
//                    (a sign bit, 15 integer bits, 16 fractional bits)    */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//   flags     - specifies the sort of the conversion and consists of the  */
//               bitwise-OR of flags, defined in the beginnig of this file */

procedure nspsbFloatToS31Fix(Src: PFloatArray; Dst: PLongArray; Len, Flags: Integer); cdecl;
procedure nspsbFloatToS15Fix(Src: PFloatArray; Dst: PShortArray; Len, Flags: Integer); cdecl;
procedure nspsbFloatToS7Fix(Src: PFloatArray; Dst: PByteArray; Len, Flags: Integer); cdecl;
procedure nspsbFloatToS1516Fix(Src: PFloatArray; Dst: PLongArray; Len, Flags: Integer); cdecl;

procedure nspdbFloatToS31Fix(Src: PDoubleArray; Dst: PLongArray; Len, Flags: Integer); cdecl;
procedure nspdbFloatToS15Fix(Src: PDoubleArray; Dst: PShortArray; Len, Flags: Integer); cdecl;
procedure nspdbFloatToS7Fix(Src: PDoubleArray; Dst: PByteArray; Len, Flags: Integer); cdecl;
procedure nspdbFloatToS1516Fix(Src: PDoubleArray; Dst: PLongArray; Len, Flags: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>bS31FixToFloat                                                */
//   nsp<s,d>bS15FixToFloat                                                */
//   nsp<s,d>bS7FixToFloat                                                 */
//   nsp<s,d>bS1516FixToFloat                                              */
// DESCRIPTION:                                                            */
//   Converts the fixed-point data in the input array and stores the       */
//       results in the output array                                       */
//  nsp?bS31FixToFloat() from format of S.31(a sign bit,31 fractional bits)*/
//  nsp?bS15FixToFloat() from format of S.15(a sign bit,15 fractional bits)*/
//  nsp?bS7FixToFloat()  from format of S.7 (a sign bit,7  fractional bits)*/
//  nsp?bS1516FixToFloat() from format of S15.16                           */
//                    (a sign bit, 15 integer bits, 16 fractional bits)    */
// PARAMETERS:                                                             */
//   src       - an input array to be converted;                           */
//   dst       - an output array to store the result;                      */
//   len       - a length of the arrays;                                   */
//                                                                         */

procedure nspsbS31FixToFloat(Src: PLongArray ; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspsbS15FixToFloat(Src: PShortArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspsbS7FixToFloat(Src: PByteArray ; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspsbS1516FixToFloat(Src: PLongArray ; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspdbS31FixToFloat(Src: PLongArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspdbS15FixToFloat(Src: PShortArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspdbS7FixToFloat(Src: PByteArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspdbS1516FixToFloat(Src: PLongArray; Dst: PDoubleArray; Len: Integer); cdecl;

//-------------------------------------------------------------------------*/
//  bReal, bImag, bCplxTo2Real, b2RealToCplx                               */
//                                                                         */
// Return the real and imaginary parts of complex vectors                  */
// or construct complex vectors from real and imaginary components         */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<c,z>bReal                                                         */
// DESCRIPTION:                                                            */
//   Return the real part of the coplex vector                             */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   dst       - an output vector to store the real part;                  */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbReal(Src: PSCplxArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspzbReal(Src: PDCplxArray; Dst: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<c,z>bImag                                                         */
// DESCRIPTION:                                                            */
//   Return the imaginary part of the coplex vector                        */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   dst       - an output vector to store the imaginary part;             */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbImag(Src: PSCplxArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspzbImag(Src: PDCplxArray; Dst: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<c,z>bCplxTo2Real                                                  */
// DESCRIPTION:                                                            */
//   Return the real and imaginary parts of the coplex vector              */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   dstReal   - an output vector to store the real part                   */
//   dstImag   - an output vector to store the imaginary part;             */
//   len       - a length of the arrays;                                   */
//                                                                         */

procedure nspcbCplxTo2Real(Src: PSCplxArray; DstReal, DstImag: PFloatArray; Len: Integer); cdecl;
procedure nspzbCplxTo2Real(Src: PDCplxArray; DstReal, DstImag: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<c,z>b2RealToCplx                                                  */
// DESCRIPTION:                                                            */
//   Construct complex vector from real and imaginary components           */
// PARAMETERS:                                                             */
//   srcReal   - an input real component. May be NULL - a real part of the */
//               output will be zero;                                      */
//   srcImag   - an input imaginary component. May be NULL - an imaginary  */
//               part of the output will be zero;                          */
//   dst       - an output complex vector;                                 */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcb2RealToCplx(srcReal, srcImag: PFloatArray; Dst: PSCplxArray; Len: Integer); cdecl;
procedure nspzb2RealToCplx(srcReal, srcImag: PDoubleArray; Dst: PDCplxArray; Len: Integer); cdecl;

//-------------------------------------------------------------------------*/
//  bCartToPolar, brCartToPolar, bPolarToCart, brPolarToCart               */
//                                                                         */
// Cartesian to polar and polar to cartesian coordinate conversions.       */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<c,z>bCartToPolar                                                  */
// DESCRIPTION:                                                            */
//   Convert cartesian coordinate to polar. Input data are formed as       */
//            a complex vector.                                            */
// PARAMETERS:                                                             */
//   src       - an input complex vector;                                  */
//   mag       - an output vector to store the magnitude components;       */
//   phase     - an output vector to store the phase components (in rad)); */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbCartToPolar(Src: PSCplxArray; Mag, Phase: PFloatArray; Len: Integer); cdecl;
procedure nspzbCartToPolar(Src: PDCplxArray; Mag, Phase: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>brCartToPolar                                                 */
// DESCRIPTION:                                                            */
//   Convert cartesian coordinate to polar. Input data are formed as       */
//            two different real vectors.                                  */
// PARAMETERS:                                                             */
//   srcReal   - an input vector containing the coordinates X;             */
//   srcImag   - an input vector containing the coordinates Y;             */
//   mag       - an output vector to store the magnitude components;       */
//   phase     - an output vector to store the phase components (in rad)); */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspsbrCartToPolar(srcReal, srcImag: PFloatArray; Mag, Phase: PFloatArray; Len: Integer); cdecl;
procedure nspdbrCartToPolar(srcReal, srcImag: PDoubleArray; Mag, Phase: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<c,z>bPolarToCart                                                  */
// DESCRIPTION:                                                            */
//   Convert polar coordinate to cartesian. Output data are formed as      */
//            a complex vector.                                            */
// PARAMETERS:                                                             */
//   mag       - an input vector containing the magnitude components;      */
//   phase     - an input vector containing the phase components(in rad)); */
//   dst       - an output complex vector to store the cartesian coords;   */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbPolarToCart(Mag, Phase: PFloatArray; Dst: PSCplxArray; Len: Integer); cdecl;
procedure nspzbPolarToCart(Mag, Phase: PDoubleArray; Dst: PDCplxArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>brPolarToCart                                                 */
// DESCRIPTION:                                                            */
//   Convert polar coordinate to cartesian. Output data are formed as      */
//            two real vectors.                                            */
// PARAMETERS:                                                             */
//   mag       - an input vector containing the magnitude components;      */
//   phase     - an input vector containing the phase components(in rad)); */
//   dstReal   - an output complex vector to store the coordinates X;      */
//   dstImag   - an output complex vector to store the coordinates Y;      */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspsbrPolarToCart(Mag, Phase: PFloatArray; DstReal, DstImag: PFloatArray; Len: Integer); cdecl;
procedure nspdbrPolarToCart(Mag, Phase: PDoubleArray; DstReal, DstImag: PDoubleArray; Len: Integer); cdecl;

//-------------------------------------------------------------------------*/
//  bMag, brMag, bPhase, brPhase                                           */
//                                                                         */
// Compute the magnitude and phase of complex vector elements.             */
//-------------------------------------------------------------------------*/
// FUNCTION:                                                               */
//   nsp<c,z>bMag                                                          */
// DESCRIPTION:                                                            */
//   Compute the magnitude of complex vector elements.                     */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   mag       - an output vector to store the magnitude components;       */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbMag(Src: PSCplxArray; Mag: PFloatArray; Len: Integer); cdecl;
procedure nspzbMag(Src: PDCplxArray; Mag: PFloatArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>brMag                                                         */
// DESCRIPTION:                                                            */
//   Compute the magnitude of complex data formed as two real vectors.     */
// PARAMETERS:                                                             */
//   srcReal   - an input vector containing a real part of complex data    */
//   srcImag   - an input vector containing an imag. part of complex data  */
//   mag       - an output vector to store the magnitude components;       */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspsbrMag(srcReal, srcImag: PFloatArray; Mag: PFloatArray; Len: Integer); cdecl;
procedure nspdbrMag(srcReal, srcImag: PDoubleArray; Mag: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<c,z>bPhase                                                        */
// DESCRIPTION:                                                            */
//   Compute the phase (in radians) of complex vector elements.            */
// PARAMETERS:                                                             */
//   src       - an input complex vector                                   */
//   phase     - an output vector to store the phase components;           */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspcbPhase(Src: PSCplxArray; Phase: PFloatArray; Len: Integer); cdecl;
procedure nspzbPhase(Src: PDCplxArray; Phase: PDoubleArray; Len: Integer); cdecl;

// FUNCTION:                                                               */
//   nsp<s,d>brPhase                                                       */
// DESCRIPTION:                                                            */
//   Compute the phase of complex data formed as two real vectors.         */
// PARAMETERS:                                                             */
//   srcReal   - an input vector containing a real part of complex data    */
//   srcImag   - an input vector containing an imag. part of complex data  */
//   phase     - an output vector to store the magnitude components;       */
//   len       - a length of the arrays;                                   */
//                                                                         */
procedure nspsbrPhase(srcReal, srcImag: PFloatArray; Phase: PFloatArray; Len: Integer); cdecl;
procedure nspdbrPhase(srcReal, srcImag: PDoubleArray; Phase: PDoubleArray; Len: Integer); cdecl;

{$ENDIF}
{$IFDEF nsp_UsesFir}
//
//  RCS:
//      $Source: nsp30/include/nspfirl.h $
//      $Revision: 1.4 $
//      $Date: 1995/08/07 10:59:38 $
//
//  Purpose: NSP Finite Impulse Response low level filter
//

//* --- FIR low level filter structure -------------------------------------*/

type
  TFirLmsTapState = record
    Taps: Pointer;
    Len: Integer;
    UpFactor: Integer;
    UpPhase: Integer;
    DownFactor: Integer;
    DownPhase: Integer;
    IsMultiRate: Integer;
    IsFilterMode: Integer;
  end;

  TFirLmsDlyState = record
    Dlyl: Pointer;
    Len: Integer;
    IsFilterMode: Integer;
  end;

//* --- Finite impulse response filter -------------------------------------*/

  TNSPFirTapState = TFirLmsTapState;

//* --- Delay line of FIR low level filter ---------------------------------*/

  TNSPFirDlyState = TFirLmsDlyState;

//* ------------------------------------------------------------------------*/
//*        FirlInit, FirlInitMr, FirlInitDlyl                               */
//*                                                                         */
//* Low level functions for cyclic FIR filtering via a tapped delay line.   */

procedure nspsFirlInit(Taps: PFloatArray; tapsLen: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspdFirlInit(Taps: PDoubleArray; tapsLen: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspcFirlInit(Taps: PSCplxArray; tapsLen: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspzFirlInit(Taps: PDCplxArray; tapsLen: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspwFirlInit(Taps: PShortArray; tapsLen: Integer; var tapState: TNSPFirTapState); cdecl;

procedure nspsFirlInitMr(Taps: PFloatArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspdFirlInitMr(Taps: PDoubleArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspcFirlInitMr(Taps: PSCplxArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspzFirlInitMr(Taps: PDCplxArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); cdecl;
procedure nspwFirlInitMr(Taps: PShortArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); cdecl;

procedure nspsFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PFloatArray;
				    var dlyState: TNSPFirDlyState); cdecl;
procedure nspcFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PSCplxArray;
				    var dlyState: TNSPFirDlyState); cdecl;
procedure nspdFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PDoubleArray;
				    var dlyState: TNSPFirDlyState); cdecl;
procedure nspzFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PDCplxArray;
				    var dlyState: TNSPFirDlyState); cdecl;
procedure nspwFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PShortArray;
				    var dlyState: TNSPFirDlyState); cdecl;

//* ------------------------------------------------------------------------*/
//*        Firl, bFirl                                                      */
//*                                                                         */
//* Filter either a single sample or  block of samples through a low-level  */
//* FIR filter.                                                             */

function nspsFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: float): float; cdecl;
function nspcFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; cdecl;
function nspdFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: Double): Double; cdecl;
function nspzFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: TDCplx): TDCplx; cdecl;
function nspwFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: Short;
			      scaleMode: Integer; var scaleFactor: Integer): Short; cdecl;

function nspscFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: Float): TSCplx; cdecl;
function nspcsFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; cdecl;
function nspdzFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: Double): TDCplx; cdecl;
function nspzdFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: TDCplx): TDCplx; cdecl;

procedure nspsbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PFloatArray; outSamps: PFloatArray; numIters: Integer); cdecl;
procedure nspcbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PSCplxArray; outSamps: PSCplxArray; numIters: Integer); cdecl;
procedure nspdbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PDoubleArray; outSamps: PDoubleArray; numIters: Integer); cdecl;
procedure nspzbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PDCplxArray; outSamps: PDCplxArray; numIters: Integer); cdecl;
procedure nspwbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PShortArray; outSamps: PShortArray; numIters: Integer;
			   scaleMode: Integer; var scaleFactor: Integer); cdecl;

procedure nspscbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PFloatArray; outSamps: PSCplxArray; numIters: Integer); cdecl;
procedure nspcsbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PSCplxArray; outSamps: PSCplxArray; numIters: Integer); cdecl;
procedure nspdzbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PDoubleArray; outSamps: PDCplxArray; numIters: Integer); cdecl;
procedure nspzdbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PDCplxArray; outSamps: PDCplxArray; numIters: Integer); cdecl;

//* ------------------------------------------------------------------------*/
//*        FirlGetTaps, FirlSetTaps                                         */
//*                                                                         */
//* Utility functions to get and set the tap coefficients of low-level FIR  */
//* filters.                                                                */

procedure nspsFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PFloatArray); cdecl;
procedure nspcFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PSCplxArray); cdecl;
procedure nspdFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PDoubleArray); cdecl;
procedure nspzFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PDCplxArray); cdecl;
procedure nspwFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PShortArray); cdecl;

procedure nspsFirlSetTaps(InTaps: PFloatArray; var tapState: TNSPFirTapState); cdecl;
procedure nspcFirlSetTaps(InTaps: PSCplxArray; var tapState: TNSPFirTapState); cdecl;
procedure nspdFirlSetTaps(InTaps: PDoubleArray; var tapState: TNSPFirTapState); cdecl;
procedure nspzFirlSetTaps(InTaps: PDCplxArray; var tapState: TNSPFirTapState); cdecl;
procedure nspwFirlSetTaps(InTaps: PShortArray; var tapState: TNSPFirTapState); cdecl;

// ------------------------------------------------------------------------*/
//        FirlGetDlyl, FirlSetDlyl                                         */
//                                                                         */
// Utility functions to get and set  the delay line contents of low-level  */
// FIR filters.                                                            */

procedure nspsFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PFloatArray); cdecl;
procedure nspcFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PSCplxArray); cdecl;
procedure nspdFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PDoubleArray); cdecl;
procedure nspzFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PDCplxArray); cdecl;
procedure nspwFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PShortArray); cdecl;


procedure nspsFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PFloatArray;
				   var dlyState: TNSPFirDlyState); cdecl;
procedure nspcFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PSCplxArray;
				   var dlyState: TNSPFirDlyState); cdecl;
procedure nspdFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PDoubleArray;
				   var dlyState: TNSPFirDlyState); cdecl;
procedure nspzFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PDCplxArray;
				   var dlyState: TNSPFirDlyState); cdecl;
procedure nspwFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PShortArray;
				   var dlyState: TNSPFirDlyState); cdecl;

//
//  RCS:
//      $Source: nsp30/include/nspfirh.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/03 12:04:36 $
//
//  Purpose: NSP Finite Impulse Response high-level filter.
//

//=== FIR high-level ======================================================*/

type
  firSect = record
    num: Integer;                             // section number                  */
    len: Integer;                             // section taps length in use      */
    xId: Integer;                             // section input data index        */
    tId: Integer;                             // section taps index              */
   // ---- fields below are special for w-flavor ------------------------- */
    initlen: Integer;			// section taps length             */
    tapsGate: array[0..3] of Pointer;	// shifted section taps pointers   */
  end; // firSect

type
  TNSPFirState = record
    upFactor:	Integer;		// up                              */
    upPhase:	Integer;                //  parameters                     */
    downFactor:	Integer;                // down                            */
    downPhase:	Integer;                //  parameters                     */
    isMultiRate:Integer;                // multi-rate mode flag            */
    isInit:	Pointer;                // init flag                       */

    tapsLen:	Integer;                // taps length in use              */
    taps:	Pointer;                // taps pointer in use             */
    tapsBlk:	Integer;

    dlylLen:	Integer;                // delay line length               */
    dlyl:	Pointer;		// delay line pointer              */
    dlylLimit:	Pointer;		// delay line buffer end pointer   */
    useDlyl:	Pointer;		// delay line pointer in use       */
    inpLen:	Integer;                // input buffer length             */
    useInp:	Integer;                // input buffer length used        */

    sect: ^firSect;			// FIR sections                    */

    // ---- fields below are special for w-flavor ------------------------- */
    tapsInitLen:Integer;                // init taps length                */
    tapsGate: array[0..3] of Pointer;	// shifted taps addresses          */
  end;

//-------------------------------------------------------------------------*/
// nsp?FirInit(), nsp?FirInitMr()                                          */
// nspFirFree()                                                            */
//                                                                         */
// Initialize a finite impulse response high-level (FIRH) filter           */

procedure nspsFirInit(TapVals: PFloatArray; tapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); cdecl;
procedure nspcFirInit(TapVals: PSCplxArray; tapsLen: Integer;
			  DlyVals: PSCplxArray; var State: TNSPFirState); cdecl;
procedure nspscFirInit(TapVals: PSCplxArray; tapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); cdecl;
procedure nspcsFirInit(TapVals: PFloatArray; tapsLen: Integer;
			  DlyVals: PSCplxArray; var State: TNSPFirState); cdecl;

procedure nspdFirInit(TapVals: PDoubleArray; tapsLen: Integer;
			  DlyVals: PDoubleArray; var State: TNSPFirState); cdecl;
procedure nspzFirInit(TapVals: PDCplxArray; tapsLen: Integer;
			  DlyVals: PDCplxArray; var State: TNSPFirState); cdecl;
procedure nspdzFirInit(TapVals: PDCplxArray; tapsLen: Integer;
			  DlyVals: PDoubleArray; var State: TNSPFirState); cdecl;
procedure nspzdFirInit(TapVals: PDoubleArray; tapsLen: Integer;
			  DlyVals: PDCplxArray; var State: TNSPFirState); cdecl;

procedure nspwFirInit(TapVals: PShortArray; tapsLen: Integer;
			  DlyVals: PShortArray; var State: TNSPFirState); cdecl;

procedure nspsFirInitMr(TapVals: PFloatArray; tapsLen: Integer;
			    DlyVals: PFloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspcFirInitMr(TapVals: PSCplxArray; tapsLen: Integer;
			    DlyVals: PSCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspscFirInitMr(TapVals: PSCplxArray; tapsLen: Integer;
			    DlyVals: PfloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspcsFirInitMr(TapVals: PFloatArray; tapsLen: Integer;
			    DlyVals: PSCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;

procedure nspdFirInitMr(TapVals: PDoubleArray; tapsLen: Integer;
			    DlyVals: PDoubleArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspzFirInitMr(TapVals: PDCplxArray; tapsLen: Integer;
			    DlyVals: PDCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspdzFirInitMr(TapVals: PDCplxArray; tapsLen: Integer;
			    DlyVals: PDoubleArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;
procedure nspzdFirInitMr(TapVals: PDoubleArray; tapsLen: Integer;
			    DlyVals: PDCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); cdecl;

procedure nspwFirInitMr(TapVals: PShortArray; tapsLen: Integer;
			   DlyVals: PShortArray; upFactor, upPhase,
			   downFactor, downPhase: Integer;
			   var State: TNSPFirState); cdecl;

procedure nspFirFree(var State: TNSPFirState); cdecl;


//-------------------------------------------------------------------------*/
// nsp?Fir(), nsp?bFir()                                                   */
//                                                                         */
// Dot and block product FIRH filtering                                    */

function nspsFir(var State: TNSPFirState; samp: Float): Float; cdecl;
function nspcFir(var State: TNSPFirState; samp: TSCplx): TSCplx; cdecl;
function nspscFir(var State: TNSPFirState; samp: float): TSCplx; cdecl;
function nspcsFir(var State: TNSPFirState; samp: TSCplx): TSCplx; cdecl;

function nspdFir(var State: TNSPFirState; samp: Double): Double; cdecl;
function nspzFir(var State: TNSPFirState; samp: TDCplx): TDCplx; cdecl;
function nspdzFir(var State: TNSPFirState; samp: double): TDCplx; cdecl;
function nspzdFir(var State: TNSPFirState; samp: TDCplx): TDCplx; cdecl;

function nspwFir(var State: TNSPFirState; samp: Short;
		       sMode: Integer; var sFct: Integer): Short; cdecl;

procedure nspsbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PFloatArray; numIters: Integer); cdecl;
procedure nspcbFir(var State: TNSPFirState; inSamps: PSCplxArray;
		       outSamps: PSCplxArray; numIters: Integer); cdecl;
procedure nspscbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PSCplxArray; numIters: Integer); cdecl;
procedure nspcsbFir(var State: TNSPFirState; inSamps: PSCplxArray;
		       outSamps: PSCplxArray; numIters: Integer); cdecl;

procedure nspdbFir(var State: TNSPFirState; inSamps: PDoubleArray;
		       outSamps: PDoubleArray; numIters: Integer); cdecl;
procedure nspzbFir(var State: TNSPFirState; inSamps: PDCplxArray;
		       outSamps: PDCplxArray; numIters: Integer); cdecl;
procedure nspdzbFir(var State: TNSPFirState; inSamps: PDoubleArray;
		       outSamps: PDCplxArray; numIters: Integer); cdecl;
procedure nspzdbFir(var State: TNSPFirState; inSamps: PDCplxArray;
		       outSamps: PDCplxArray; numIters: Integer); cdecl;

procedure nspwbFir(var State: TNSPFirState; inSamps: PShortArray;
		       outSamps: PShortArray; numIters, sMode: Integer;
		       var sFct: Integer); cdecl;


//-------------------------------------------------------------------------*/
// nsp?FirGetTaps(), nsp?FirSetTaps()                                      */
//                                                                         */
// Utility functions to get and set the FIR taps coefficients              */

procedure nspsFirGetTaps(const State: TNSPFirState; OutTaps: PFloatArray); cdecl;
procedure nspcFirGetTaps(const State: TNSPFirState; OutTaps: PSCplxArray); cdecl;
procedure nspdFirGetTaps(const State: TNSPFirState; OutTaps: PDoubleArray); cdecl;
procedure nspzFirGetTaps(const State: TNSPFirState; OutTaps: PDCplxArray); cdecl;

procedure nspwFirGetTaps(const State: TNSPFirState; OutTaps: PShortArray); cdecl;

procedure nspsFirSetTaps(InTaps: PFloatArray; var State: TNSPFirState); cdecl;
procedure nspcFirSetTaps(InTaps: PSCplxArray; var State: TNSPFirState); cdecl;
procedure nspdFirSetTaps(InTaps: PDoubleArray; var State: TNSPFirState); cdecl;
procedure nspzFirSetTaps(InTaps: PDCplxArray; var State: TNSPFirState); cdecl;

procedure nspwFirSetTaps(InTaps: PShortArray; var State: TNSPFirState); cdecl;


//-------------------------------------------------------------------------*/
// nsp?FirGetDlyl(), nsp?FirSetDlyl()                                      */
//                                                                         */
// Utility functions to get and set the FIR delay line contents            */

procedure nspsFirGetDlyl(const State: TNSPFirState; OutDlyl: PFloatArray); cdecl;
procedure nspcFirGetDlyl(const State: TNSPFirState; OutDlyl: PSCplxArray); cdecl;
procedure nspdFirGetDlyl(const State: TNSPFirState; OutDlyl: PDoubleArray); cdecl;
procedure nspzFirGetDlyl(const State: TNSPFirState; OutDlyl: PDCplxArray); cdecl;

procedure nspwFirGetDlyl(const State: TNSPFirState; OutDlyl: PShortArray); cdecl;

procedure nspsFirSetDlyl(inDlyl: PFloatArray; var State: TNSPFirState); cdecl;
procedure nspcFirSetDlyl(inDlyl: PSCplxArray; var State: TNSPFirState); cdecl;
procedure nspdFirSetDlyl(inDlyl: PDoubleArray; var State: TNSPFirState); cdecl;
procedure nspzFirSetDlyl(inDlyl: PDCplxArray; var State: TNSPFirState); cdecl;

procedure nspwFirSetDlyl(inDlyl: PShortArray; var State: TNSPFirState); cdecl;
{$ENDIF}
{$IFDEF nsp_UsesIir}
//  nspiirl
(*---- User tap type ------------------------------------------------------*)

type
  TNSPIirType = (NSP_IirNull, NSP_IirDefault);

(*---- Implementation tap type of initialization --------------------------*)

  TNSPIirInit = (NSP_IirInit, NSP_IirInitBq);

(*---- IIR filter tap state structure -------------------------------------*)

  TNSPIirTapState = record
    order: Integer;		//*  order/number of biquads        */
    taps: Pointer;		//*  pointer to taps                */
    iirtype: TNSPIirType;	//*  user type                      */
    initype: TNSPIirInit;	//*  type of initialiuzation        */
  end;

(*---- IIR filter delay line state structure ------------------------------*)

  TNSPIirDlyState = record
    dlyl: Pointer;                  (*  pointer to delay line          *)
  end;

(*---- IIR filter state structure -----------------------------------------*)

  TNSPIirState = record
    tapSt: TNSPIirTapState;		(* tap state structure             *)
    dlySt: TNSPIirDlyState;		(* delay line state structure      *)
    init: Integer;			(* initialization flag             *)
  end;

(*-------------------------------------------------------------------------*)
(*         IirlInit, IirlInitBq, IirlInitDlyl                              *)
(*                                                                         *)
(* Perform infinite impulse response filtering.                            *)
(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>IirlInit                                                  *)
(* DESCRIPTION:                                                            *)
(*   Taps initialization of low-level arbitrary order IIR-filter with      *)
(*   transfer function expressed as the ratio of two polinomials of order  *)
(*   "order".                                                              *)
(* PARAMETERS:                                                             *)
(*   iirType  - input, determines the filter structure to use, currently   *)
(*              must be NSP_IirDefault;                                    *)
(*   taps     - input/output, sets the array taps of filter with transfer  *)
(*              function                                                   *)
(*                   B0+B1/z+B2/z/z ... +Border/z**order                   *)
(*              H(z)=------------------------------------                  *)
(*                   A0+A1/z+A2/z/z ... +Aorder/z**order                   *)
(*              in following order:                                        *)
(*                  B0, B1, ... Border,A0, A1, ... Aorder.                 *)
(*              A0 must not be zero;                                       *)
(*   order    - input, sets polinomial order;                              *)
(*   tapStPtr - output, the pointer to the tap state structure.            *)
(*                                                                         *)

procedure nspwIirlInit(IirType: TNSPIirType; Taps: PShortArray; Order: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspsIirlInit(IirType: TNSPIirType; Taps: PFloatArray; Order: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspcIirlInit(IirType: TNSPIirType; Taps: PSCplxArray; Order: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspdIirlInit(IirType: TNSPIirType; Taps: PDoubleArray; Order: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspzIirlInit(IirType: TNSPIirType; Taps: PDCplxArray; Order: Integer; var TapState: TNSPIirTapState); cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>IirlInitBq                                                *)
(* DESCRIPTION:                                                            *)
(*   Low-level IIR-filter iinitialization taps to reference a cascade of   *)
(*   biquads. Transfer function is a product of numQuads biquads.          *)
(* PARAMETERS:                                                             *)
(*   iirType  - input, determines the filter structure to use, currently   *)
(*              must be NSP_IirDefault;                                    *)
(*   taps     - input/output, sets the array taps of filter                *)
(*              with the transfer function                                 *)
(*                   (B10+B11/z+B12/z**2)* ... *(BnQ0+BnQ1/z+BnQ2/z**2)    *)
(*              H(z)=--------------------------------------------------    *)
(*                   (A10+A11/z+A12/z**2)* ... *(AnQ0+AnQ1/z+AnQ2/z**2)    *)
(*              in following order:                                        *)
(*                   B10, B11, B12, A10, A11, A12, ... ,                   *)
(*                   BnQ0, BnQ1, BnQ2, AnQ0, AnQ1, AnQ2.                   *)
(*              All Bi0 and Ai0 must not be zero;                          *)
(*   numQuads - input, sets the number of biduads;                         *)
(*   tapStPtr - output, pointer to the tap state structure.                *)
(*                                                                         *)


procedure nspwIirlInitBq(IirType: TNSPIirType; Taps: PShortArray; NumQuads: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspsIirlInitBq(IirType: TNSPIirType; Taps: PFloatArray; NumQuads: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspcIirlInitBq(IirType: TNSPIirType; Taps: PSCplxArray; NumQuads: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspdIirlInitBq(IirType: TNSPIirType; Taps: PDoubleArray; NumQuads: Integer; var TapState: TNSPIirTapState); cdecl;
procedure nspzIirlInitBq(IirType: TNSPIirType; Taps: PDCplxArray; NumQuads: Integer; var TapState: TNSPIirTapState); cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>IirlInitDlyl                                              *)
(* DESCRIPTION:                                                            *)
(*   Initialization of delay array for low-level IIR-filter.               *)
(* PARAMETERS:                                                             *)
(*   tapStPtr - input, the pointer to the tap state structure,             *)
(*              initializated previously by the function nsp?IirInit or    *)
(*              nsp?IirInitBq;                                             *)
(*   dlyl     - input/output, the delay line array. In this implementation *)
(*              the input array element values are ignored.                *)
(*   dlyStPtr - output, the pointer to the delay line state structure.     *)
(*                                                                         *)

procedure nspwIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PShortArray; var DlyState: TNSPIirDlyState); cdecl;
procedure nspsIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PFloatArray; var DlyState: TNSPIirDlyState); cdecl;
procedure nspcIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PSCplxArray; var DlyState: TNSPIirDlyState); cdecl;
procedure nspdIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PDoubleArray; var DlyState: TNSPIirDlyState); cdecl;
procedure nspzIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PDCplxArray; var DlyState: TNSPIirDlyState); cdecl;

(*-------------------------------------------------------------------------*)
(*         Iirl, bIirl                                                     *)
(*                                                                         *)
(* Filter a signal through a low-level IIR filter.                         *)
(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>Iirl                                                      *)
(*   nsp<sc,cs,dz,zd>Iirl                                                  *)
(* DESCRIPTION:                                                            *)
(*   filters a single sample throuth a low-level IIR filter and returns    *)
(*   the result.                                                           *)
(* PARAMETERS:                                                             *)
(*   tapStPtr - input, pointer to the tap state structure;                 *)
(*   dlyStPtr - input, pointer to the delay line state structure;          *)
(*   smp      - input, the sample value.                                   *)
(* RETURNS:                                                                *)
(*   the filtering result.                                                 *)
(*                                                                         *)

function nspwIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Short;
		ScaleMode: Integer; var ScaleFactor: Integer): Short; cdecl;
function nspsIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Float): Float; cdecl;
function nspcIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; cdecl;
function nspdIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Double): Double; cdecl;
function nspzIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TDCplx): TDCplx; cdecl;

function nspscIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Float): TSCplx; cdecl;
function nspcsIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; cdecl;
function nspdzIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Double): TDCplx; cdecl;
function nspzdIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TDCplx): TDCplx; cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>bIirl                                                     *)
(*   nsp<sc,cs,dz,zd>bIirl                                                 *)
(* DESCRIPTION:                                                            *)
(*   filters an input block of samples throuth a low-level IIR filter and  *)
(*   returns the result in output block.                                   *)
(* PARAMETERS:                                                             *)
(*   tapStPtr - input, the pointer to the tap state structure;             *)
(*   dlyStPtr - input, the pointer to the delay line state structure;      *)
(*   inSamps  - input, the block of samples;                               *)
(*   outSamps - output, the block of filtered samples;                     *)
(*   numIters - input, the size of samples' block.                         *)
(*                                                                         *)

procedure nspwbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PShortArray; outSamps: PShortArray;
			NumIters, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspsbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PFloatArray; outSamps: PFloatArray;
			NumIters: Integer); cdecl;
procedure nspcbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray;
			NumIters: Integer); cdecl;
procedure nspdbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDoubleArray; outSamps: PDoubleArray;
			NumIters: Integer); cdecl;
procedure nspzbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray;
			NumIters: Integer); cdecl;
procedure nspscbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PFloatArray; outSamps: PSCplxArray;
			NumIters: Integer); cdecl;
procedure nspcsbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray;
			NumIters: Integer); cdecl;
procedure nspdzbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDoubleArray; outSamps: PDCplxArray;
			NumIters: Integer); cdecl;
procedure nspzdbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray;
			NumIters: Integer); cdecl;
// nspiirh

//
//  RCS:
//      $Source: nsp30/include/nspiirh.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 11:00:02 $
//
//  Purpose: NSP Infinite Impulse Response Filtering.

(*-------------------------------------------------------------------------*)
(*         IirInit, IirInitBq, IirFree                                     *)
(*                                                                         *)
(* These functions initialize an IIR filter and provide a higher-level     *)
(* interface than the corresponding low-level IIR functions.               *)
(*                                                                         *)

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>IirInit                                                   *)
(*   nsp<sc,cs,dz,zd>IirInit                                               *)
(* DESCRIPTION:                                                            *)
(*   These functions initialize an arbitrary order IIR filter.             *)
(* PARAMETERS:                                                             *)
(*   iirType  - input, describes the filter structure to use, and currently*)
(*              must be NSP_IirDefault;                                    *)
(*   TapVals  - input, the 2*(order+1) length array specifies the filter   *)
(*              coefficients as discussed for the low-level IIR function   *)
(*              nsp?IirlInit());                                           *)
(*   order    - input, sets polinomial order;                              *)
(*   statePtr - output, the pointer to the IIR filter state structure.     *)
(*                                                                         *)

procedure nspwIirInit(IirType: TNSPIirType; TapVals: PShortArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspsIirInit(IirType: TNSPIirType; TapVals: PFloatArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspcIirInit(IirType: TNSPIirType; TapVals: PSCplxArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspdIirInit(IirType: TNSPIirType; TapVals: PDoubleArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspzIirInit(IirType: TNSPIirType; TapVals: PDCplxArray; Order: Integer; var State: TNSPIirState); cdecl;

procedure nspcsIirInit(IirType: TNSPIirType; TapVals: PFloatArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspscIirInit(IirType: TNSPIirType; TapVals: PSCplxArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspzdIirInit(IirType: TNSPIirType; TapVals: PDoubleArray; Order: Integer; var State: TNSPIirState); cdecl;
procedure nspdzIirInit(IirType: TNSPIirType; TapVals: PDCplxArray; Order: Integer; var State: TNSPIirState); cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>IirInitBq                                                 *)
(*   nsp<sc,cs,dz,zd>IirInitBq                                             *)
(* DESCRIPTION:                                                            *)
(*   These functions initialize an IIR filter defined by a cascade of      *)
(*   biquads.                                                              *)
(* PARAMETERS:                                                             *)
(*   iirType  - input, describes the filter structure to use, and currently*)
(*              must be NSP_IirDefault;                                    *)
(*   TapVals  - input, the 6*numQuads length array specifies the filter    *)
(*              coefficients as discussed for the low-level IIR function   *)
(*              nsp?IirlInitBq());                                         *)
(*   numQuads - input, sets the number of biduads;                         *)
(*   tapStPtr - output, pointer to the IIR state structure.                *)
(*                                                                         *)

procedure nspwIirInitBq(IirType: TNSPIirType; TapVals: PShortArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspsIirInitBq(IirType: TNSPIirType; TapVals: PFloatArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspcIirInitBq(IirType: TNSPIirType; TapVals: PSCplxArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspdIirInitBq(IirType: TNSPIirType; TapVals: PDoubleArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspzIirInitBq(IirType: TNSPIirType; TapVals: PDCplxArray; NumQuads: Integer; var State: TNSPIirState); cdecl;

procedure nspcsIirInitBq(IirType: TNSPIirType; TapVals: PFloatArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspscIirInitBq(IirType: TNSPIirType; TapVals: PSCplxArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspzdIirInitBq(IirType: TNSPIirType; TapVals: PDoubleArray; NumQuads: Integer; var State: TNSPIirState); cdecl;
procedure nspdzIirInitBq(IirType: TNSPIirType; TapVals: PDCplxArray; NumQuads: Integer; var State: TNSPIirState); cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nspIirFree                                                            *)
(* DESCRIPTION:                                                            *)
(*   This function must be called after all filtering is done to free      *)
(*   dynamic memory associated with statePtr. After calling this function, *)
(*   statePtr should not be referenced again.                              *)
(* PARAMETERS:                                                             *)
(*   statePtr - pointer to the IIR filter state structure.                 *)
(*                                                                         *)

procedure nspIirFree(var State: TNSPIirState); cdecl;

(*-------------------------------------------------------------------------*)
(*         Iir, bIir                                                       *)
(*                                                                         *)
(* Filter a signal through a IIR filter.                                   *)
(*                                                                         *)

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>Iir                                                       *)
(*   nsp<sc,cs,dz,zd>Iir                                                   *)
(* DESCRIPTION:                                                            *)
(*   These functions filter a single sample samp through an IIR filter and *)
(*   return the result.                                                    *)
(* PARAMETERS:                                                             *)
(*   statePtr - input, pointer to the IIR state structure;                 *)
(*   samp     - input, the sample value.                                   *)
(* RETURNS:                                                                *)
(*   the filtering result.                                                 *)
(*                                                                         *)

function nspsIir(var State: TNSPIirState; Samp: Float): Float; cdecl;
function nspcIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; cdecl;
function nspdIir(var State: TNSPIirState; Samp: Double): Double; cdecl;
function nspzIir(var State: TNSPIirState; Samp: TDCplx): TDCplx; cdecl;
function nspwIir(var State: TNSPIirState; Samp: Short; ScaleMode: Integer; var ScaleFactor: Integer): Short; cdecl;

function nspscIir(var State: TNSPIirState; Samp: Float): TSCplx; cdecl;
function nspcsIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; cdecl;
function nspdzIir(var State: TNSPIirState; Samp: Double): TDCplx; cdecl;
function nspzdIir(var State: TNSPIirState; Samp: TDCplx): TDCplx; cdecl;

(*-------------------------------------------------------------------------*)
(* FUNCTION:                                                               *)
(*   nsp<s,c,d,z>bIir                                                      *)
(*   nsp<sc,cs,dz,zd>bIir                                                  *)
(* DESCRIPTION:                                                            *)
(*   These functions filter a block of numIters samples inSamps through an *)
(*   IIR filter and return the result in outSamps.                         *)
(* PARAMETERS:                                                             *)
(*   statePtr - input, pointer to the IIR state structure;                 *)
(*   inSamps  - input, the pointer to block of numIters samples;           *)
(*   outSamps - output, the pointer to block of numIters filtered samples; *)
(*   numIters - input, the size of samples' block.                         *)
(*                                                                         *)

procedure nspsbIir(var State: TNSPIirState; inSamps: PFloatArray; outSamps: PFloatArray; NumIters: Integer); cdecl;
procedure nspcbIir(var State: TNSPIirState; inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspdbIir(var State: TNSPIirState; inSamps: PDoubleArray; outSamps: PDoubleArray; NumIters: Integer); cdecl;
procedure nspzbIir(var State: TNSPIirState; inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;
procedure nspwbIir(var State: TNSPIirState; inSamps: PShortArray; outSamps: PShortArray; NumIters, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

procedure nspscbIir(var State: TNSPIirState; inSamps: PFloatArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspcsbIir(var State: TNSPIirState; inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspdzbIir(var State: TNSPIirState; inSamps: PDoubleArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;
procedure nspzdbIir(var State: TNSPIirState; inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;
{$ENDIF}
{$IFDEF nsp_UsesLms}
//
//  RCS:
//      $Source: nsp30/include/nsplmsl.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 11:01:00 $
//
//  Purpose: NSP  Adaptive FIR filter that uses the LMS algorithm.

(* --- LMS filter (method) type -------------------------------------------*)

type
  TNSPLmsType = (NSP_LmsNull, NSP_LmsDefault);

(* --- LMS low-level filter state structure -------------------------------*)

  TNSPLmsTapState = record
   // FirLmsTapState;                        (* Base filter state, see FIRL.H *)
    taps: Pointer;
    len: Integer;
    upFactor: Integer;
    upPhase: Integer;
    downFactor: Integer;
    downPhase: Integer;
    isMultiRate: Integer;
    isFilterMode: Integer;
   // end FirLmsTapState
    Leak: Float;			(* Leak value to range taps magn *)
    Step: Float;			(* Step-size to change taps magn *)
    ErrDly: Integer;			(* Delay of error signal         *)
    LmsType: TNSPLmsType;
  end;

(* --- LMS low-level filter delay line ------------------------------------*)

  TNSPLmsDlyState = record
    // FirLmsDlyState;
    Dlyl: Pointer;
    Len: Integer;
    IsFilterMode: Integer;
    // end FirLmsDlyState
    AdaptB: Integer;		(* Is used in delay length definition     *)
  end;

  TNSPWLmsTapState = record
    // FirLmsTapState;               (* Base filter state, see FIRL.H          *)
    Taps: Pointer;
    Len: Integer;
    UpFactor: Integer;
    UpPhase: Integer;
    DownFactor: Integer;
    DownPhase: Integer;
    IsMultiRate: Integer;
    IsFilterMode: Integer;
    // end
    Leak:	Short;		(* Leak value to range taps magn, not used*)
    _leakpad:	Short;		(* for compatible with float leak         *)
    Step:	Short;		(* neg. degree of 2 to decrease err*dly   *)
    _steppad:	Short;		(* for compatible with float step         *)
    ErrDly: Integer;		(* Delay of error signal                  *)
    LmsType: TNSPLmsType;
  end;

  TNSPWLmsDlyState = TNSPLmsDlyState;

(* ------------------------------------------------------------------------*)
(* LmslInit, LmslInitMr, LmslInitDlyl                                      *)
(*                                                                         *)
(* Initialize  an adaptive  FIR filter  that uses  the least  mean-square  *)
(* (LMS) algorithm.                                                        *)


(*--- Single-rate init ----------------------------------------------------*)

procedure nspsLmslInit(LmsType: TNSPLmsType; Taps: PFloatArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspdLmslInit(LmsType: TNSPLmsType; Taps: PDoubleArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspcLmslInit(LmsType: TNSPLmsType; Taps: PSCplxArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspzLmslInit(LmsType: TNSPLmsType; Taps: PDCplxArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); cdecl;

(*--- Multi-rate init -----------------------------------------------------*)

procedure nspsLmslInitMr(LmsType: TNSPLmsType; Taps: PFloatArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspdLmslInitMr(LmsType: TNSPLmsType; Taps: PDoubleArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspcLmslInitMr(LmsType: TNSPLmsType; Taps: PSCplxArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); cdecl;

procedure nspzLmslInitMr(LmsType: TNSPLmsType; Taps: PDCplxArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); cdecl;

(*--- Delay line init -----------------------------------------------------*)

procedure nspsLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PFloatArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspdLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PDoubleArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspcLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PSCplxArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspzLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PDCplxArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); cdecl;

(* -------------------------------------------------------------------------*)
(* LmslGetStep, LmslSetStep, LmslGetLeak, LmslSetLeak                       *)
(*                                                                          *)
(* Utility  functions  to  get  and  set  the  leak  and step values of a   *)
(* low-level LMS filter.                                                    *)

function nspsLmslGetStep(const State: TNSPLmsTapState): Float; cdecl;
function nspsLmslGetLeak(const State: TNSPLmsTapState): Float; cdecl;

procedure nspsLmslSetStep(Step: Float; var State: TNSPLmsTapState); cdecl;
procedure nspsLmslSetLeak(Leak: Float; var State: TNSPLmsTapState); cdecl;

(* -------------------------------------------------------------------------*)
(* Lmsl, bLmsl                                                              *)
(*                                                                          *)
(* Filter samples through a low-level LMS filter.                           *)

function nspsLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: Float): Float; cdecl;
function nspcLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: TSCplx): TSCplx; cdecl;
function nspdLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: Double): Double; cdecl;
function nspzLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: TDCplx): TDCplx; cdecl;

function nspscLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp: Float; Err: TSCplx): TSCplx; cdecl;
function nspdzLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp: Double; Err: TDCplx): TDCplx; cdecl;

function nspsbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PFloatArray; Err: Float): Float; cdecl;
function nspcbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PSCplxArray; Err: TSCplx): TSCplx; cdecl;
function nspdbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDoubleArray; Err: Double): Double; cdecl;
function nspzbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDCplxArray; Err: TDCplx): TDCplx; cdecl;

function nspscbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PFloatArray; Err: TSCplx): TSCplx; cdecl;
function nspdzbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDoubleArray; Err: TDCplx): TDCplx; cdecl;

(* ------------------------------------------------------------------------*)
(* LmslNa, bLmslNa                                                         *)
(*                                                                         *)
(* Filter a signal using a LMS filter, but without adapting the filter.    *)

function nspsLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Float): Float; cdecl;
function nspcLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; cdecl;
function nspdLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Double): Double; cdecl;
function nspzLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TDCplx): TDCplx; cdecl;

function nspscLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Float): TSCplx; cdecl;
function nspcsLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; cdecl;
function nspdzLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Double): TDCplx; cdecl;
function nspzdLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TDCplx): TDCplx; cdecl;

procedure nspsbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PFloatArray; outSamps: PFloatArray; NumIters: Integer); cdecl;
procedure nspcbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspdbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDoubleArray; outSamps: PDoubleArray; NumIters: Integer); cdecl;
procedure nspzbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;

procedure nspscbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PFloatArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspcsbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspdzbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDoubleArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;
procedure nspzdbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); cdecl;

procedure nspsLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PFloatArray); cdecl;
procedure nspcLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PSCplxArray); cdecl;
procedure nspdLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PDoubleArray); cdecl;
procedure nspzLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PDCplxArray); cdecl;

procedure nspsLmslSetTaps(InTaps: PFloatArray; var TapState: TNSPLmsTapState); cdecl;
procedure nspcLmslSetTaps(InTaps: PSCplxArray; var TapState: TNSPLmsTapState); cdecl;
procedure nspdLmslSetTaps(InTaps: PDoubleArray; var TapState: TNSPLmsTapState); cdecl;
procedure nspzLmslSetTaps(InTaps: PDCplxArray; var TapState: TNSPLmsTapState); cdecl;

procedure nspsLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PFloatArray); cdecl;
procedure nspcLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PSCplxArray); cdecl;
procedure nspdLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PDoubleArray); cdecl;
procedure nspzLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PDCplxArray); cdecl;

procedure nspsLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PFloatArray; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspcLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PSCplxArray; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspdLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PDoubleArray; var DlyState: TNSPLmsDlyState); cdecl;
procedure nspzLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PDCplxArray; var DlyState: TNSPLmsDlyState); cdecl;

(* New flavors for short data type.
   Notes:
      1. leak is not used
      2. step is negative degree of 2, so
         w[k] += (err*dly[k+errDly]) >> step;
      3. adaptB is not used, use FIR functions instead.
      4. The structure for short data have "flavored" name now.
      5. About scale mode. I think we must use SATURATE mode only
	 for LMS functions, because it is important to save
	 sign of the multiply result in a convergence process.
     13. Convergence of LMS adaptation is very difficult job now.
*)

procedure nspwLmslInit(LmsType: TNSPLmsType; Taps: PShortArray; TapsLen: Integer;
			Step, Leak: Short; ErrDly: Integer; var TapState: TNSPWLmsTapState); cdecl;
procedure nspwLmslInitDlyl(var TapState: TNSPWLmsTapState; Dlyl: PShortArray;
			AdaptB: Integer; var DlyState: TNSPWLmsDlyState); cdecl;
function nspwLmsl(var TapState: TNSPWLmsTapState; var DlyState: TNSPWLmsDlyState; Samp, Err: Short): Short; cdecl;
procedure nspwLmslSetTaps(InTaps: PShortArray; var TapState: TNSPWLmsTapState); cdecl;
procedure nspwLmslSetDlyl(const TapState: TNSPWLmsTapState; inDlyl: PShortArray; var DlyState: TNSPWLmsDlyState); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nsplmsh.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 11:00:40 $
//
//  Purpose: NSP  Adaptive FIR filter that uses the LMS algorithm

type
  TNSPLmsState = record
    TapState: TNSPLmsTapState;
    DlyState: TNSPLmsDlyState;
    Err: record
      case Integer of
	0: (sVal: Float);
	1: (dVal: Double);
	2: (cVal: TSCplx);
	3: (zVal: TDCplx);
    end;
  end;

procedure nspsLmsInit(LmsType: TNSPLmsType;
			TapVals: PFloatArray; TapsLen: Integer;
			DlyVals: PFLoatArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;
procedure nspdLmsInit(LmsType: TNSPLmsType;
			TapVals: PDoubleArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;
procedure nspcLmsInit(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PSCplxArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;
procedure nspzLmsInit(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDCplxArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;
procedure nspscLmsInit(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PFloatArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;
procedure nspdzLmsInit(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); cdecl;

procedure nspsLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PFloatArray; TapsLen: Integer;
			DlyVals: PFLoatArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;
procedure nspdLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDoubleArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;
procedure nspcLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PSCplxArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;
procedure nspzLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDCplxArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;
procedure nspscLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PFloatArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;
procedure nspdzLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); cdecl;

procedure nspLmsFree(var State: TNSPLmsState); cdecl;

function nspsLms(var State: TNSPLmsState; Samp, Err: Float): Float; cdecl;
function nspdLms(var State: TNSPLmsState; Samp, Err: Double): Double; cdecl;
function nspcLms(var State: TNSPLmsState; Samp, Err: TSCplx): TSCplx; cdecl;
function nspzLms(var State: TNSPLmsState; Samp, Err: TDCplx): TDCplx; cdecl;
function nspscLms(var State: TNSPLmsState; Samp: Float; Err: TSCplx): TSCplx; cdecl;
function nspdzLms(var State: TNSPLmsState; Samp: Double; Err: TDCplx): TDCplx; cdecl;

function nspsbLms(var State: TNSPLmsState; inSamps: PFloatArray; Err: Float): Float; cdecl;
function nspdbLms(var State: TNSPLmsState; inSamps: PDoubleArray; Err: Double): Double; cdecl;
function nspcbLms(var State: TNSPLmsState; inSamps: PSCplxArray; Err: TSCplx): TSCplx; cdecl;
function nspzbLms(var State: TNSPLmsState; inSamps: PDCplxArray; Err: TDCplx): TDCplx; cdecl;
function nspscbLms(var State: TNSPLmsState; inSamps: PFloatArray; Err: TSCplx): TSCplx; cdecl;
function nspdzbLms(var State: TNSPLmsState; inSamps: PDoubleArray; Err: TDCplx): TDCplx; cdecl;

procedure nspsLmsGetTaps(const State: TNSPLmsState; OutTaps: PFloatArray); cdecl;
procedure nspdLmsGetTaps(const State: TNSPLmsState; OutTaps: PDoubleArray); cdecl;
procedure nspcLmsGetTaps(const State: TNSPLmsState; OutTaps: PSCplxArray); cdecl;
procedure nspzLmsGetTaps(const State: TNSPLmsState; OutTaps: PDCplxArray); cdecl;

procedure nspsLmsSetTaps(InTaps: PFloatArray; var State: TNSPLmsState); cdecl;
procedure nspdLmsSetTaps(InTaps: PDoubleArray; var State: TNSPLmsState); cdecl;
procedure nspcLmsSetTaps(InTaps: PSCplxArray; var State: TNSPLmsState); cdecl;
procedure nspzLmsSetTaps(InTaps: PDCplxArray; var State: TNSPLmsState); cdecl;

procedure nspsLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PFloatArray); cdecl;
procedure nspdLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PDoubleArray); cdecl;
procedure nspcLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PSCplxArray); cdecl;
procedure nspzLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PDCplxArray); cdecl;

procedure nspsLmsSetDlyl(inDlyl: PFloatArray; var State: TNSPLmsState); cdecl;
procedure nspdLmsSetDlyl(inDlyl: PDoubleArray; var State: TNSPLmsState); cdecl;
procedure nspcLmsSetDlyl(inDlyl: PSCplxArray; var State: TNSPLmsState); cdecl;
procedure nspzLmsSetDlyl(inDlyl: PDCplxArray; var State: TNSPLmsState); cdecl;

function nspsLmsGetStep(const State: TNSPLmsState): Float; cdecl;
function nspsLmsGetLeak(const State: TNSPLmsState): Float; cdecl;

procedure nspsLmsSetStep(Step: Float; var State: TNSPLmsState); cdecl;
procedure nspsLmsSetLeak(Leak: Float; var State: TNSPLmsState); cdecl;

function nspsLmsDes(var State: TNSPLmsState; Samp: Float; Des: Float): Float; cdecl;
function nspdLmsDes(var State: TNSPLmsState; Samp: Double; Des: Double): Double; cdecl;
function nspcLmsDes(var State: TNSPLmsState; Samp: TSCplx; Des: TSCplx): TSCplx; cdecl;
function nspzLmsDes(var State: TNSPLmsState; Samp: TDCplx; Des: TDCplx): TDCplx; cdecl;
function nspscLmsDes(var State: TNSPLmsState; Samp: Float; Des: TSCplx): TSCplx; cdecl;
function nspdzLmsDes(var State: TNSPLmsState; Samp: Double; Des: TDCplx): TDCplx; cdecl;

procedure nspsbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TFloatArray;
			outSamps: PFloatArray; NumIters: Integer); cdecl;
procedure nspdbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TDoubleArray;
			outSamps: PDoubleArray; NumIters: Integer); cdecl;
procedure nspcbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TSCplxArray;
			outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspzbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TDCplxArray;
			outSamps: PDCplxArray; NumIters: Integer); cdecl;
procedure nspscbLmsDes(var State: TNSPLmsState;
			inSamps: PFloatArray;
			DesSamps: PSCplxArray;
			outSamps: PSCplxArray; NumIters: Integer); cdecl;
procedure nspdzbLmsDes(var State: TNSPLmsState;
			inSamps: PDoubleArray;
			DesSamps: PDCplxArray;
			outSamps: PDCplxArray; NumIters: Integer); cdecl;

function nspsLmsGetErrVal(const State: TNSPLmsState): Float; cdecl;
function nspdLmsGetErrVal(const State: TNSPLmsState): Double; cdecl;
function nspcLmsGetErrVal(const State: TNSPLmsState): TSCplx; cdecl;
function nspzLmsGetErrVal(const State: TNSPLmsState): TDCplx; cdecl;

procedure nspsLmsSetErrVal(Err: Float; var State: TNSPLmsState); cdecl;
procedure nspdLmsSetErrVal(Err: Double; var State: TNSPLmsState); cdecl;
procedure nspcLmsSetErrVal(Err: TSCplx; var State: TNSPLmsState); cdecl;
procedure nspzLmsSetErrVal(Err: TDCplx; var State: TNSPLmsState); cdecl;
{$ENDIF}
{$IFDEF nsp_UsesMisc}
//                                                                         */
//            Miscellaneous Signal Processing                              */
//                                                                         */
//      BitRev, CalcBitRevTbl, GetBitRevTbl, FreeBitRevTbls                */
//                                                                         */
// Obtain bit-reversed numbers, indices, and indexing tables.              */
//                                                                         */

function nspBitRev(A, Order: Integer): Integer; cdecl;
function nspGetBitRevTbl(Order: Integer): PIntArray; cdecl;
procedure nspCalcBitRevTbl(Tbl: PIntArray; Order: Integer); cdecl;
procedure nspFreeBitRevTbls; cdecl;
//                                                                         */
//        bBitRev1, bBitRev2                                               */
//                                                                         */
// Permute a vector into binary bit-reversed order.                        */
//                                                                         */
procedure nspwbBitRev1(Vector: PShortArray; Order: Integer); cdecl;
procedure nspvbBitRev1(Vector: PWCplxArray; Order: Integer); cdecl;
procedure nspsbBitRev1(Vector: PFloatArray; Order: Integer); cdecl;
procedure nspcbBitRev1(Vector: PSCplxArray; Order: Integer); cdecl;
procedure nspdbBitRev1(Vector: PDoubleArray; Order: Integer); cdecl;
procedure nspzbBitRev1(Vector: PDCplxArray; Order: Integer); cdecl;

procedure nspwbBitRev2(Src: PShortArray;  Dst: PShortArray;  Order: Integer); cdecl;
procedure nspvbBitRev2(Src: PWCplxArray;  Dst: PWCplxArray;  Order: Integer); cdecl;
procedure nspsbBitRev2(Src: PFloatArray;  Dst: PFloatArray;  Order: Integer); cdecl;
procedure nspcbBitRev2(Src: PSCplxArray;  Dst: PSCplxArray;  Order: Integer); cdecl;
procedure nspdbBitRev2(Src: PDoubleArray; Dst: PDoubleArray; Order: Integer); cdecl;
procedure nspzbBitRev2(Src: PDCplxArray;  Dst: PDCplxArray;  Order: Integer); cdecl;
//                                                                         */
//        GetFftTwdTbl, GetDftTwdTbl, FreeTwdTbls                          */
//                                                                         */
// Compute and store twiddle factors for FFT and DFT computations.         */
procedure nspvCalcFftTwdTbl(var Tbl: TWCplx; order: Integer); cdecl;
procedure nspcCalcDftTwdTbl(var Tbl: TSCplx; length: Integer); cdecl;
procedure nspcCalcFftTwdTbl(var Tbl: TSCplx; order: Integer); cdecl;
procedure nspzCalcDftTwdTbl(var Tbl: TDCplx; length: Integer); cdecl;
procedure nspzCalcFftTwdTbl(var Tbl: TDCplx; order: Integer); cdecl;

function nspvGetFftTwdTbl(Order: Integer):  PWCplxArray; cdecl;
function nspcGetDftTwdTbl(Length: Integer): PSCplxArray; cdecl;
function nspcGetFftTwdTbl(Order: Integer):  PSCplxArray; cdecl;
function nspzGetDftTwdTbl(Length: Integer): PDCplxArray; cdecl;
function nspzGetFftTwdTbl(Order: Integer):  PDCplxArray; cdecl;

procedure nspvFreeTwdTbls; cdecl;
procedure nspcFreeTwdTbls; cdecl;
procedure nspzFreeTwdTbls; cdecl;
{$ENDIF}
{$IFDEF nsp_UsesSampleGen}
(*
   Uniform distribution.
   Algorithm by G.Marsaglia and A.Zaman (Computers in Physics, vol.8,
   #1, 1994, 117) are used to build generator of uniformly distributed
   random numbers.
*)
procedure nspsRandUniInit(Seed, Low, High: Float; var State: TNSPSRandUniState); cdecl;
procedure nspcRandUniInit(Seed, Low, High: Float; var State: TNSPCRandUniState); cdecl;
procedure nspdRandUniInit(Seed, Low, High: Double; var State: TNSPDRandUniState); cdecl;
procedure nspzRandUniInit(Seed, Low, High: Double; var State: TNSPZRandUniState); cdecl;
procedure nspwRandUniInit(Seed: UInt; Low, High: Short; var State: TNSPWRandUniState); cdecl;
procedure nspvRandUniInit(Seed: UInt; Low, High: Short; var State: TNSPVRandUniState); cdecl;

function nspsRandUni(var State: TNSPSRandUniState): Float; cdecl;
function nspcRandUni(var State: TNSPCRandUniState): TSCplx; cdecl;
function nspdRandUni(var State: TNSPDRandUniState): Double; cdecl;
function nspzRandUni(var State: TNSPZRandUniState): TDCplx; cdecl;
function nspwRandUni(var State: TNSPWRandUniState): Short; cdecl;
function nspvRandUni(var State: TNSPVRandUniState): TWCplx; cdecl;

procedure nspsbRandUni(var State: TNSPSRandUniState; Samps: PFloatArray; SampsLen: Integer); cdecl;
procedure nspcbRandUni(var State: TNSPCRandUniState; Samps: PSCplxArray; SampsLen: Integer); cdecl;
procedure nspdbRandUni(var State: TNSPDRandUniState; Samps: PDoubleArray; SampsLen: Integer); cdecl;
procedure nspzbRandUni(var State: TNSPZRandUniState; Samps: PDCplxArray; SampsLen: Integer); cdecl;
procedure nspwbRandUni(var State: TNSPWRandUniState; Samps: PShortArray; SampsLen: Integer); cdecl;
procedure nspvbRandUni(var State: TNSPVRandUniState; Samps: PWCplxArray; SampsLen: Integer); cdecl;
(*
   Normal distribution.
   Algorithm by G.Box and M.Muller and by G.Marsaglia (Reference:
   D.Knuth. The Art of Computer Programming. vol.2, 1969) are used
   to build generator of normally distributed random numbers.
*)
procedure nspsRandGausInit(Seed, Mean, StDev: Float; var State: TNSPSRandGausState); cdecl;
procedure nspcRandGausInit(Seed, Mean, StDev: Float; var State: TNSPCRandGausState); cdecl;
procedure nspdRandGausInit(Seed, Mean, StDev: Double; var State: TNSPDRandGausState); cdecl;
procedure nspzRandGausInit(Seed, Mean, StDev: Double; var State: TNSPZRandGausState); cdecl;
procedure nspwRandGausInit(Seed, Mean, StDev: Short; var State: TNSPWRandGausState); cdecl;
procedure nspvRandGausInit(Seed, Mean, StDev: Short; var State: TNSPVRandGausState); cdecl;

function nspsRandGaus(var State: TNSPSRandGausState): Float; cdecl;
function nspcRandGaus(var State: TNSPCRandGausState): TSCplx; cdecl;
function nspdRandGaus(var State: TNSPDRandGausState): Double; cdecl;
function nspzRandGaus(var State: TNSPZRandGausState): TDCplx; cdecl;
function nspwRandGaus(var State: TNSPWRandGausState): Short; cdecl;
function nspvRandGaus(var State: TNSPVRandGausState): TWCplx; cdecl;

procedure nspsbRandGaus(var State: TNSPSRandGausState; Samps: PFloatArray; SampsLen: Integer); cdecl;
procedure nspcbRandGaus(var State: TNSPCRandGausState; Samps: PSCplxArray; SampsLen: Integer); cdecl;
procedure nspdbRandGaus(var State: TNSPDRandGausState; Samps: PDoubleArray; SampsLen: Integer); cdecl;
procedure nspzbRandGaus(var State: TNSPZRandGausState; Samps: PDCplxArray; SampsLen: Integer); cdecl;
procedure nspwbRandGaus(var State: TNSPWRandGausState; Samps: PShortArray; SampsLen: Integer); cdecl;
procedure nspvbRandGaus(var State: TNSPVRandGausState; Samps: PWCplxArray; SampsLen: Integer); cdecl;
//
//  Purpose: NSP Tone Generator. Declarator
//
//--- Initialization ------------------------------------------------------*/
procedure nspwToneInit(rfreq, phase: Float; mag: Short; var State: TNSPWToneState); cdecl;
procedure nspvToneInit(rfreq, phase: Float; mag: Short; var State: TNSPVToneState); cdecl;
procedure nspsToneInit(rfreq, phase, mag: Float; var State: TNSPSToneState); cdecl;
procedure nspcToneInit(rfreq, phase, mag: Float; var State: TNSPCToneState); cdecl;
procedure nspdToneInit(rfreq, phase, mag: Double; var State: TNSPDToneState); cdecl;
procedure nspzToneInit(rfreq, phase, mag: Double; var State: TNSPZToneState); cdecl;

//--- Dot product tone functions ------------------------------------------*/

function nspwTone(var State: TNSPWToneState): Short; cdecl;
function nspvTone(var State: TNSPVToneState): TWCplx; cdecl;
function nspsTone(var State: TNSPSToneState): Float; cdecl;
function nspcTone(var State: TNSPCToneState): TSCplx; cdecl;
function nspdTone(var State: TNSPDToneState): Double; cdecl;
function nspzTone(var State: TNSPZToneState): TDCplx; cdecl;

//--- Array product tone functions ----------------------------------------*/

procedure nspwbTone(var State: TNSPWToneState; Samps: PShortArray; SampsLen: Integer); cdecl;
procedure nspvbTone(var State: TNSPVToneState; Samps: PWCplxArray; SampsLen: Integer); cdecl;
procedure nspsbTone(var State: TNSPSToneState; Samps: PFloatArray; SampsLen: Integer); cdecl;
procedure nspcbTone(var State: TNSPCToneState; Samps: PSCplxArray; SampsLen: Integer); cdecl;
procedure nspdbTone(var State: TNSPDToneState; Samps: PDoubleArray; SampsLen: Integer); cdecl;
procedure nspzbTone(var State: TNSPZToneState; Samps: PDCplxArray; SampsLen: Integer); cdecl;
//
//  Purpose: Generating of signals with triangle wave form.
//           Provides samples of a triangle of arbitrary frequency,
//           phase, magnitude, and asymmetry.
//
//           0 =< phase < 2PI,  -PI < asym < PI,
//           mag > 0, 0 =< rfrec < 0.5
//
//           asym = 0       => Triangle is symmetric (old version)
//           asym = -PI+eps => Triangle with sheer back (0<eps)
//           asym =  PI-eps => Triangle with sheer front-fore
//
//           Like (cos) tone functions the triangle has period equal 2PI
//           Triangle phase is given in radians.
//
//--- Triangle initialization ---------------------------------------------*/
procedure nspwTrnglInit(rfrq, phase: Float; Mag: Short; Asym: Float; var State: TNSPWTrnglState); cdecl;
procedure nspvTrnglInit(rfrq, phase: Float; Mag: Short; Asym: Float; var State: TNSPVTrnglState); cdecl;
procedure nspsTrnglInit(rfrq, phase, Mag, Asym: Float; var State: TNSPSTrnglState); cdecl;
procedure nspcTrnglInit(rfrq, phase, Mag, Asym: Float; var State: TNSPCTrnglState); cdecl;
procedure nspdTrnglInit(rfrq, phase, Mag, Asym: Double; var State: TNSPDTrnglState); cdecl;
procedure nspzTrnglInit(rfrq, phase, Mag, Asym: Double; var State: TNSPZTrnglState); cdecl;

//--- Single-Sample-Generating triangle functions -------------------------*/

function nspwTrngl(var State: TNSPWTrnglState): Short; cdecl;
function nspvTrngl(var State: TNSPVTrnglState): TWCplx; cdecl;
function nspsTrngl(var State: TNSPSTrnglState): Float; cdecl;
function nspcTrngl(var State: TNSPCTrnglState): TSCplx; cdecl;
function nspdTrngl(var State: TNSPDTrnglState): Double; cdecl;
function nspzTrngl(var State: TNSPZTrnglState): TDCplx; cdecl;

//--- Block-of-Samples-Generating triangle functions ----------------------*/

procedure nspwbTrngl(var State: TNSPWTrnglState; Samps: PShortArray; SampsLen: Integer); cdecl;
procedure nspvbTrngl(var State: TNSPVTrnglState; Samps: PWCplxArray; SampsLen: Integer); cdecl;
procedure nspsbTrngl(var State: TNSPSTrnglState; Samps: PFloatArray; SampsLen: Integer); cdecl;
procedure nspcbTrngl(var State: TNSPCTrnglState; Samps: PSCplxArray; SampsLen: Integer); cdecl;
procedure nspdbTrngl(var State: TNSPDTrnglState; Samps: PDoubleArray; SampsLen: Integer); cdecl;
procedure nspzbTrngl(var State: TNSPZTrnglState; Samps: PDCplxArray; SampsLen: Integer); cdecl;
{$ENDIF}
{$IFDEF nsp_UsesTransform}
procedure nspvDft(
	inSamps: PWCplxArray;
	outSamps: PWCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspcDft(
	inSamps: PSCplxArray;
	outSamps: PSCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspzDft(
	inSamps: PDCplxArray;
	outSamps: PDCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspvFft(
	Samps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspcFft(
	Samps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspzFft(
	Samps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspvFftNip(
	inSamps: PWCplxArray;
	outSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspcFftNip(
	inSamps: PSCplxArray;
	outSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspzFftNip(
	inSamps: PDCplxArray;
	outSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspvrFft(
	reSamps: PShortArray;
	imSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspcrFft(
	reSamps: PFloatArray;
	imSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspzrFft(
	reSamps: PDoubleArray;
	imSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspvrFftNip(
	reInSamps: PShortArray;
	imInSamps: PShortArray;
	reOutSamps: PShortArray;
	imOutSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspcrFftNip(
	reInSamps: PFloatArray;
	imInSamps: PFloatArray;
	reOutSamps: PFloatArray;
	imOutSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspzrFftNip(
	reInSamps: PDoubleArray;
	imInSamps: PDoubleArray;
	reOutSamps: PDoubleArray;
	imOutSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwRealFftl(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsRealFftl(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdRealFftl(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwRealFftlNip(
	inSamps: PShortArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsRealFftlNip(
	inSamps: PFloatArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdRealFftlNip(
	inSamps: PDoubleArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcsFftl(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcsFftl(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcsFftl(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcsFftlNip(
	inSamps: PShortArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcsFftlNip(
	inSamps: PFloatArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcsFftlNip(
	inSamps: PDoubleArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwMpyRCPack2(
	Src: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsMpyRCPack2(
	Src: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); cdecl;
procedure nspdMpyRCPack2(
	Src: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); cdecl;
procedure nspwMpyRCPack3(
	SrcA: PShortArray;
	SrcB: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsMpyRCPack3(
	SrcA: PFloatArray;
	SrcB: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); cdecl;
procedure nspdMpyRCPack3(
	SrcA: PDoubleArray;
	SrcB: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); cdecl;
procedure nspwMpyRCPerm2(
	Src: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsMpyRCPerm2(
	Src: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); cdecl;
procedure nspdMpyRCPerm2(
	Src: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); cdecl;
procedure nspwMpyRCPerm3(
	SrcA: PShortArray;
	SrcB: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsMpyRCPerm3(
	SrcA: PFloatArray;
	SrcB: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); cdecl;
procedure nspdMpyRCPerm3(
	SrcA: PDoubleArray;
	SrcB: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); cdecl;
procedure nspwRealFft(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsRealFft(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdRealFft(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwRealFftNip(
	inSamps: PShortArray;
	outSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsRealFftNip(
	inSamps: PFloatArray;
	outSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdRealFftNip(
	inSamps: PDoubleArray;
	outSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcsFft(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcsFft(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcsFft(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcsFftNip(
	inSamps: PWCplxArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcsFftNip(
	inSamps: PSCplxArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcsFftNip(
	inSamps: PDCplxArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwReal2Fft(
	xSamps: PShortArray;
	ySamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsReal2Fft(
	xSamps: PFloatArray;
	ySamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdReal2Fft(
	xSamps: PDoubleArray;
	ySamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwReal2FftNip(
	xInSamps: PShortArray;	xOutSamps: PWCplxArray;
	yInSamps: PShortArray;	yOutSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsReal2FftNip(
	xInSamps: PFloatArray;	xOutSamps: PSCplxArray;
	yInSamps: PFloatArray;	yOutSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdReal2FftNip(
	xInSamps: PDoubleArray; xOutSamps: PDCplxArray;
	yInSamps: PDoubleArray; yOutSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcs2Fft(
	xSamps: PShortArray;
	ySamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcs2Fft(
	xSamps: PFloatArray;
	ySamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcs2Fft(
	xSamps: PDoubleArray;
	ySamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspwCcs2FftNip(
	xInSamps: PWCplxArray; xOutSamps: PShortArray;
	yInSamps: PWCplxArray; yOutSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); cdecl;
procedure nspsCcs2FftNip(
	xInSamps: PSCplxArray; xOutSamps: PFloatArray;
	yInSamps: PSCplxArray; yOutSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
procedure nspdCcs2FftNip(
	xInSamps: PDCplxArray; xOutSamps: PDoubleArray;
	yInSamps: PDCplxArray; yOutSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); cdecl;
//        GoertzInit                                                       */
// Initializes the coefficients and zeros the delay line.                  */
procedure nspsGoertzInit(Freq: Float; var State: TNspSGoertzState); cdecl;
procedure nspcGoertzInit(Freq: Float; var State: TNspCGoertzState); cdecl;
procedure nspwGoertzInit(Freq: Float; var State: TNspWGoertzState); cdecl;
procedure nspvGoertzInit(Freq: Float; var State: TNspVGoertzState); cdecl;
procedure nspdGoertzInit(Freq: Double; var State: TNspDGoertzState); cdecl;
procedure nspzGoertzInit(Freq: Double; var State: TNspZGoertzState); cdecl;
//        GoertzReset                                                      */
// Zeros the delay line.                                                   */
procedure nspsGoertzReset(var State: TNspSGoertzState); cdecl;
procedure nspcGoertzReset(var State: TNspCGoertzState); cdecl;
procedure nspwGoertzReset(var State: TNspWGoertzState); cdecl;
procedure nspvGoertzReset(var State: TNspVGoertzState); cdecl;
procedure nspdGoertzReset(var State: TNspDGoertzState); cdecl;
procedure nspzGoertzReset(var State: TNspZGoertzState); cdecl;
//        Goertz                                                           */
// Single Frequency DFT (Goertzel algorithm)                               */
function nspsGoertz(var State: TNspSGoertzState; Sample: Float): TSCplx; cdecl;
function nspcGoertz(var State: TNspCGoertzState; Sample: TSCplx): TSCplx; cdecl;
function nspwGoertz(var State: TNspWGoertzState; Sample: Short; ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspvGoertz(var State: TNspVGoertzState; Sample: TWCplx; ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspdGoertz(var State: TNspDGoertzState; Sample: Double): TDCplx; cdecl;
function nspzGoertz(var State: TNspZGoertzState; Sample: TDCplx): TDCplx; cdecl;
function nspsbGoertz(var State: TNspSGoertzState; inSamps: PFloatArray; Len: Integer): TSCplx; cdecl;
function nspcbGoertz(var State: TNspCGoertzState; inSamps: PSCplxArray; Len: Integer): TSCplx; cdecl;
function nspwbGoertz(var State: TNspWGoertzState; inSamps: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspvbGoertz(var State: TNspVGoertzState; inSamps: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspdbGoertz(var State: TNspDGoertzState; inSamps: PDoubleArray; Len: Integer): TDCplx; cdecl;
function nspzbGoertz(var State: TNspZGoertzState; inSamps: PDCplxArray; Len: Integer): TDCplx; cdecl;
{$ENDIF}
{$IFDEF nsp_UsesVector}
  // nsparith,
//
//  RCS:
//      $Source: nsp30/include/nsparith.h $
//      $Revision: 1.6 $
//      $Date: 1996/01/11 09:00:00 $
//
//  Purpose: NSP Scalar and Vector Arithmetics

//-------------------------------------------------------------------------*/
//   Miscellaneous Scalar Functions and Vector Functions                   */
//        Complex Add, Sub, Mpy, Div, Conj                                 */
//                                                                         */
//   These functions perform addition, subtraction, multiplication,        */
//   division, and conjugation on complex numbers a and b.                 */

function nspcAdd(a, b: TSCplx): TSCplx; cdecl;
function nspcSub(a, b: TSCplx): TSCplx; cdecl;
function nspcMpy(a, b: TSCplx): TSCplx; cdecl;
function nspcDiv(a, b: TSCplx): TSCplx; cdecl;
function nspcConj(a: TSCplx): TSCplx; cdecl;

//---- Additional Functions -----------------------------------------------*/

function nspcSet(re, im: Float): TSCplx; cdecl;

//-------------------------------------------------------------------------*/
//        Vector Initialization                                            */
//   These functions initialize vectors of length n.                       */

procedure  nspsbZero(Dst: PFloatArray; n: Integer); cdecl;
procedure  nspcbZero(Dst: PSCplxArray; n: Integer); cdecl;

procedure  nspsbSet(Val: Float; Dst: PFloatArray; n: Integer); cdecl;
procedure  nspcbSet(re, im: Float; Dst: PSCplxArray; n: Integer); cdecl;

procedure  nspsbCopy(Src: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure  nspcbCopy(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//        Vector Addition and multiplication                               */
//   These functions perform element-wise arithmetic on vectors of length n*/

// dst[i]=dst[i]+val;                                                      */

procedure nspsbAdd1(Val: Float; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbAdd1(Val: TSCplx; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]+src[i];                                                   */

procedure nspsbAdd2(Src: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbAdd2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=srcA[i]+srcB[i];                                                 */

procedure nspsbAdd3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbAdd3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]-val;                                                      */

procedure nspsbSub1(Val: Float; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbSub1(Val: TSCplx; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]-val[i];                                                   */

procedure nspsbSub2(Val: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbSub2(Val: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=src[i]-val[i];                                                   */

procedure nspsbSub3(Src, Val: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbSub3(Src, Val: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]*val;                                                      */

procedure nspsbMpy1(Val: Float; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbMpy1(Val: TSCplx; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]*src[i];                                                   */

procedure nspsbMpy2(Src: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbMpy2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=srcA[i]*srcB[i];                                                 */

procedure nspsbMpy3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspcbMpy3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//        Complex conjugates of scalars and vectors                        */

procedure  nspcbConj1(Vec: PSCplxArray; n: Integer); cdecl;
procedure  nspcbConj2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;
procedure  nspcbConjFlip2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;
procedure  nspcbConjExtend1(Vec: PSCplxArray; n: Integer); cdecl;
procedure  nspcbConjExtend2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//   Miscellaneous Scalar Functions and Vector Functions                   */
//        Complex Add, Sub, Mpy, Div, Conj                                 */
//   These functions perform addition, subtraction, multiplication,        */
//   division, and conjugation on complex numbers a and b.                 */
//                                                                         */

function nspzAdd(a, b: TDCplx): TDCplx; cdecl;
function nspzSub(a, b: TDCplx): TDCplx; cdecl;
function nspzMpy(a, b: TDCplx): TDCplx; cdecl;
function nspzDiv(a, b: TDCplx): TDCplx; cdecl;
function nspzConj(a: TDCplx): TDCplx; cdecl;

//---- Additional Functions -----------------------------------------------*/

function nspzSet(re, im: Double): TDCplx; cdecl;

//-------------------------------------------------------------------------*/
//        Vector Initialization                                            */
//   These functions initialize vectors of length n.                       */

procedure  nspdbZero(Dst: PDoubleArray; n: Integer); cdecl;
procedure  nspzbZero(Dst: PDCplxArray; n: Integer); cdecl;

procedure  nspdbSet(Val: Double; Dst: PDoubleArray; n: Integer); cdecl;
procedure  nspzbSet(re, im: Double; Dst: PDCplxArray; n: Integer); cdecl;

procedure  nspdbCopy(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); cdecl;
procedure  nspzbCopy(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//        Vector Addition and multiplication                               */
//   These functions perform element-wise arithmetic on vectors of length n*/

// dst[i]=dst[i]+val;                                                      */

procedure nspdbAdd1(Val: Double; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbAdd1(Val: TDCplx; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]+src[i];                                                   */

procedure nspdbAdd2(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbAdd2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=srcA[i]+srcB[i];                                                 */

procedure nspdbAdd3(SrcA, SrcB: PDoubleArray; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbAdd3(SrcA, SrcB: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]-val;                                                      */

procedure nspdbSub1(Val: Double; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbSub1(Val: TDCplx; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]-val[i];                                                   */

procedure nspdbSub2(Val: PDoubleArray; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbSub2(Val: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=src[i]-val[i];                                                   */

procedure nspdbSub3(Src, Val: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspzbSub3(Src, Val: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]*val;                                                      */

procedure nspdbMpy1(Val: Double; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbMpy1(Val: TDCplx; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=dst[i]*src[i];                                                   */

procedure nspdbMpy2(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); cdecl;
procedure nspzbMpy2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

// dst[i]=srcA[i]*srcB[i];                                                 */

procedure nspdbMpy3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); cdecl;
procedure nspzbMpy3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//                                                                         */
//        Complex conjugates of scalars and vectors                        */
//                                                                         */

procedure  nspzbConj1(Vec: PDCplxArray; n: Integer); cdecl;
procedure  nspzbConj2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;
procedure  nspzbConjFlip2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;
procedure  nspzbConjExtend1(Vec: PDCplxArray; n: Integer); cdecl;
procedure  nspzbConjExtend2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//   Miscellaneous Scalar Functions and Vector Functions                   */
//        Complex Add, Sub, Mpy, Div, Conj                                 */
//   These functions perform addition, subtraction, multiplication,        */
//   division, and conjugation on complex numbers a and b.                 */
//                                                                         */

function nspvAdd(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; cdecl;
function nspvSub(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; cdecl;
function nspvMpy(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; cdecl;
function nspvDiv(a, b: TWCplx): TWCplx; cdecl;
function nspvConj(a: TWCplx): TWCplx; cdecl;

//---- Additional Functions -----------------------------------------------*/

function nspvSet(re, im: Short): TWCplx; cdecl;

//-------------------------------------------------------------------------*/
//        Vector Initialization                                            */
//   These functions initialize vectors of length n.                       */

procedure  nspwbZero(Dst: PShortArray; n: Integer); cdecl;
procedure  nspvbZero(Dst: PWCplxArray; n: Integer); cdecl;

procedure  nspwbSet(Val: Short; Dst: PShortArray; n: Integer); cdecl;
procedure  nspvbSet(re, im: Short; Dst: PWCplxArray; n: Integer); cdecl;

procedure  nspwbCopy(Src: PShortArray; Dst: PShortArray; n: Integer); cdecl;
procedure  nspvbCopy(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); cdecl;

//-------------------------------------------------------------------------*/
//        Vector Addition and multiplication                               */
//   These functions perform element-wise arithmetic on vectors of length n*/

procedure nspwbAdd1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbAdd1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

procedure nspwbAdd2(Src: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbAdd2(Src: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

procedure nspwbAdd3(SrcA, SrcB: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbAdd3(SrcA, SrcB: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=dst[i]-val;                                                      */

procedure nspwbSub1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbSub1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=dst[i]-val[i];                                                   */

procedure nspwbSub2(Val: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbSub2(Val: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=src[i]-val[i];                                                   */

procedure nspwbSub3(Src, Val: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbSub3(Src, Val: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=dst[i]*val;                                                      */

procedure nspwbMpy1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbMpy1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=dst[i]*src[i];                                                   */

procedure nspwbMpy2(Src: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbMpy2(Src: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

// dst[i]=srcA[i]*srcB[i];                                                 */

procedure nspwbMpy3(SrcA, SrcB: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbMpy3(SrcA, SrcB: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

//-------------------------------------------------------------------------*/
//        Complex conjugates of vectors                                    */

procedure  nspvbConj1(Vec: PWCplxArray; n: Integer); cdecl;
procedure  nspvbConj2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); cdecl;
procedure  nspvbConjFlip2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); cdecl;
procedure  nspvbConjExtend1(Vec: PWCplxArray; n: Integer); cdecl;
procedure  nspvbConjExtend2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nspcorr.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 10:57:46 $
//
//  Purpose: NSP Correlation

type
  TNSPAutoCorrType = (NSP_Normal, NSP_Biased, NSP_UnBias);

// ------------------------------------------------------------------------*/
//        AutoCorrelation                                                  */
//                                                                         */
{
  Normal:
  ~~~~~~~
  dst[n] = S ~src[k] * src[k+n],  0 <= n < nLags
				  0 <= k < len

  Biased:
  ~~~~~~~
  dst[n] = K* S ~src[k] * src[k+n],  0 <= n < nLags
                                     0 <= k < len
				     K = 1/len
  UnBiased:
  ~~~~~~~~~
  dst[n] = K* S ~src[k] * src[k+n],  0 <= n < nLags
				     0 <= k < len
				     K = 1/(len-n)

					     | src[k], 0<=k<len
				   src(k) = <
					     | 0, otherwise
}

procedure nspsAutoCorr(Src: PFloatArray; Len: Integer; Dst: PFloatArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); cdecl;
procedure nspcAutoCorr(Src: PSCplxArray; Len: Integer; Dst: PSCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); cdecl;
procedure nspdAutoCorr(Src: PDoubleArray; Len: Integer; Dst: PDoubleArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); cdecl;
procedure nspzAutoCorr(Src: PDCplxArray; Len: Integer; Dst: PDCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); cdecl;
procedure nspwAutoCorr(Src: PShortArray; Len: Integer; Dst: PShortArray; nLags: Integer;
				 corrType: TNSPAutoCorrType;
				 scaleMode: Integer; var Scale: Integer); cdecl;
procedure nspvAutoCorr(Src: PWCplxArray; Len: Integer; Dst: PWCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType;
				 scaleMode: Integer; var Scale: Integer); cdecl;

// ------------------------------------------------------------------------*/
//        CrossCorralation                                                 */
{
  dst[n] = S ~srcA[k] * srcB[k+n+loLag],  0 <= n <= hiLag-loLag
					  0 <= k < lenA

						 | srcB[k], 0<=k<lenB
				       srcB(k) = <
						 | 0, otherwise
  Number of result elements is hiLag-loLag+1.
  Estimate at lag of loLag in dst[0].
  Estimate at lag of hiLag in dst[hiLag-loLag].
}

procedure nspsCrossCorr(SrcA: PFloatArray; lenA: Integer;
			SrcB: PFloatArray; lenB: Integer; Dst: PFloatArray;
			loLag, hiLag: Integer); cdecl;
procedure nspcCrossCorr(SrcA: PSCplxArray; lenA: Integer;
			SrcB: PSCplxArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure nspscCrossCorr(SrcA: PFloatArray; lenA: Integer;
			SrcB: PSCplxArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure nspcsCrossCorr(SrcA: PSCplxArray; lenA: Integer;
			SrcB: PFloatArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure  nspdCrossCorr(SrcA: PDoubleArray; lenA: Integer;
			SrcB: PDoubleArray; lenB: Integer; Dst: PDoubleArray;
			loLag, hiLag: Integer); cdecl;
procedure  nspzCrossCorr(SrcA: PDCplxArray; lenA: Integer;
			SrcB: PDCplxArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure nspdzCrossCorr(SrcA: PDoubleArray; lenA: Integer;
			SrcB: PDCplxArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure nspzdCrossCorr(SrcA: PDCplxArray; lenA: Integer;
			SrcB: PDoubleArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); cdecl;
procedure nspwCrossCorr(SrcA: PShortArray; lenA: Integer;
			SrcB: PShortArray; lenB: Integer; Dst: PShortArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); cdecl;
procedure nspvCrossCorr(SrcA: PWCplxArray; lenA: Integer;
			SrcB: PWCplxArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); cdecl;
procedure nspwvCrossCorr(SrcA: PShortArray; lenA: Integer;
			SrcB: PWCplxArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); cdecl;
procedure nspvwCrossCorr(SrcA: PWCplxArray; lenA: Integer;
			SrcB: PShortArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nsplaw.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 11:00:22 $
//
//  Purpose :  Convert samples from 8-bit A-law and Mu-law encoded format
//             to linear, or vice-versa.
//  Contents:
//             nsp?bLinToALaw  -
//                     Convert linear PCM samples to 8-bit A-law format.
//             nsp?bALawToLin  -
//                     Convert samples from 8-bit A-law encoded format
//                     to linear PCM.
//             nsp?bLinToMuLaw  -
//                     Convert linear PCM samples to 8-bit Mu-law format.
//             nsp?bMuLawToLin  -
//                     Convert samples from 8-bit Mu-law encoded format
//                     to linear PCM.
//             nspMuLawToALaw  -
//                     Converts samples from Mu-law encoded format
//                     to A-law encoded format.
//             nspALawToMuLaw  -
//                     Converts samples from A-law encoded format
//                     to Mu-law encoded format.

procedure nspwbLinToALaw(src: PShortArray;  dst: PByteArray; len: Integer); cdecl;
procedure nspibLinToALaw(src: PLongArray;   dst: PByteArray; len: Integer); cdecl;
procedure nspsbLinToALaw(src: PFloatArray;  dst: PByteArray; len: Integer); cdecl;
procedure nspdbLinToALaw(src: PDoubleArray; dst: PByteArray; len: Integer); cdecl;

procedure nspwbALawToLin(src: PByteArray; dst: PShortArray;  len: Integer); cdecl;
procedure nspibALawToLin(src: PByteArray; dst: PLongArray;   len: Integer); cdecl;
procedure nspsbALawToLin(src: PByteArray; dst: PFloatArray;  len: Integer); cdecl;
procedure nspdbALawToLin(src: PByteArray; dst: PDoubleArray; len: Integer); cdecl;

procedure nspwbLinToMuLaw(src: PShortArray;  dst: PByteArray; len: Integer); cdecl;
procedure nspibLinToMuLaw(src: PLongArray;   dst: PByteArray; len: Integer); cdecl;
procedure nspsbLinToMuLaw(src: PFloatArray;  dst: PByteArray; len: Integer); cdecl;
procedure nspdbLinToMuLaw(src: PDoubleArray; dst: PByteArray; len: Integer); cdecl;

procedure nspwbMuLawToLin(src: PByteArray; dst: PShortArray;  len: Integer); cdecl;
procedure nspibMuLawToLin(src: PByteArray; dst: PLongArray;   len: Integer); cdecl;
procedure nspsbMuLawToLin(src: PByteArray; dst: PFloatArray;  len: Integer); cdecl;
procedure nspdbMuLawToLin(src: PByteArray; dst: PDoubleArray; len: Integer); cdecl;

procedure nspMuLawToALaw(src, dst: PByteArray; len: Integer); cdecl;
procedure nspALawToMuLaw(src, dst: PByteArray; len: Integer); cdecl;
//
//  RCS:
//      $Source: nsp30/include/nsplnexp.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/07 11:01:10 $
//
//  Purpose : Compute the natural log of vector elements
//            Compute e to the power of vector elements
//
//  Contents:
//           nsp?bLn1   -
//                      Computes the natural log of each element
//                      in the array in-place
//           nsp?bLn2   -
//                      Computes the natural log of each element
//                      in the array and stores the results
//                      in the another array
//           nsp?bExp1  -
//                      Computes e to the power of each element
//                      in the array in-place
//           nsp?bExp2  -
//                      Computes e to the power of each element
//                      in the array and stores the results
//                      in the another array
//

procedure  nspsbLn1 (vec:	PFloatArray;	len: Integer); cdecl;
procedure  nspsbLn2 (src, dst:	PFloatArray;	len: Integer); cdecl;
procedure  nspsbExp1(vec:	PFloatArray;	len: Integer); cdecl;
procedure  nspsbExp2(src, dst:	PFloatArray;	len: Integer); cdecl;

procedure  nspdbLn1 (vec:	PDoubleArray;	len: Integer); cdecl;
procedure  nspdbLn2 (src, dst:	PDoubleArray;	len: Integer); cdecl;
procedure  nspdbExp1(vec:	PDoubleArray;	len: Integer); cdecl;
procedure  nspdbExp2(src, dst:	PDoubleArray;	len: Integer); cdecl;

procedure  nspwbLn1 (vec:	PShortArray;	len: Integer); cdecl;
procedure  nspwbLn2 (src, dst:	PShortArray;	len: Integer); cdecl;
procedure  nspwbExp1(vec:	PShortArray;	len: Integer;
		scaleMode: Integer; var scaleFactor: Integer); cdecl;
procedure  nspwbExp2(src, dst:	PShortArray;	len: Integer;
		scaleMode: Integer; var scaleFactor: Integer); cdecl;

//
//  RCS:
//      $Source: nsp30/include/nspsampl.h $
//      $Revision: 1.4 $
//      $Date: 1995/08/07 11:01:48 $
//
//  Purpose: NSP sample

// ------------------------------------------------------------------------*/
//        UpSample                                                         */
//                                                                         */
// Up-sample  a  signal, conceptually increasing  its  sampling rate by an */
// integer factor.                                                         */

procedure nspsUpSample(Factor, Phase: Integer; Src: PFloatArray; SrcLen: Integer; Dst: PFloatArray); cdecl;
procedure nspcUpSample(Factor, Phase: Integer; Src: PSCplxArray; SrcLen: Integer; Dst: PSCplxArray); cdecl;
procedure nspdUpSample(Factor, Phase: Integer; Src: PDoubleArray; SrcLen: Integer; Dst: PDoubleArray); cdecl;
procedure nspzUpSample(Factor, Phase: Integer; Src: PDCplxArray; SrcLen: Integer; Dst: PDCplxArray); cdecl;
procedure nspwUpSample(Factor, Phase: Integer; Src: PShortArray; SrcLen: Integer; Dst: PShortArray); cdecl;
procedure nspvUpSample(Factor, Phase: Integer; Src: PWCplxArray; SrcLen: Integer; Dst: PWCplxArray); cdecl;

// ------------------------------------------------------------------------*/
//        DownSample                                                       */
//                                                                         */
// Down-sample a  signal, conceptually decreasing  its sample rate  by an  */
// integer factor.                                                         */

procedure nspsDownSample(Factor, Phase: Integer; Src: PFloatArray; SrcLen: Integer; Dst: PFloatArray); cdecl;
procedure nspcDownSample(Factor, Phase: Integer; Src: PSCplxArray; SrcLen: Integer; Dst: PSCplxArray); cdecl;
procedure nspdDownSample(Factor, Phase: Integer; Src: PDoubleArray; SrcLen: Integer; Dst: PDoubleArray); cdecl;
procedure nspzDownSample(Factor, Phase: Integer; Src: PDCplxArray; SrcLen: Integer; Dst: PDCplxArray); cdecl;
procedure nspwDownSample(Factor, Phase: Integer; Src: PShortArray; SrcLen: Integer; Dst: PShortArray); cdecl;
procedure nspvDownSample(Factor, Phase: Integer; Src: PWCplxArray; SrcLen: Integer; Dst: PWCplxArray); cdecl;

//
//  RCS:
//      $Source: nsp30/include/nspvec.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/03 10:50:42 $
//
//  Purpose: NSP Vector Arithmetic and Algebraic Functions

const
  NSP_Gt	= 1;
  NSP_Lt	= 0;
  HUGE_VAL_S	= 3.402823466e+38;
  HUGE_VAL_D	= 1.7976931348623158e+308;

procedure nspsbSqr1(Vec: PFloatArray; Len: Integer); cdecl;
procedure nspcbSqr1(Vec: PSCplxArray; Len: Integer); cdecl;
procedure nspdbSqr1(Vec: PDoubleArray; Len: Integer); cdecl;
procedure nspzbSqr1(Vec: PDCplxArray; Len: Integer); cdecl;
procedure nspwbSqr1(Vec: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbSqr1(Vec: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

procedure nspsbSqr2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspcbSqr2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer); cdecl;
procedure nspdbSqr2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspzbSqr2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer); cdecl;
procedure nspwbSqr2(Src: PShortArray; Dst: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;
procedure nspvbSqr2(Src: PWCplxArray; Dst: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); cdecl;

procedure nspsbSqrt1(Vec: PFloatArray; Len: Integer); cdecl;
procedure nspcbSqrt1(Vec: PSCplxArray; Len: Integer); cdecl;
procedure nspdbSqrt1(Vec: PDoubleArray; Len: Integer); cdecl;
procedure nspzbSqrt1(Vec: PDCplxArray; Len: Integer); cdecl;
procedure nspwbSqrt1(Vec: PShortArray; Len: Integer); cdecl;
procedure nspvbSqrt1(Vec: PWCplxArray; Len: Integer); cdecl;

procedure nspsbSqrt2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspcbSqrt2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer); cdecl;
procedure nspdbSqrt2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspzbSqrt2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer); cdecl;
procedure nspwbSqrt2(Src: PShortArray; Dst: PShortArray; Len: Integer); cdecl;
procedure nspvbSqrt2(Src: PWCplxArray; Dst: PWCplxArray; Len: Integer); cdecl;

procedure nspsbThresh1(Vec: PFloatArray; Len: Integer; thresh: Float; relOp: Integer); cdecl;
procedure nspcbThresh1(Vec: PSCplxArray; Len: Integer; thresh: Float; relOp: Integer); cdecl;
procedure nspdbThresh1(Vec: PDoubleArray; Len: Integer; thresh: Double; relOp: Integer); cdecl;
procedure nspzbThresh1(Vec: PDCplxArray; Len: Integer; thresh: Double; relOp: Integer); cdecl;
procedure nspwbThresh1(Vec: PShortArray; Len: Integer; thresh: Short; relOp: Integer); cdecl;
procedure nspvbThresh1(Vec: PWCplxArray; Len: Integer; thresh: Short; relOp: Integer); cdecl;

procedure nspsbThresh2(Src: PFloatArray; Dst: PFloatArray; Len: Integer;
				thresh: Float; relOp: Integer); cdecl;
procedure nspcbThresh2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer;
				thresh: Float; relOp: Integer); cdecl;
procedure nspdbThresh2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer;
				thresh: Double; relOp: Integer); cdecl;
procedure nspzbThresh2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer;
				thresh: Double; relOp: Integer); cdecl;
procedure nspwbThresh2(Src: PShortArray; Dst: PShortArray; Len: Integer;
				thresh: Short; relOp: Integer); cdecl;
procedure nspvbThresh2(Src: PWCplxArray; Dst: PWCplxArray; Len: Integer;
				thresh: Short; relOp: Integer); cdecl;

procedure nspsbInvThresh1(Vec: PFloatArray; Len: Integer; Thresh: Float); cdecl;
procedure nspcbInvThresh1(Vec: PSCplxArray; Len: Integer; Thresh: Float); cdecl;
procedure nspdbInvThresh1(Vec: PDoubleArray; Len: Integer; Thresh: Double); cdecl;
procedure nspzbInvThresh1(Vec: PDCplxArray; Len: Integer; Thresh: Double); cdecl;

procedure nspsbInvThresh2(Src: PFloatArray; Dst: PFloatArray; Len: Integer; Thresh: Float); cdecl;
procedure nspcbInvThresh2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer; Thresh: Float); cdecl;
procedure nspdbInvThresh2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer; Thresh: Double); cdecl;
procedure nspzbInvThresh2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer; Thresh: Double); cdecl;

procedure nspsbAbs1(Vec: PFloatArray; Len: Integer); cdecl;
procedure nspdbAbs1(Vec: PDoubleArray; Len: Integer); cdecl;
procedure nspwbAbs1(Vec: PShortArray; Len: Integer); cdecl;

procedure nspsbAbs2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); cdecl;
procedure nspdbAbs2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); cdecl;
procedure nspwbAbs2(Src: PShortArray; Dst: PShortArray; Len: Integer); cdecl;

function nspsMax(Vec: PFloatArray; Len: Integer): Float; cdecl;
function nspdMax(Vec: PDoubleArray; Len: Integer): Double; cdecl;
function nspwMax(Vec: PShortArray; Len: Integer): Short; cdecl;

function nspsMean(Vec: PFloatArray; Len: Integer): Float; cdecl;
function nspdMean(Vec: PDoubleArray; Len: Integer): Double; cdecl;
function nspwMean(Vec: PShortArray; Len: Integer): Short; cdecl;

function nspsMin(Vec: PFloatArray; Len: Integer): Float; cdecl;
function nspdMin(Vec: PDoubleArray; Len: Integer): Double; cdecl;
function nspwMin(Vec: PShortArray; Len: Integer): Short; cdecl;

function nspsStdDev(Vec: PFloatArray; Len: Integer): Float; cdecl;
function nspdStdDev(Vec: PDoubleArray; Len: Integer): Double; cdecl;
function nspwStdDev(Vec: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): Short; cdecl;

//
//  RCS:
//      $Source: nsp30/include/nspdotp.h $
//      $Revision: 1.3 $
//      $Date: 1995/08/03 10:50:42 $
//
//  Purpose: NSP Vector Product Functions

function nspsDotProd(SrcA, SrcB: PFloatArray; Len: Integer): Float; cdecl;
function nspcDotProd(SrcA, SrcB: PSCplxArray; Len: Integer): TSCplx; cdecl;
function nspdDotProd(SrcA, SrcB: PDoubleArray; Len: Integer): Double; cdecl;
function nspzDotProd(SrcA, SrcB: PDCplxArray; Len: Integer): TDCplx; cdecl;
function nspwDotProd(SrcA, SrcB: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): Short; cdecl;
function nspvDotProd(SrcA, SrcB: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspscDotProd(SrcA: PFloatArray; SrcB: PSCplxArray; Len: Integer): TSCplx; cdecl;
function nspcsDotProd(SrcA: PSCplxArray; SrcB: PFloatArray; Len: Integer): TSCplx; cdecl;
function nspdzDotProd(SrcA: PDoubleArray; SrcB: PDCplxArray; Len: Integer): TDCplx; cdecl;
function nspzdDotProd(SrcA: PDCplxArray; SrcB: PDoubleArray; Len: Integer): TDCplx; cdecl;
function nspwvDotProd(SrcA: PShortArray; SrcB: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
function nspvwDotProd(SrcA: PWCplxArray; SrcB: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; cdecl;
{$ENDIF}
{$IFDEF nsp_UsesWin}
//  Purpose: NSP Windowing Functions. Declaration
//
//   Win                                                                   */
// Multiply a vector by a windowing function.                              */
procedure nspsWinBartlett(Vector: PFloatArray; N: Integer); cdecl;
procedure nspcWinBartlett(Vector: PSCplxArray; N: Integer); cdecl;
procedure nspdWinBartlett(Vector: PDoubleArray; N: Integer); cdecl;
procedure nspzWinBartlett(Vector: PDCplxArray; N: Integer); cdecl;
procedure nspwWinBartlett(Vector: PShortArray; N: Integer); cdecl;
procedure nspvWinBartlett(Vector: PWCplxArray; N: Integer); cdecl;

procedure nspsWinHann(Vector: PFloatArray; N: Integer); cdecl;
procedure nspcWinHann(Vector: PSCplxArray; N: Integer); cdecl;
procedure nspdWinHann(Vector: PDoubleArray; N: Integer); cdecl;
procedure nspzWinHann(Vector: PDCplxArray; N: Integer); cdecl;
procedure nspwWinHann(Vector: PShortArray; N: Integer); cdecl;
procedure nspvWinHann(Vector: PWCplxArray; N: Integer); cdecl;

procedure nspsWinHamming(Vector: PFloatArray; N: Integer); cdecl;
procedure nspcWinHamming(Vector: PSCplxArray; N: Integer); cdecl;
procedure nspdWinHamming(Vector: PDoubleArray; N: Integer); cdecl;
procedure nspzWinHamming(Vector: PDCplxArray; N: Integer); cdecl;
procedure nspwWinHamming(Vector: PShortArray; N: Integer); cdecl;
procedure nspvWinHamming(Vector: PWCplxArray; N: Integer); cdecl;

procedure nspsWinBlackman(Vector: PFloatArray; N: Integer; Alpha: Float); cdecl;
procedure nspcWinBlackman(Vector: PSCplxArray; N: Integer; Alpha: Float); cdecl;
procedure nspdWinBlackman(Vector: PDoubleArray; N: Integer; Alpha: Double); cdecl;
procedure nspzWinBlackman(Vector: PDCplxArray ; N: Integer; Alpha: Double); cdecl;
procedure nspwWinBlackman(Vector: PShortArray; N: Integer; Alpha: Float); cdecl;
procedure nspvWinBlackman(Vector: PWCplxArray; N: Integer; Alpha: Float); cdecl;

procedure nspsWinBlackmanStd(Vector: PFloatArray; N: Integer); cdecl;
procedure nspcWinBlackmanStd(Vector: PSCplxArray; N: Integer); cdecl;
procedure nspdWinBlackmanStd(Vector: PDoubleArray; N: Integer); cdecl;
procedure nspzWinBlackmanStd(Vector: PDCplxArray; N: Integer); cdecl;
procedure nspwWinBlackmanStd(Vector: PShortArray; N: Integer); cdecl;
procedure nspvWinBlackmanStd(Vector: PWCplxArray; N: Integer); cdecl;

procedure nspsWinBlackmanOpt(Vector: PFloatArray; N: Integer); cdecl;
procedure nspcWinBlackmanOpt(Vector: PSCplxArray; N: Integer); cdecl;
procedure nspdWinBlackmanOpt(Vector: PDoubleArray; N: Integer); cdecl;
procedure nspzWinBlackmanOpt(Vector: PDCplxArray; N: Integer); cdecl;
procedure nspwWinBlackmanOpt(Vector: PShortArray; N: Integer); cdecl;
procedure nspvWinBlackmanOpt(Vector: PWCplxArray; N: Integer); cdecl;

procedure nspsWinKaiser(Vector: PFloatArray; N: Integer; Beta: Float); cdecl;
procedure nspcWinKaiser(Vector: PSCplxArray; N: Integer; Beta: Float); cdecl;
procedure nspdWinKaiser(Vector: PDoubleArray; N: Integer; Beta: Double); cdecl;
procedure nspzWinKaiser(Vector: PDCplxArray ; N: Integer; Beta: Double); cdecl;
procedure nspwWinKaiser(Vector: PShortArray; N: Integer; Beta: Float); cdecl;
procedure nspvWinKaiser(Vector: PWCplxArray; N: Integer; Beta: Float); cdecl;
{$ENDIF}

implementation

const nspl = 'NSP.DLL';

function NSPsDegToRad(deg: float): float;
//  NSPsDegToRad(deg) ((float) ((deg)/180.0 * NSP_PI))
begin
  Result := deg * (PI / 180);
end;

function NSPdDegToRad(deg: double): double;
//  NSPdDegToRad(deg) ((double)((deg)/180.0 * NSP_PI))
begin
  Result := deg * (PI / 180);
end;

// nsperror.h

(*--- Macros --------------------------------------------------------------*

#define NSP_ERROR(status,func,context)\
		nspError((status),(func),(context),__FILE__,__LINE__)

#define NSP_ERRCHK(func,context)\
		((nspGetErrStatus() >= 0) ? NSP_StsOk \
		: NSP_ERROR(NSP_StsBackTrace,(func),(context)))

#define NSP_ASSERT(expr,func,context)\
		((expr) ? NSP_StsOk \
		: NSP_ERROR(NSP_StsInternal,(func),(context)))

procedure NSP_RSTERR;
begin
  nspSetErrStatus(NSP_StsOk);
end;
*)
function nspGetLibVersion: PNSPLibVersion; external nspl;

function nspGetErrStatus: NSPStatus; external nspl;
procedure nspSetErrStatus(Status: NSPStatus); external nspl;
function nspGetErrMode: Integer; external nspl;
procedure nspSetErrMode(Mode: Integer); external nspl;
function nspError(
	Status: NSPStatus;
	Func: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; external nspl;
function nspErrorStr(Status: NSPStatus): PChar; external nspl;
function nspNulDevReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; external nspl;
function nspStdErrReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; external nspl;
function nspGuiBoxReport(
	Status: NSPStatus;
	FuncName: PChar;
	Context: PChar;
	FileName: PChar;
	Line: Integer): NSPStatus; external nspl;
function nspRedirectError(nspErrorFunc: TNspErrorCallBack): TNspErrorCallBack; external nspl;

// nspalloc.h
function nspMalloc(Length: Integer): Pointer; external nspl;
function nspsMalloc(Length: Integer): PFloatArray; external nspl;
function nspdMalloc(Length: Integer): PDoubleArray; external nspl;
function nspcMalloc(Length: Integer): PSCplxArray; external nspl;
function nspzMalloc(Length: Integer): PDCplxArray; external nspl;
function nspwMalloc(Length: Integer): PShortArray; external nspl;
function nspvMalloc(Length: Integer): PWCplxArray; external nspl;
function nspiMalloc(Length: Integer): PLongArray; external nspl;
function nspjMalloc(Length: Integer): PICplxArray; external nspl;
procedure nspFree(P: Pointer); external nspl;

{$IFDEF nsp_UsesConversion}
procedure nspvbReal(Src: PWCplxArray; Dst: PShortArray; len: Integer); external nspl;
procedure nspvbImag(Src: PWCplxArray; Dst: PShortArray; len: Integer); external nspl;
procedure  nspvbMag(Src: PWCplxArray; Mag: PShortArray; len: Integer); external nspl;
procedure  nspvbPhase(Src: PWCplxArray; Phase: PShortArray; len: Integer); external nspl;
procedure  nspwbrMag(srcReal, srcImag: PShortArray; Mag: PShortArray; len: Integer); external nspl;
procedure  nspwbrPhase(srcReal, srcImag: PShortArray; Phase: PShortArray; len: Integer); external nspl;
procedure  nspvbCplxTo2Real(Src: PWCplxArray; DstReal, DstImag: PShortArray; len: Integer); external nspl;
procedure  nspvb2RealToCplx(srcReal, srcImag: PShortArray; Dst: PWCplxArray; len: Integer); external nspl;
procedure nspsbFloatToInt(Src: PFloatArray; var dst; len, wordSize, Flags: Integer); external nspl;
procedure nspdbFloatToInt(Src: PDoubleArray; var dst; len, wordSize, Flags: Integer); external nspl;
procedure nspsbIntToFloat(const src; Dst: PFloatArray; len, wordSize, flags: Integer); external nspl;
procedure nspdbIntToFloat(const src; Dst: PDoubleArray; len, wordSize, flags: Integer); external nspl;
procedure nspsbFloatToFix(Src: PFloatArray; var dst; len, wordSize, fractBits, flags: Integer); external nspl;
procedure nspdbFloatToFix(Src: PDoubleArray; var dst; len, wordSize, fractBits, flags: Integer); external nspl;
procedure nspsbFixToFloat(const src; Dst: PFloatArray; len, wordSize, fractBits, flags: Integer); external nspl;
procedure nspdbFixToFloat(const src; Dst: PDoubleArray; len, wordSize, fractBits, flags: Integer); external nspl;
procedure nspsbFloatToS31Fix(Src: PFloatArray; Dst: PLongArray; Len, Flags: Integer); external nspl;
procedure nspsbFloatToS15Fix(Src: PFloatArray; Dst: PShortArray; Len, Flags: Integer); external nspl;
procedure nspsbFloatToS7Fix(Src: PFloatArray; Dst: PByteArray; Len, Flags: Integer); external nspl;
procedure nspsbFloatToS1516Fix(Src: PFloatArray; Dst: PLongArray; Len, Flags: Integer); external nspl;
procedure nspdbFloatToS31Fix(Src: PDoubleArray; Dst: PLongArray; Len, Flags: Integer); external nspl;
procedure nspdbFloatToS15Fix(Src: PDoubleArray; Dst: PShortArray; Len, Flags: Integer); external nspl;
procedure nspdbFloatToS7Fix(Src: PDoubleArray; Dst: PByteArray; Len, Flags: Integer); external nspl;
procedure nspdbFloatToS1516Fix(Src: PDoubleArray; Dst: PLongArray; Len, Flags: Integer); external nspl;
procedure nspsbS31FixToFloat(Src: PLongArray ; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspsbS15FixToFloat(Src: PShortArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspsbS7FixToFloat(Src: PByteArray ; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspsbS1516FixToFloat(Src: PLongArray ; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspdbS31FixToFloat(Src: PLongArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspdbS15FixToFloat(Src: PShortArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspdbS7FixToFloat(Src: PByteArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspdbS1516FixToFloat(Src: PLongArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspcbReal(Src: PSCplxArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspzbReal(Src: PDCplxArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspcbImag(Src: PSCplxArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspzbImag(Src: PDCplxArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspcbCplxTo2Real(Src: PSCplxArray; DstReal, DstImag: PFloatArray; Len: Integer); external nspl;
procedure nspzbCplxTo2Real(Src: PDCplxArray; DstReal, DstImag: PDoubleArray; Len: Integer); external nspl;
procedure nspcb2RealToCplx(srcReal, srcImag: PFloatArray; Dst: PSCplxArray; Len: Integer); external nspl;
procedure nspzb2RealToCplx(srcReal, srcImag: PDoubleArray; Dst: PDCplxArray; Len: Integer); external nspl;
procedure nspcbCartToPolar(Src: PSCplxArray; Mag, Phase: PFloatArray; Len: Integer); external nspl;
procedure nspzbCartToPolar(Src: PDCplxArray; Mag, Phase: PDoubleArray; Len: Integer); external nspl;
procedure nspsbrCartToPolar(srcReal, srcImag: PFloatArray; Mag, Phase: PFloatArray; Len: Integer); external nspl;
procedure nspdbrCartToPolar(srcReal, srcImag: PDoubleArray; Mag, Phase: PDoubleArray; Len: Integer); external nspl;
procedure nspcbPolarToCart(Mag, Phase: PFloatArray; Dst: PSCplxArray; Len: Integer); external nspl;
procedure nspzbPolarToCart(Mag, Phase: PDoubleArray; Dst: PDCplxArray; Len: Integer); external nspl;
procedure nspsbrPolarToCart(Mag, Phase: PFloatArray; DstReal, DstImag: PFloatArray; Len: Integer); external nspl;
procedure nspdbrPolarToCart(Mag, Phase: PDoubleArray; DstReal, DstImag: PDoubleArray; Len: Integer); external nspl;
procedure nspcbMag(Src: PSCplxArray; Mag: PFloatArray; Len: Integer); external nspl;
procedure nspzbMag(Src: PDCplxArray; Mag: PFloatArray; Len: Integer); external nspl;
procedure nspsbrMag(srcReal, srcImag: PFloatArray; Mag: PFloatArray; Len: Integer); external nspl;
procedure nspdbrMag(srcReal, srcImag: PDoubleArray; Mag: PDoubleArray; Len: Integer); external nspl;
procedure nspcbPhase(Src: PSCplxArray; Phase: PFloatArray; Len: Integer); external nspl;
procedure nspzbPhase(Src: PDCplxArray; Phase: PDoubleArray; Len: Integer); external nspl;
procedure nspsbrPhase(srcReal, srcImag: PFloatArray; Phase: PFloatArray; Len: Integer); external nspl;
procedure nspdbrPhase(srcReal, srcImag: PDoubleArray; Phase: PDoubleArray; Len: Integer); external nspl;
{$ENDIF}
{$IFDEF nsp_UsesFir}
procedure nspsFirlInit(Taps: PFloatArray; tapsLen: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspdFirlInit(Taps: PDoubleArray; tapsLen: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspcFirlInit(Taps: PSCplxArray; tapsLen: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspzFirlInit(Taps: PDCplxArray; tapsLen: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspwFirlInit(Taps: PShortArray; tapsLen: Integer; var tapState: TNSPFirTapState); external nspl;

procedure nspsFirlInitMr(Taps: PFloatArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspdFirlInitMr(Taps: PDoubleArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspcFirlInitMr(Taps: PSCplxArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspzFirlInitMr(Taps: PDCplxArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); external nspl;
procedure nspwFirlInitMr(Taps: PShortArray; tapsLen, upFactor, upPhase: Integer;
		    downFactor, downPhase: Integer; var tapState: TNSPFirTapState); external nspl;

procedure nspsFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PFloatArray;
				    var dlyState: TNSPFirDlyState); external nspl;
procedure nspcFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PSCplxArray;
				    var dlyState: TNSPFirDlyState); external nspl;
procedure nspdFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PDoubleArray;
				    var dlyState: TNSPFirDlyState); external nspl;
procedure nspzFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PDCplxArray;
				    var dlyState: TNSPFirDlyState); external nspl;
procedure nspwFirlInitDlyl(const tapState: TNSPFirTapState; Dlyl: PShortArray;
				    var dlyState: TNSPFirDlyState); external nspl;

function nspsFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: float): float; external nspl;
function _nspcFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; external nspl name 'nspcFirl';
function nspcFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.im
	push	Samp.re
	push	dlyState
	push	tapState
	call	_nspcFirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: Double): Double; external nspl;
function nspzFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: TDCplx): TDCplx; external nspl;
function nspwFirl(const tapState: TNSPFirTapState;
			      var dlyState: TNSPFirDlyState; samp: Short;
			      scaleMode: Integer; var scaleFactor: Integer): Short; external nspl;

function _nspscFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: Float): TSCplx; external nspl name 'nspscFirl';
function nspscFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: Float): TSCplx; assembler;
asm
	push	Samp
	push	dlyState
	push	tapState
	call	_nspscFirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; external nspl name 'nspcsFirl';
function nspcsFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.im
	push	Samp.re
	push	dlyState
	push	tapState
	call	_nspcsFirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: Double): TDCplx; external nspl;
function nspzdFirl(const tapState: TNSPFirTapState;
			       var dlyState: TNSPFirDlyState; samp: TDCplx): TDCplx; external nspl;

procedure nspsbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PFloatArray; outSamps: PFloatArray; numIters: Integer); external nspl;
procedure nspcbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PSCplxArray; outSamps: PSCplxArray; numIters: Integer); external nspl;
procedure nspdbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PDoubleArray; outSamps: PDoubleArray; numIters: Integer); external nspl;
procedure nspzbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PDCplxArray; outSamps: PDCplxArray; numIters: Integer); external nspl;
procedure nspwbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			 inSamps: PShortArray; outSamps: PShortArray; numIters: Integer;
			   scaleMode: Integer; var scaleFactor: Integer); external nspl;

procedure nspscbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PFloatArray; outSamps: PSCplxArray; numIters: Integer); external nspl;
procedure nspcsbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PSCplxArray; outSamps: PSCplxArray; numIters: Integer); external nspl;
procedure nspdzbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PDoubleArray; outSamps: PDCplxArray; numIters: Integer); external nspl;
procedure nspzdbFirl(const tapState: TNSPFirTapState; var dlyState: TNSPFirDlyState;
			  inSamps: PDCplxArray; outSamps: PDCplxArray; numIters: Integer); external nspl;

procedure nspsFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PFloatArray); external nspl;
procedure nspcFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PSCplxArray); external nspl;
procedure nspdFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PDoubleArray); external nspl;
procedure nspzFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PDCplxArray); external nspl;
procedure nspwFirlGetTaps(const tapState: TNSPFirTapState; OutTaps: PShortArray); external nspl;

procedure nspsFirlSetTaps(InTaps: PFloatArray; var tapState: TNSPFirTapState); external nspl;
procedure nspcFirlSetTaps(InTaps: PSCplxArray; var tapState: TNSPFirTapState); external nspl;
procedure nspdFirlSetTaps(InTaps: PDoubleArray; var tapState: TNSPFirTapState); external nspl;
procedure nspzFirlSetTaps(InTaps: PDCplxArray; var tapState: TNSPFirTapState); external nspl;
procedure nspwFirlSetTaps(InTaps: PShortArray; var tapState: TNSPFirTapState); external nspl;

procedure nspsFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PFloatArray); external nspl;
procedure nspcFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PSCplxArray); external nspl;
procedure nspdFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PDoubleArray); external nspl;
procedure nspzFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PDCplxArray); external nspl;
procedure nspwFirlGetDlyl(const tapState: TNSPFirTapState;
			     const dlyState: TNSPFirDlyState; OutDlyl: PShortArray); external nspl;


procedure nspsFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PFloatArray;
				   var dlyState: TNSPFirDlyState); external nspl;
procedure nspcFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PSCplxArray;
				   var dlyState: TNSPFirDlyState); external nspl;
procedure nspdFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PDoubleArray;
				   var dlyState: TNSPFirDlyState); external nspl;
procedure nspzFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PDCplxArray;
				   var dlyState: TNSPFirDlyState); external nspl;
procedure nspwFirlSetDlyl(const tapState: TNSPFirTapState; inDlyl: PShortArray;
				   var dlyState: TNSPFirDlyState); external nspl;

// nspfirh.h

procedure nspsFirInit(TapVals: PFloatArray; tapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); external nspl;
procedure nspcFirInit(TapVals: PSCplxArray; tapsLen: Integer;
			  DlyVals: PSCplxArray; var State: TNSPFirState); external nspl;
procedure nspscFirInit(TapVals: PSCplxArray; tapsLen: Integer;
			  DlyVals: PFloatArray; var State: TNSPFirState); external nspl;
procedure nspcsFirInit(TapVals: PFloatArray; tapsLen: Integer;
			  DlyVals: PSCplxArray; var State: TNSPFirState); external nspl;

procedure nspdFirInit(TapVals: PDoubleArray; tapsLen: Integer;
			  DlyVals: PDoubleArray; var State: TNSPFirState); external nspl;
procedure nspzFirInit(TapVals: PDCplxArray; tapsLen: Integer;
			  DlyVals: PDCplxArray; var State: TNSPFirState); external nspl;
procedure nspdzFirInit(TapVals: PDCplxArray; tapsLen: Integer;
			  DlyVals: PDoubleArray; var State: TNSPFirState); external nspl;
procedure nspzdFirInit(TapVals: PDoubleArray; tapsLen: Integer;
			  DlyVals: PDCplxArray; var State: TNSPFirState); external nspl;

procedure nspwFirInit(TapVals: PShortArray; tapsLen: Integer;
			  DlyVals: PShortArray; var State: TNSPFirState); external nspl;

procedure nspsFirInitMr(TapVals: PFloatArray; tapsLen: Integer;
			    DlyVals: PFloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspcFirInitMr(TapVals: PSCplxArray; tapsLen: Integer;
			    DlyVals: PSCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspscFirInitMr(TapVals: PSCplxArray; tapsLen: Integer;
			    DlyVals: PfloatArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspcsFirInitMr(TapVals: PFloatArray; tapsLen: Integer;
			    DlyVals: PSCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;

procedure nspdFirInitMr(TapVals: PDoubleArray; tapsLen: Integer;
			    DlyVals: PDoubleArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspzFirInitMr(TapVals: PDCplxArray; tapsLen: Integer;
			    DlyVals: PDCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspdzFirInitMr(TapVals: PDCplxArray; tapsLen: Integer;
			    DlyVals: PDoubleArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;
procedure nspzdFirInitMr(TapVals: PDoubleArray; tapsLen: Integer;
			    DlyVals: PDCplxArray; upFactor, upPhase: Integer;
				  downFactor, downPhase: Integer;
			    var State: TNSPFirState); external nspl;

procedure nspwFirInitMr(TapVals: PShortArray; tapsLen: Integer;
			   DlyVals: PShortArray; upFactor, upPhase,
			   downFactor, downPhase: Integer;
			   var State: TNSPFirState); external nspl;

procedure nspFirFree(var State: TNSPFirState); external nspl;


function nspsFir(var State: TNSPFirState; samp: Float): Float; external nspl;

function _nspcFir(var State: TNSPFirState; samp: TSCplx): TSCplx; external nspl name 'nspcFir';
function nspcFir(var State: TNSPFirState; samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcFir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspscFir(var State: TNSPFirState; samp: float): TSCplx; external nspl name 'nspscFir';
function nspscFir(var State: TNSPFirState; samp: float): TSCplx; assembler;
asm
	push	Samp
	push	State
	call	_nspscFir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsFir(var State: TNSPFirState; samp: TSCplx): TSCplx; external nspl name 'nspcsFir';
function nspcsFir(var State: TNSPFirState; samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcsFir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdFir(var State: TNSPFirState; samp: Double): Double; external nspl;
function nspzFir(var State: TNSPFirState; samp: TDCplx): TDCplx; external nspl;
function nspdzFir(var State: TNSPFirState; samp: double): TDCplx; external nspl;
function nspzdFir(var State: TNSPFirState; samp: TDCplx): TDCplx; external nspl;

function nspwFir(var State: TNSPFirState; samp: Short;
		       sMode: Integer; var sFct: Integer): Short; external nspl;

procedure nspsbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PFloatArray; numIters: Integer); external nspl;
procedure nspcbFir(var State: TNSPFirState; inSamps: PSCplxArray;
		       outSamps: PSCplxArray; numIters: Integer); external nspl;
procedure nspscbFir(var State: TNSPFirState; inSamps: PFloatArray;
		       outSamps: PSCplxArray; numIters: Integer); external nspl;
procedure nspcsbFir(var State: TNSPFirState; inSamps: PSCplxArray;
		       outSamps: PSCplxArray; numIters: Integer); external nspl;

procedure nspdbFir(var State: TNSPFirState; inSamps: PDoubleArray;
		       outSamps: PDoubleArray; numIters: Integer); external nspl;
procedure nspzbFir(var State: TNSPFirState; inSamps: PDCplxArray;
		       outSamps: PDCplxArray; numIters: Integer); external nspl;
procedure nspdzbFir(var State: TNSPFirState; inSamps: PDoubleArray;
		       outSamps: PDCplxArray; numIters: Integer); external nspl;
procedure nspzdbFir(var State: TNSPFirState; inSamps: PDCplxArray;
		       outSamps: PDCplxArray; numIters: Integer); external nspl;

procedure nspwbFir(var State: TNSPFirState; inSamps: PShortArray;
		       outSamps: PShortArray; numIters, sMode: Integer;
		       var sFct: Integer); external nspl;


procedure nspsFirGetTaps(const State: TNSPFirState; OutTaps: PFloatArray); external nspl;
procedure nspcFirGetTaps(const State: TNSPFirState; OutTaps: PSCplxArray); external nspl;
procedure nspdFirGetTaps(const State: TNSPFirState; OutTaps: PDoubleArray); external nspl;
procedure nspzFirGetTaps(const State: TNSPFirState; OutTaps: PDCplxArray); external nspl;

procedure nspwFirGetTaps(const State: TNSPFirState; OutTaps: PShortArray); external nspl;

procedure nspsFirSetTaps(InTaps: PFloatArray; var State: TNSPFirState); external nspl;
procedure nspcFirSetTaps(InTaps: PSCplxArray; var State: TNSPFirState); external nspl;
procedure nspdFirSetTaps(InTaps: PDoubleArray; var State: TNSPFirState); external nspl;
procedure nspzFirSetTaps(InTaps: PDCplxArray; var State: TNSPFirState); external nspl;

procedure nspwFirSetTaps(InTaps: PShortArray; var State: TNSPFirState); external nspl;


procedure nspsFirGetDlyl(const State: TNSPFirState; OutDlyl: PFloatArray); external nspl;
procedure nspcFirGetDlyl(const State: TNSPFirState; OutDlyl: PSCplxArray); external nspl;
procedure nspdFirGetDlyl(const State: TNSPFirState; OutDlyl: PDoubleArray); external nspl;
procedure nspzFirGetDlyl(const State: TNSPFirState; OutDlyl: PDCplxArray); external nspl;

procedure nspwFirGetDlyl(const State: TNSPFirState; OutDlyl: PShortArray); external nspl;

procedure nspsFirSetDlyl(inDlyl: PFloatArray; var State: TNSPFirState); external nspl;
procedure nspcFirSetDlyl(inDlyl: PSCplxArray; var State: TNSPFirState); external nspl;
procedure nspdFirSetDlyl(inDlyl: PDoubleArray; var State: TNSPFirState); external nspl;
procedure nspzFirSetDlyl(inDlyl: PDCplxArray; var State: TNSPFirState); external nspl;

procedure nspwFirSetDlyl(inDlyl: PShortArray; var State: TNSPFirState); external nspl;
{$ENDIF}
{$IFDEF nsp_UsesTransform}
// nspfft.h
  procedure nspvDft(
	inSamps: PWCplxArray;
	outSamps: PWCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;

  procedure nspcDft(
	inSamps: PSCplxArray;
	outSamps: PSCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;

  procedure nspzDft(
	inSamps: PDCplxArray;
	outSamps: PDCplxArray;
	Length: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;

  procedure nspvFft(
	Samps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspcFft(
	Samps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspzFft(
	Samps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspvFftNip(
	inSamps: PWCplxArray;
	outSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspcFftNip(
	inSamps: PSCplxArray;
	outSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspzFftNip(
	inSamps: PDCplxArray;
	outSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspvrFft(
	reSamps: PShortArray;
	imSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspcrFft(
	reSamps: PFloatArray;
	imSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspzrFft(
	reSamps: PDoubleArray;
	imSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspvrFftNip(
	reInSamps: PShortArray;
	imInSamps: PShortArray;
	reOutSamps: PShortArray;
	imOutSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspcrFftNip(
	reInSamps: PFloatArray;
	imInSamps: PFloatArray;
	reOutSamps: PFloatArray;
	imOutSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspzrFftNip(
	reInSamps: PDoubleArray;
	imInSamps: PDoubleArray;
	reOutSamps: PDoubleArray;
	imOutSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwRealFftl(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsRealFftl(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdRealFftl(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwRealFftlNip(
	inSamps: PShortArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsRealFftlNip(
	inSamps: PFloatArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdRealFftlNip(
	inSamps: PDoubleArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcsFftl(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcsFftl(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcsFftl(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcsFftlNip(
	inSamps: PShortArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcsFftlNip(
	inSamps: PFloatArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcsFftlNip(
	inSamps: PDoubleArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwMpyRCPack2(
	Src: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsMpyRCPack2(
	Src: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); external nspl;
  procedure nspdMpyRCPack2(
	Src: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); external nspl;

  procedure nspwMpyRCPack3(
	SrcA: PShortArray;
	SrcB: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsMpyRCPack3(
	SrcA: PFloatArray;
	SrcB: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); external nspl;
  procedure nspdMpyRCPack3(
	SrcA: PDoubleArray;
	SrcB: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); external nspl;

  procedure nspwMpyRCPerm2(
	Src: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsMpyRCPerm2(
	Src: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); external nspl;
  procedure nspdMpyRCPerm2(
	Src: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); external nspl;

  procedure nspwMpyRCPerm3(
	SrcA: PShortArray;
	SrcB: PShortArray;
	Dst: PShortArray;
	Order: Integer;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsMpyRCPerm3(
	SrcA: PFloatArray;
	SrcB: PFloatArray;
	Dst: PFloatArray;
	Order: Integer); external nspl;
  procedure nspdMpyRCPerm3(
	SrcA: PDoubleArray;
	SrcB: PDoubleArray;
	Dst: PDoubleArray;
	Order: Integer); external nspl;

  procedure nspwRealFft(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsRealFft(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdRealFft(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwRealFftNip(
	inSamps: PShortArray;
	outSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsRealFftNip(
	inSamps: PFloatArray;
	outSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdRealFftNip(
	inSamps: PDoubleArray;
	outSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcsFft(
	Samps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcsFft(
	Samps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcsFft(
	Samps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcsFftNip(
	inSamps: PWCplxArray;
	outSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcsFftNip(
	inSamps: PSCplxArray;
	outSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcsFftNip(
	inSamps: PDCplxArray;
	outSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwReal2Fft(
	xSamps: PShortArray;
	ySamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsReal2Fft(
	xSamps: PFloatArray;
	ySamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdReal2Fft(
	xSamps: PDoubleArray;
	ySamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwReal2FftNip(
	xInSamps: PShortArray;	xOutSamps: PWCplxArray;
	yInSamps: PShortArray;	yOutSamps: PWCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsReal2FftNip(
	xInSamps: PFloatArray;	xOutSamps: PSCplxArray;
	yInSamps: PFloatArray;	yOutSamps: PSCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdReal2FftNip(
	xInSamps: PDoubleArray; xOutSamps: PDCplxArray;
	yInSamps: PDoubleArray; yOutSamps: PDCplxArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcs2Fft(
	xSamps: PShortArray;
	ySamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcs2Fft(
	xSamps: PFloatArray;
	ySamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcs2Fft(
	xSamps: PDoubleArray;
	ySamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

  procedure nspwCcs2FftNip(
	xInSamps: PWCplxArray; xOutSamps: PShortArray;
	yInSamps: PWCplxArray; yOutSamps: PShortArray;
	Order: Integer;
	Flags: TNspFourierTrOptions;
	ScaleMode: Integer;
	var ScaleFactor: Integer); external nspl;
  procedure nspsCcs2FftNip(
	xInSamps: PSCplxArray; xOutSamps: PFloatArray;
	yInSamps: PSCplxArray; yOutSamps: PFloatArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;
  procedure nspdCcs2FftNip(
	xInSamps: PDCplxArray; xOutSamps: PDoubleArray;
	yInSamps: PDCplxArray; yOutSamps: PDoubleArray;
	Order: Integer;
	Flags: TNspFourierTrOptions); external nspl;

procedure nspsGoertzInit(Freq: Float; var State: TNspSGoertzState); external nspl;
procedure nspcGoertzInit(Freq: Float; var State: TNspCGoertzState); external nspl;
procedure nspwGoertzInit(Freq: Float; var State: TNspWGoertzState); external nspl;
procedure nspvGoertzInit(Freq: Float; var State: TNspVGoertzState); external nspl;
procedure nspdGoertzInit(Freq: Double; var State: TNspDGoertzState); external nspl;
procedure nspzGoertzInit(Freq: Double; var State: TNspZGoertzState); external nspl;

procedure nspsGoertzReset(var State: TNspSGoertzState); external nspl;
procedure nspcGoertzReset(var State: TNspCGoertzState); external nspl;
procedure nspwGoertzReset(var State: TNspWGoertzState); external nspl;
procedure nspvGoertzReset(var State: TNspVGoertzState); external nspl;
procedure nspdGoertzReset(var State: TNspDGoertzState); external nspl;
procedure nspzGoertzReset(var State: TNspZGoertzState); external nspl;

function _nspsGoertz(var State: TNspSGoertzState; Sample: Float): TSCplx; external nspl name 'nspsGoertz';
function nspsGoertz(var State: TNspSGoertzState; Sample: Float): TSCplx; assembler;
asm
	push	Sample
	push	State
	call	_nspsGoertz
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcGoertz(var State: TNspCGoertzState; Sample: TSCplx): TSCplx; external nspl name 'nspcGoertz';
function nspcGoertz(var State: TNspCGoertzState; Sample: TSCplx): TSCplx; assembler;
asm
	push	Sample.Im
	push	Sample.Re
	push	State
	call	_nspcGoertz
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspwGoertz(var State: TNspWGoertzState; Sample: Short; ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
function nspvGoertz(var State: TNspVGoertzState; Sample: TWCplx; ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
function nspdGoertz(var State: TNspDGoertzState; Sample: Double): TDCplx; external nspl;
function nspzGoertz(var State: TNspZGoertzState; Sample: TDCplx): TDCplx; external nspl;

function _nspsbGoertz(var State: TNspSGoertzState; inSamps: PFloatArray; Len: Integer): TSCplx; external nspl name 'nspsbGoertz';
function nspsbGoertz(var State: TNspSGoertzState; inSamps: PFloatArray; Len: Integer): TSCplx; assembler;
asm
	push	Len
	push	InSamps
	push	State
	call	_nspsbGoertz
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcbGoertz(var State: TNspCGoertzState; inSamps: PSCplxArray; Len: Integer): TSCplx; external nspl name 'nspcbGoertz';
function nspcbGoertz(var State: TNspCGoertzState; inSamps: PSCplxArray; Len: Integer): TSCplx; assembler;
asm
	push	Len
	push	InSamps
	push	State
	call	_nspcbGoertz
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspwbGoertz(var State: TNspWGoertzState; inSamps: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
function nspvbGoertz(var State: TNspVGoertzState; inSamps: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
function nspdbGoertz(var State: TNspDGoertzState; inSamps: PDoubleArray; Len: Integer): TDCplx; external nspl;
function nspzbGoertz(var State: TNspZGoertzState; inSamps: PDCplxArray; Len: Integer): TDCplx; external nspl;
{$ENDIF}
{$IFDEF nsp_UsesWin}
procedure nspsWinBartlett(Vector: PFloatArray; N: Integer); external nspl;
procedure nspcWinBartlett(Vector: PSCplxArray; N: Integer); external nspl;
procedure nspdWinBartlett(Vector: PDoubleArray; N: Integer); external nspl;
procedure nspzWinBartlett(Vector: PDCplxArray; N: Integer); external nspl;
procedure nspwWinBartlett(Vector: PShortArray; N: Integer); external nspl;
procedure nspvWinBartlett(Vector: PWCplxArray; N: Integer); external nspl;

procedure nspsWinHann(Vector: PFloatArray; N: Integer); external nspl;
procedure nspcWinHann(Vector: PSCplxArray; N: Integer); external nspl;
procedure nspdWinHann(Vector: PDoubleArray; N: Integer); external nspl;
procedure nspzWinHann(Vector: PDCplxArray; N: Integer); external nspl;
procedure nspwWinHann(Vector: PShortArray; N: Integer); external nspl;
procedure nspvWinHann(Vector: PWCplxArray; N: Integer); external nspl;

procedure nspsWinHamming(Vector: PFloatArray; N: Integer); external nspl;
procedure nspcWinHamming(Vector: PSCplxArray; N: Integer); external nspl;
procedure nspdWinHamming(Vector: PDoubleArray; N: Integer); external nspl;
procedure nspzWinHamming(Vector: PDCplxArray; N: Integer); external nspl;
procedure nspwWinHamming(Vector: PShortArray; N: Integer); external nspl;
procedure nspvWinHamming(Vector: PWCplxArray; N: Integer); external nspl;

procedure nspsWinBlackman(Vector: PFloatArray; N: Integer; Alpha: Float); external nspl;
procedure nspcWinBlackman(Vector: PSCplxArray; N: Integer; Alpha: Float); external nspl;
procedure nspdWinBlackman(Vector: PDoubleArray; N: Integer; Alpha: Double); external nspl;
procedure nspzWinBlackman(Vector: PDCplxArray ; N: Integer; Alpha: Double); external nspl;
procedure nspwWinBlackman(Vector: PShortArray; N: Integer; Alpha: Float); external nspl;
procedure nspvWinBlackman(Vector: PWCplxArray; N: Integer; Alpha: Float); external nspl;

procedure nspsWinBlackmanStd(Vector: PFloatArray; N: Integer); external nspl;
procedure nspcWinBlackmanStd(Vector: PSCplxArray; N: Integer); external nspl;
procedure nspdWinBlackmanStd(Vector: PDoubleArray; N: Integer); external nspl;
procedure nspzWinBlackmanStd(Vector: PDCplxArray; N: Integer); external nspl;
procedure nspwWinBlackmanStd(Vector: PShortArray; N: Integer); external nspl;
procedure nspvWinBlackmanStd(Vector: PWCplxArray; N: Integer); external nspl;

procedure nspsWinBlackmanOpt(Vector: PFloatArray; N: Integer); external nspl;
procedure nspcWinBlackmanOpt(Vector: PSCplxArray; N: Integer); external nspl;
procedure nspdWinBlackmanOpt(Vector: PDoubleArray; N: Integer); external nspl;
procedure nspzWinBlackmanOpt(Vector: PDCplxArray; N: Integer); external nspl;
procedure nspwWinBlackmanOpt(Vector: PShortArray; N: Integer); external nspl;
procedure nspvWinBlackmanOpt(Vector: PWCplxArray; N: Integer); external nspl;

procedure nspsWinKaiser(Vector: PFloatArray; N: Integer; Beta: Float); external nspl;
procedure nspcWinKaiser(Vector: PSCplxArray; N: Integer; Beta: Float); external nspl;
procedure nspdWinKaiser(Vector: PDoubleArray; N: Integer; Beta: Double); external nspl;
procedure nspzWinKaiser(Vector: PDCplxArray ; N: Integer; Beta: Double); external nspl;
procedure nspwWinKaiser(Vector: PShortArray; N: Integer; Beta: Float); external nspl;
procedure nspvWinKaiser(Vector: PWCplxArray; N: Integer; Beta: Float); external nspl;
{$ENDIF}
{$IFDEF nsp_UsesMisc}
function nspBitRev(A, Order: Integer): Integer; external nspl;
function nspGetBitRevTbl(Order: Integer): PIntArray; external nspl;
procedure nspCalcBitRevTbl(Tbl: PIntArray; Order: Integer); external nspl;
procedure nspFreeBitRevTbls; external nspl;
procedure nspwbBitRev1(Vector: PShortArray; Order: Integer); external nspl;
procedure nspvbBitRev1(Vector: PWCplxArray; Order: Integer); external nspl;
procedure nspsbBitRev1(Vector: PFloatArray; Order: Integer); external nspl;
procedure nspcbBitRev1(Vector: PSCplxArray; Order: Integer); external nspl;
procedure nspdbBitRev1(Vector: PDoubleArray; Order: Integer); external nspl;
procedure nspzbBitRev1(Vector: PDCplxArray; Order: Integer); external nspl;

procedure nspwbBitRev2(Src: PShortArray;  Dst: PShortArray;  Order: Integer); external nspl;
procedure nspvbBitRev2(Src: PWCplxArray;  Dst: PWCplxArray;  Order: Integer); external nspl;
procedure nspsbBitRev2(Src: PFloatArray;  Dst: PFloatArray;  Order: Integer); external nspl;
procedure nspcbBitRev2(Src: PSCplxArray;  Dst: PSCplxArray;  Order: Integer); external nspl;
procedure nspdbBitRev2(Src: PDoubleArray; Dst: PDoubleArray; Order: Integer); external nspl;
procedure nspzbBitRev2(Src: PDCplxArray;  Dst: PDCplxArray;  Order: Integer); external nspl;
procedure nspvCalcFftTwdTbl(var Tbl: TWCplx; order: Integer); external nspl;
procedure nspcCalcDftTwdTbl(var Tbl: TSCplx; length: Integer); external nspl;
procedure nspcCalcFftTwdTbl(var Tbl: TSCplx; order: Integer); external nspl;
procedure nspzCalcDftTwdTbl(var Tbl: TDCplx; length: Integer); external nspl;
procedure nspzCalcFftTwdTbl(var Tbl: TDCplx; order: Integer); external nspl;

function nspvGetFftTwdTbl(Order: Integer):  PWCplxArray; external nspl;
function nspcGetDftTwdTbl(Length: Integer): PSCplxArray; external nspl;
function nspcGetFftTwdTbl(Order: Integer):  PSCplxArray; external nspl;
function nspzGetDftTwdTbl(Length: Integer): PDCplxArray; external nspl;
function nspzGetFftTwdTbl(Order: Integer):  PDCplxArray; external nspl;

procedure nspvFreeTwdTbls; external nspl;
procedure nspcFreeTwdTbls; external nspl;
procedure nspzFreeTwdTbls; external nspl;
{$ENDIF nsp_UsesMisc}
{$IFDEF nsp_UsesSampleGen}
procedure nspsRandUniInit(Seed, Low, High: Float; var State: TNSPSRandUniState); external nspl;
procedure nspcRandUniInit(Seed, Low, High: Float; var State: TNSPCRandUniState); external nspl;
procedure nspdRandUniInit(Seed, Low, High: Double; var State: TNSPDRandUniState); external nspl;
procedure nspzRandUniInit(Seed, Low, High: Double; var State: TNSPZRandUniState); external nspl;
procedure nspwRandUniInit(Seed: UInt; Low, High: Short; var State: TNSPWRandUniState); external nspl;
procedure nspvRandUniInit(Seed: UInt; Low, High: Short; var State: TNSPVRandUniState); external nspl;
function nspsRandUni(var State: TNSPSRandUniState): Float; external nspl;

function _nspcRandUni(var State: TNSPCRandUniState): TSCplx; external nspl name 'nspcRandUni';
function nspcRandUni(var State: TNSPCRandUniState): TSCplx; assembler;
asm
	push	State
	call	_nspcRandUni
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdRandUni(var State: TNSPDRandUniState): Double; external nspl;
function nspzRandUni(var State: TNSPZRandUniState): TDCplx; external nspl;
function nspwRandUni(var State: TNSPWRandUniState): Short; external nspl;
function nspvRandUni(var State: TNSPVRandUniState): TWCplx; external nspl;

procedure nspsbRandUni(var State: TNSPSRandUniState; Samps: PFloatArray; SampsLen: Integer); external nspl;
procedure nspcbRandUni(var State: TNSPCRandUniState; Samps: PSCplxArray; SampsLen: Integer); external nspl;
procedure nspdbRandUni(var State: TNSPDRandUniState; Samps: PDoubleArray; SampsLen: Integer); external nspl;
procedure nspzbRandUni(var State: TNSPZRandUniState; Samps: PDCplxArray; SampsLen: Integer); external nspl;
procedure nspwbRandUni(var State: TNSPWRandUniState; Samps: PShortArray; SampsLen: Integer); external nspl;
procedure nspvbRandUni(var State: TNSPVRandUniState; Samps: PWCplxArray; SampsLen: Integer); external nspl;
procedure nspsRandGausInit(Seed, Mean, StDev: Float; var State: TNSPSRandGausState); external nspl;
procedure nspcRandGausInit(Seed, Mean, StDev: Float; var State: TNSPCRandGausState); external nspl;
procedure nspdRandGausInit(Seed, Mean, StDev: Double; var State: TNSPDRandGausState); external nspl;
procedure nspzRandGausInit(Seed, Mean, StDev: Double; var State: TNSPZRandGausState); external nspl;
procedure nspwRandGausInit(Seed, Mean, StDev: Short; var State: TNSPWRandGausState); external nspl;
procedure nspvRandGausInit(Seed, Mean, StDev: Short; var State: TNSPVRandGausState); external nspl;

function nspsRandGaus(var State: TNSPSRandGausState): Float; external nspl;

function _nspcRandGaus(var State: TNSPCRandGausState): TSCplx; external nspl name 'nspcRandGaus';
function nspcRandGaus(var State: TNSPCRandGausState): TSCplx; assembler;
asm
	push	State
	call	_nspcRandGaus
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdRandGaus(var State: TNSPDRandGausState): Double; external nspl;
function nspzRandGaus(var State: TNSPZRandGausState): TDCplx; external nspl;
function nspwRandGaus(var State: TNSPWRandGausState): Short; external nspl;
function nspvRandGaus(var State: TNSPVRandGausState): TWCplx; external nspl;

procedure nspsbRandGaus(var State: TNSPSRandGausState; Samps: PFloatArray; SampsLen: Integer); external nspl;
procedure nspcbRandGaus(var State: TNSPCRandGausState; Samps: PSCplxArray; SampsLen: Integer); external nspl;
procedure nspdbRandGaus(var State: TNSPDRandGausState; Samps: PDoubleArray; SampsLen: Integer); external nspl;
procedure nspzbRandGaus(var State: TNSPZRandGausState; Samps: PDCplxArray; SampsLen: Integer); external nspl;
procedure nspwbRandGaus(var State: TNSPWRandGausState; Samps: PShortArray; SampsLen: Integer); external nspl;
procedure nspvbRandGaus(var State: TNSPVRandGausState; Samps: PWCplxArray; SampsLen: Integer); external nspl;

procedure nspwToneInit(rfreq, phase: Float; mag: Short; var State: TNSPWToneState); external nspl;
procedure nspvToneInit(rfreq, phase: Float; mag: Short; var State: TNSPVToneState); external nspl;
procedure nspsToneInit(rfreq, phase, mag: Float; var State: TNSPSToneState); external nspl;
procedure nspcToneInit(rfreq, phase, mag: Float; var State: TNSPCToneState); external nspl;
procedure nspdToneInit(rfreq, phase, mag: Double; var State: TNSPDToneState); external nspl;
procedure nspzToneInit(rfreq, phase, mag: Double; var State: TNSPZToneState); external nspl;

function nspwTone(var State: TNSPWToneState): Short; external nspl;
function nspvTone(var State: TNSPVToneState): TWCplx; external nspl;
function nspsTone(var State: TNSPSToneState): Float; external nspl;

function _nspcTone(var State: TNSPCToneState): TSCplx; external nspl name 'nspcTone';
function nspcTone(var State: TNSPCToneState): TSCplx; assembler;
asm
	push	State
	call	_nspcTone
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdTone(var State: TNSPDToneState): Double; external nspl;
function nspzTone(var State: TNSPZToneState): TDCplx; external nspl;

procedure nspwbTone(var State: TNSPWToneState; Samps: PShortArray; SampsLen: Integer); external nspl;
procedure nspvbTone(var State: TNSPVToneState; Samps: PWCplxArray; SampsLen: Integer); external nspl;
procedure nspsbTone(var State: TNSPSToneState; Samps: PFloatArray; SampsLen: Integer); external nspl;
procedure nspcbTone(var State: TNSPCToneState; Samps: PSCplxArray; SampsLen: Integer); external nspl;
procedure nspdbTone(var State: TNSPDToneState; Samps: PDoubleArray; SampsLen: Integer); external nspl;
procedure nspzbTone(var State: TNSPZToneState; Samps: PDCplxArray; SampsLen: Integer); external nspl;

procedure nspwTrnglInit(rfrq, phase: Float; Mag: Short; Asym: Float; var State: TNSPWTrnglState); external nspl;
procedure nspvTrnglInit(rfrq, phase: Float; Mag: Short; Asym: Float; var State: TNSPVTrnglState); external nspl;
procedure nspsTrnglInit(rfrq, phase, Mag, Asym: Float; var State: TNSPSTrnglState); external nspl;
procedure nspcTrnglInit(rfrq, phase, Mag, Asym: Float; var State: TNSPCTrnglState); external nspl;
procedure nspdTrnglInit(rfrq, phase, Mag, Asym: Double; var State: TNSPDTrnglState); external nspl;
procedure nspzTrnglInit(rfrq, phase, Mag, Asym: Double; var State: TNSPZTrnglState); external nspl;

function nspwTrngl(var State: TNSPWTrnglState): Short; external nspl;
function nspvTrngl(var State: TNSPVTrnglState): TWCplx; external nspl;
function nspsTrngl(var State: TNSPSTrnglState): Float; external nspl;

function _nspcTrngl(var State: TNSPCTrnglState): TSCplx; external nspl name 'nspcTrngl';
function nspcTrngl(var State: TNSPCTrnglState): TSCplx; assembler;
asm
	push	State
	call	_nspcTrngl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdTrngl(var State: TNSPDTrnglState): Double; external nspl;
function nspzTrngl(var State: TNSPZTrnglState): TDCplx; external nspl;

procedure nspwbTrngl(var State: TNSPWTrnglState; Samps: PShortArray; SampsLen: Integer); external nspl;
procedure nspvbTrngl(var State: TNSPVTrnglState; Samps: PWCplxArray; SampsLen: Integer); external nspl;
procedure nspsbTrngl(var State: TNSPSTrnglState; Samps: PFloatArray; SampsLen: Integer); external nspl;
procedure nspcbTrngl(var State: TNSPCTrnglState; Samps: PSCplxArray; SampsLen: Integer); external nspl;
procedure nspdbTrngl(var State: TNSPDTrnglState; Samps: PDoubleArray; SampsLen: Integer); external nspl;
procedure nspzbTrngl(var State: TNSPZTrnglState; Samps: PDCplxArray; SampsLen: Integer); external nspl;
{$ENDIF nsp_UsesSampleGen}
{$IFDEF nsp_UsesVector}
// nsparith
//function nspcAdd(a, b: TSCplx): TSCplx; external nspl;
function nspcAdd(a, b: TSCplx): TSCplx;
begin
  Result.re := a.re + b.re;
  Result.im := a.im + b.im;
end;

//function nspcSub(a, b: TSCplx): TSCplx; external nspl;
function nspcSub(a, b: TSCplx): TSCplx;
begin
  Result.re := a.re - b.re;
  Result.im := a.im - b.im;
end;

//function nspcMpy(a, b: TSCplx): TSCplx; external nspl;
function nspcMpy(a, b: TSCplx): TSCplx;
begin
  Result.Re := a.Re * b.Re - a.Im * b.Im;
  Result.Im := a.Re * b.Im + a.Im * b.Re;
end;

function _nspcDiv(a, b: TSCplx): TSCplx; external nspl name 'nspcDiv';
function nspcDiv(a, b: TSCplx): TSCplx; assembler;
{
var
  D: float;
begin
  D := Sqr(b.Re) + Sqr(b.Im);
  Result.Re := (a.Re * b.Re + a.Im * b.Im) / D;
  Result.Im := (a.Re * b.Im - a.Im * b.Re) / D;
end;
}
asm
	push	b.im
	push	b.re
	push	a.im
	push	a.re
	call	_nspcDiv
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspcConj(a: TSCplx): TSCplx;
begin
  Result.re := a.re;
  Result.im := -a.im;
end;

//function nspcSet(re, im: Float): TSCplx; external nspl;
function nspcSet(re, im: Float): TSCplx;
begin
  Result.re := re;
  Result.im := im;
end;

procedure nspsbZero(Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbZero(Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbSet(Val: Float; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbSet(re, im: Float; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbCopy(Src: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbCopy(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbAdd1(Val: Float; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbAdd1(Val: TSCplx; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbAdd2(Src: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbAdd2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbAdd3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbAdd3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbSub1(Val: Float; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbSub1(Val: TSCplx; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbSub2(Val: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbSub2(Val: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbSub3(Src, Val: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbSub3(Src, Val: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbMpy1(Val: Float; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbMpy1(Val: TSCplx; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbMpy2(Src: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbMpy2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspsbMpy3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspcbMpy3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure  nspcbConj1(Vec: PSCplxArray; n: Integer); external nspl;
procedure  nspcbConj2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure  nspcbConjFlip2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure  nspcbConjExtend1(Vec: PSCplxArray; n: Integer); external nspl;
procedure  nspcbConjExtend2(Src: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
function nspzAdd(a, b: TDCplx): TDCplx; external nspl;
function nspzSub(a, b: TDCplx): TDCplx; external nspl;
function nspzMpy(a, b: TDCplx): TDCplx; external nspl;
function nspzDiv(a, b: TDCplx): TDCplx; external nspl;
function nspzConj(a: TDCplx): TDCplx; external nspl;
function nspzSet(re, im: Double): TDCplx; external nspl;
procedure  nspdbZero(Dst: PDoubleArray; n: Integer); external nspl;
procedure  nspzbZero(Dst: PDCplxArray; n: Integer); external nspl;
procedure  nspdbSet(Val: Double; Dst: PDoubleArray; n: Integer); external nspl;
procedure  nspzbSet(re, im: Double; Dst: PDCplxArray; n: Integer); external nspl;
procedure  nspdbCopy(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); external nspl;
procedure  nspzbCopy(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbAdd1(Val: Double; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbAdd1(Val: TDCplx; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbAdd2(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbAdd2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbAdd3(SrcA, SrcB: PDoubleArray; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbAdd3(SrcA, SrcB: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbSub1(Val: Double; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbSub1(Val: TDCplx; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbSub2(Val: PDoubleArray; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbSub2(Val: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbSub3(Src, Val: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspzbSub3(Src, Val: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspdbMpy1(Val: Double; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbMpy1(Val: TDCplx; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbMpy2(Src: PDoubleArray; Dst: PDoubleArray; n: Integer); external nspl;
procedure nspzbMpy2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspdbMpy3(SrcA, SrcB: PFloatArray; Dst: PFloatArray; n: Integer); external nspl;
procedure nspzbMpy3(SrcA, SrcB: PSCplxArray; Dst: PSCplxArray; n: Integer); external nspl;
procedure nspzbConj1(Vec: PDCplxArray; n: Integer); external nspl;
procedure nspzbConj2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspzbConjFlip2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
procedure nspzbConjExtend1(Vec: PDCplxArray; n: Integer); external nspl;
procedure nspzbConjExtend2(Src: PDCplxArray; Dst: PDCplxArray; n: Integer); external nspl;
function nspvAdd(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; external nspl;
function nspvSub(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; external nspl;
function nspvMpy(a, b: TWCplx; scaleMode: Integer; var scaleFactor: Integer): TWCplx; external nspl;
function nspvDiv(a, b: TWCplx): TWCplx; external nspl;
function nspvConj(a: TWCplx): TWCplx; external nspl;
function nspvSet(re, im: Short): TWCplx; external nspl;
procedure nspwbZero(Dst: PShortArray; n: Integer); external nspl;
procedure nspvbZero(Dst: PWCplxArray; n: Integer); external nspl;
procedure nspwbSet(Val: Short; Dst: PShortArray; n: Integer); external nspl;
procedure nspvbSet(re, im: Short; Dst: PWCplxArray; n: Integer); external nspl;
procedure nspwbCopy(Src: PShortArray; Dst: PShortArray; n: Integer); external nspl;
procedure nspvbCopy(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); external nspl;
procedure nspwbAdd1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbAdd1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbAdd2(Src: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbAdd2(Src: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbAdd3(SrcA, SrcB: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbAdd3(SrcA, SrcB: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbSub1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbSub1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbSub2(Val: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbSub2(Val: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbSub3(Src, Val: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbSub3(Src, Val: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbMpy1(Val: Short; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbMpy1(Val: TWCplx; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbMpy2(Src: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbMpy2(Src: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspwbMpy3(SrcA, SrcB: PShortArray; Dst: PShortArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbMpy3(SrcA, SrcB: PWCplxArray; Dst: PWCplxArray; n, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure  nspvbConj1(Vec: PWCplxArray; n: Integer); external nspl;
procedure  nspvbConj2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); external nspl;
procedure  nspvbConjFlip2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); external nspl;
procedure  nspvbConjExtend1(Vec: PWCplxArray; n: Integer); external nspl;
procedure  nspvbConjExtend2(Src: PWCplxArray; Dst: PWCplxArray; n: Integer); external nspl;
// nspcorr
procedure nspsAutoCorr(Src: PFloatArray; Len: Integer; Dst: PFloatArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); external nspl;
procedure nspcAutoCorr(Src: PSCplxArray; Len: Integer; Dst: PSCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); external nspl;
procedure nspdAutoCorr(Src: PDoubleArray; Len: Integer; Dst: PDoubleArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); external nspl;
procedure nspzAutoCorr(Src: PDCplxArray; Len: Integer; Dst: PDCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType); external nspl;
procedure nspwAutoCorr(Src: PShortArray; Len: Integer; Dst: PShortArray; nLags: Integer;
				 corrType: TNSPAutoCorrType;
				 scaleMode: Integer; var Scale: Integer); external nspl;
procedure nspvAutoCorr(Src: PWCplxArray; Len: Integer; Dst: PWCplxArray; nLags: Integer;
				 corrType: TNSPAutoCorrType;
				 scaleMode: Integer; var Scale: Integer); external nspl;
procedure nspsCrossCorr(SrcA: PFloatArray; lenA: Integer;
			SrcB: PFloatArray; lenB: Integer; Dst: PFloatArray;
			loLag, hiLag: Integer); external nspl;
procedure nspcCrossCorr(SrcA: PSCplxArray; lenA: Integer;
			SrcB: PSCplxArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure nspscCrossCorr(SrcA: PFloatArray; lenA: Integer;
			SrcB: PSCplxArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure nspcsCrossCorr(SrcA: PSCplxArray; lenA: Integer;
			SrcB: PFloatArray; lenB: Integer; Dst: PSCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure  nspdCrossCorr(SrcA: PDoubleArray; lenA: Integer;
			SrcB: PDoubleArray; lenB: Integer; Dst: PDoubleArray;
			loLag, hiLag: Integer); external nspl;
procedure  nspzCrossCorr(SrcA: PDCplxArray; lenA: Integer;
			SrcB: PDCplxArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure nspdzCrossCorr(SrcA: PDoubleArray; lenA: Integer;
			SrcB: PDCplxArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure nspzdCrossCorr(SrcA: PDCplxArray; lenA: Integer;
			SrcB: PDoubleArray; lenB: Integer; Dst: PDCplxArray;
			loLag, hiLag: Integer); external nspl;
procedure nspwCrossCorr(SrcA: PShortArray; lenA: Integer;
			SrcB: PShortArray; lenB: Integer; Dst: PShortArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); external nspl;
procedure nspvCrossCorr(SrcA: PWCplxArray; lenA: Integer;
			SrcB: PWCplxArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); external nspl;
procedure nspwvCrossCorr(SrcA: PShortArray; lenA: Integer;
			SrcB: PWCplxArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); external nspl;
procedure nspvwCrossCorr(SrcA: PWCplxArray; lenA: Integer;
			SrcB: PShortArray; lenB: Integer; Dst: PWCplxArray;
			loLag, hiLag, ScaleMode: Integer; var Scale: Integer); external nspl;
// nsplaw.h
procedure nspwbLinToALaw(src: PShortArray;  dst: PByteArray; len: Integer); external nspl;
procedure nspibLinToALaw(src: PLongArray;   dst: PByteArray; len: Integer); external nspl;
procedure nspsbLinToALaw(src: PFloatArray;  dst: PByteArray; len: Integer); external nspl;
procedure nspdbLinToALaw(src: PDoubleArray; dst: PByteArray; len: Integer); external nspl;

procedure nspwbALawToLin(src: PByteArray; dst: PShortArray;  len: Integer); external nspl;
procedure nspibALawToLin(src: PByteArray; dst: PLongArray;   len: Integer); external nspl;
procedure nspsbALawToLin(src: PByteArray; dst: PFloatArray;  len: Integer); external nspl;
procedure nspdbALawToLin(src: PByteArray; dst: PDoubleArray; len: Integer); external nspl;

procedure nspwbLinToMuLaw(src: PShortArray;  dst: PByteArray; len: Integer); external nspl;
procedure nspibLinToMuLaw(src: PLongArray;   dst: PByteArray; len: Integer); external nspl;
procedure nspsbLinToMuLaw(src: PFloatArray;  dst: PByteArray; len: Integer); external nspl;
procedure nspdbLinToMuLaw(src: PDoubleArray; dst: PByteArray; len: Integer); external nspl;

procedure nspwbMuLawToLin(src: PByteArray; dst: PShortArray;  len: Integer); external nspl;
procedure nspibMuLawToLin(src: PByteArray; dst: PLongArray;   len: Integer); external nspl;
procedure nspsbMuLawToLin(src: PByteArray; dst: PFloatArray;  len: Integer); external nspl;
procedure nspdbMuLawToLin(src: PByteArray; dst: PDoubleArray; len: Integer); external nspl;

procedure nspMuLawToALaw(src, dst: PByteArray; len: Integer); external nspl;
procedure nspALawToMuLaw(src, dst: PByteArray; len: Integer); external nspl;
// nsplnexp.h
procedure  nspsbLn1 (vec:	PFloatArray;	len: Integer); external nspl;
procedure  nspsbLn2 (src, dst:	PFloatArray;	len: Integer); external nspl;
procedure  nspsbExp1(vec:	PFloatArray;	len: Integer); external nspl;
procedure  nspsbExp2(src, dst:	PFloatArray;	len: Integer); external nspl;

procedure  nspdbLn1 (vec:	PDoubleArray;	len: Integer); external nspl;
procedure  nspdbLn2 (src, dst:	PDoubleArray;	len: Integer); external nspl;
procedure  nspdbExp1(vec:	PDoubleArray;	len: Integer); external nspl;
procedure  nspdbExp2(src, dst:	PDoubleArray;	len: Integer); external nspl;

procedure  nspwbLn1 (vec:	PShortArray;	len: Integer); external nspl;
procedure  nspwbLn2 (src, dst:	PShortArray;	len: Integer); external nspl;
procedure  nspwbExp1(vec:	PShortArray;	len: Integer;
		scaleMode: Integer; var scaleFactor: Integer); external nspl;
procedure  nspwbExp2(src, dst:	PShortArray;	len: Integer;
		scaleMode: Integer; var scaleFactor: Integer); external nspl;
// nspsampl
procedure nspsUpSample(Factor, Phase: Integer; Src: PFloatArray; SrcLen: Integer; Dst: PFloatArray); external nspl;
procedure nspcUpSample(Factor, Phase: Integer; Src: PSCplxArray; SrcLen: Integer; Dst: PSCplxArray); external nspl;
procedure nspdUpSample(Factor, Phase: Integer; Src: PDoubleArray; SrcLen: Integer; Dst: PDoubleArray); external nspl;
procedure nspzUpSample(Factor, Phase: Integer; Src: PDCplxArray; SrcLen: Integer; Dst: PDCplxArray); external nspl;
procedure nspwUpSample(Factor, Phase: Integer; Src: PShortArray; SrcLen: Integer; Dst: PShortArray); external nspl;
procedure nspvUpSample(Factor, Phase: Integer; Src: PWCplxArray; SrcLen: Integer; Dst: PWCplxArray); external nspl;

procedure nspsDownSample(Factor, Phase: Integer; Src: PFloatArray; SrcLen: Integer; Dst: PFloatArray); external nspl;
procedure nspcDownSample(Factor, Phase: Integer; Src: PSCplxArray; SrcLen: Integer; Dst: PSCplxArray); external nspl;
procedure nspdDownSample(Factor, Phase: Integer; Src: PDoubleArray; SrcLen: Integer; Dst: PDoubleArray); external nspl;
procedure nspzDownSample(Factor, Phase: Integer; Src: PDCplxArray; SrcLen: Integer; Dst: PDCplxArray); external nspl;
procedure nspwDownSample(Factor, Phase: Integer; Src: PShortArray; SrcLen: Integer; Dst: PShortArray); external nspl;
procedure nspvDownSample(Factor, Phase: Integer; Src: PWCplxArray; SrcLen: Integer; Dst: PWCplxArray); external nspl;
// nspvec
procedure nspsbSqr1(Vec: PFloatArray; Len: Integer); external nspl;
procedure nspcbSqr1(Vec: PSCplxArray; Len: Integer); external nspl;
procedure nspdbSqr1(Vec: PDoubleArray; Len: Integer); external nspl;
procedure nspzbSqr1(Vec: PDCplxArray; Len: Integer); external nspl;
procedure nspwbSqr1(Vec: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbSqr1(Vec: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;

procedure nspsbSqr2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspcbSqr2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer); external nspl;
procedure nspdbSqr2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspzbSqr2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer); external nspl;
procedure nspwbSqr2(Src: PShortArray; Dst: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspvbSqr2(Src: PWCplxArray; Dst: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;

procedure nspsbSqrt1(Vec: PFloatArray; Len: Integer); external nspl;
procedure nspcbSqrt1(Vec: PSCplxArray; Len: Integer); external nspl;
procedure nspdbSqrt1(Vec: PDoubleArray; Len: Integer); external nspl;
procedure nspzbSqrt1(Vec: PDCplxArray; Len: Integer); external nspl;
procedure nspwbSqrt1(Vec: PShortArray; Len: Integer); external nspl;
procedure nspvbSqrt1(Vec: PWCplxArray; Len: Integer); external nspl;

procedure nspsbSqrt2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspcbSqrt2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer); external nspl;
procedure nspdbSqrt2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspzbSqrt2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer); external nspl;
procedure nspwbSqrt2(Src: PShortArray; Dst: PShortArray; Len: Integer); external nspl;
procedure nspvbSqrt2(Src: PWCplxArray; Dst: PWCplxArray; Len: Integer); external nspl;

procedure nspsbThresh1(Vec: PFloatArray; Len: Integer; thresh: Float; relOp: Integer); external nspl;
procedure nspcbThresh1(Vec: PSCplxArray; Len: Integer; thresh: Float; relOp: Integer); external nspl;
procedure nspdbThresh1(Vec: PDoubleArray; Len: Integer; thresh: Double; relOp: Integer); external nspl;
procedure nspzbThresh1(Vec: PDCplxArray; Len: Integer; thresh: Double; relOp: Integer); external nspl;
procedure nspwbThresh1(Vec: PShortArray; Len: Integer; thresh: Short; relOp: Integer); external nspl;
procedure nspvbThresh1(Vec: PWCplxArray; Len: Integer; thresh: Short; relOp: Integer); external nspl;

procedure nspsbThresh2(Src: PFloatArray; Dst: PFloatArray; Len: Integer;
				thresh: Float; relOp: Integer); external nspl;
procedure nspcbThresh2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer;
				thresh: Float; relOp: Integer); external nspl;
procedure nspdbThresh2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer;
				thresh: Double; relOp: Integer); external nspl;
procedure nspzbThresh2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer;
				thresh: Double; relOp: Integer); external nspl;
procedure nspwbThresh2(Src: PShortArray; Dst: PShortArray; Len: Integer;
				thresh: Short; relOp: Integer); external nspl;
procedure nspvbThresh2(Src: PWCplxArray; Dst: PWCplxArray; Len: Integer;
				thresh: Short; relOp: Integer); external nspl;

procedure nspsbInvThresh1(Vec: PFloatArray; Len: Integer; Thresh: Float); external nspl;
procedure nspcbInvThresh1(Vec: PSCplxArray; Len: Integer; Thresh: Float); external nspl;
procedure nspdbInvThresh1(Vec: PDoubleArray; Len: Integer; Thresh: Double); external nspl;
procedure nspzbInvThresh1(Vec: PDCplxArray; Len: Integer; Thresh: Double); external nspl;

procedure nspsbInvThresh2(Src: PFloatArray; Dst: PFloatArray; Len: Integer; Thresh: Float); external nspl;
procedure nspcbInvThresh2(Src: PSCplxArray; Dst: PSCplxArray; Len: Integer; Thresh: Float); external nspl;
procedure nspdbInvThresh2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer; Thresh: Double); external nspl;
procedure nspzbInvThresh2(Src: PDCplxArray; Dst: PDCplxArray; Len: Integer; Thresh: Double); external nspl;

procedure nspsbAbs1(Vec: PFloatArray; Len: Integer); external nspl;
procedure nspdbAbs1(Vec: PDoubleArray; Len: Integer); external nspl;
procedure nspwbAbs1(Vec: PShortArray; Len: Integer); external nspl;

procedure nspsbAbs2(Src: PFloatArray; Dst: PFloatArray; Len: Integer); external nspl;
procedure nspdbAbs2(Src: PDoubleArray; Dst: PDoubleArray; Len: Integer); external nspl;
procedure nspwbAbs2(Src: PShortArray; Dst: PShortArray; Len: Integer); external nspl;

function nspsMax(Vec: PFloatArray; Len: Integer): Float; external nspl;
function nspdMax(Vec: PDoubleArray; Len: Integer): Double; external nspl;
function nspwMax(Vec: PShortArray; Len: Integer): Short; external nspl;

function nspsMean(Vec: PFloatArray; Len: Integer): Float; external nspl;
function nspdMean(Vec: PDoubleArray; Len: Integer): Double; external nspl;
function nspwMean(Vec: PShortArray; Len: Integer): Short; external nspl;

function nspsMin(Vec: PFloatArray; Len: Integer): Float; external nspl;
function nspdMin(Vec: PDoubleArray; Len: Integer): Double; external nspl;
function nspwMin(Vec: PShortArray; Len: Integer): Short; external nspl;

function nspsStdDev(Vec: PFloatArray; Len: Integer): Float; external nspl;
function nspdStdDev(Vec: PDoubleArray; Len: Integer): Double; external nspl;
function nspwStdDev(Vec: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): Short; external nspl;
// nspdotp
function nspsDotProd(SrcA, SrcB: PFloatArray; Len: Integer): Float; external nspl;

function _nspcDotProd(SrcA, SrcB: PSCplxArray; Len: Integer): TSCplx; external nspl name 'nspcDotProd';
function nspcDotProd(SrcA, SrcB: PSCplxArray; Len: Integer): TSCplx; assembler;
asm
	push	Len
	push	SrcB
	push	SrcA
	call	_nspcDotProd
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdDotProd(SrcA, SrcB: PDoubleArray; Len: Integer): Double; external nspl;
function nspzDotProd(SrcA, SrcB: PDCplxArray; Len: Integer): TDCplx; external nspl;
function nspwDotProd(SrcA, SrcB: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): Short; external nspl;
function nspvDotProd(SrcA, SrcB: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;

function _nspscDotProd(SrcA: PFloatArray; SrcB: PSCplxArray; Len: Integer): TSCplx; external nspl name 'nspscDotProd';
function nspscDotProd(SrcA: PFloatArray; SrcB: PSCplxArray; Len: Integer): TSCplx; assembler;
asm
	push	Len
	push	SrcB
	push	SrcA
	call	_nspscDotProd
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsDotProd(SrcA: PSCplxArray; SrcB: PFloatArray; Len: Integer): TSCplx; external nspl name 'nspcsDotProd';
function nspcsDotProd(SrcA: PSCplxArray; SrcB: PFloatArray; Len: Integer): TSCplx; assembler;
asm
	push	Len
	push	SrcB
	push	SrcA
	call	_nspcsDotProd
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzDotProd(SrcA: PDoubleArray; SrcB: PDCplxArray; Len: Integer): TDCplx; external nspl;
function nspzdDotProd(SrcA: PDCplxArray; SrcB: PDoubleArray; Len: Integer): TDCplx; external nspl;
function nspwvDotProd(SrcA: PShortArray; SrcB: PWCplxArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
function nspvwDotProd(SrcA: PWCplxArray; SrcB: PShortArray; Len, ScaleMode: Integer; var ScaleFactor: Integer): TWCplx; external nspl;
{$ENDIF}
{$IFDEF nsp_UsesConvolution}
//nspconv
procedure nspsConv(x: PFloatArray; xLen: Integer; h: PFloatArray; hLen: Integer; y: PFloatArray); external nspl;
procedure nspcConv(x: PSCplxArray; xLen: Integer; h: PSCplxArray; hLen: Integer; y: PSCplxArray); external nspl;
procedure nspdConv(x: PDoubleArray; xLen: Integer; h: PDoubleArray; hLen: Integer; y: PDoubleArray); external nspl;
procedure nspzConv(x: PDCplxArray; xLen: Integer; h: PDCplxArray; hLen: Integer; y: PDCplxArray); external nspl;
procedure nspscConv(x: PFloatArray; xLen: Integer; h: PSCplxArray; hLen: Integer; y: PSCplxArray); external nspl;
procedure nspcsConv(x: PSCplxArray; xLen: Integer; h: PFloatArray; hLen: Integer; y: PSCplxArray); external nspl;
procedure nspdzConv(x: PDoubleArray; xLen: Integer; h: PDCplxArray; hLen: Integer; y: PDCplxArray); external nspl;
procedure nspzdConv(x: PDCplxArray; xLen: Integer; h: PDoubleArray; hLen: Integer; y: PDCplxArray); external nspl;
procedure nspwConv(x: PShortArray; xLen: Integer; h: PShortArray; hLen: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
// nspcnv2d
procedure nspsConv2D(x: PFloatArray; xCols, xRows: Integer; h: PFloatArray; hCols, hRows: Integer; y: PFloatArray); external nspl;
procedure nspdConv2D(x: PDoubleArray; xCols, xRows: Integer; h: PDoubleArray; hCols, hRows: Integer; y: PDoubleArray); external nspl;
procedure nspwConv2D(x: PShortArray; xCols, xRows: Integer; h: PShortArray; hCols, hRows: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
// nspfir2
procedure nspsFilter2D(x: PFloatArray; xCols, xRows: Integer; h: PFloatArray; hCols, hRows: Integer; y: PFloatArray); external nspl;
procedure nspdFilter2D(x: PDoubleArray; xCols, xRows: Integer; h: PDoubleArray; hCols, hRows: Integer; y: PDoubleArray); external nspl;
procedure nspwFilter2D(x: PShortArray; xCols, xRows: Integer; h: PShortArray; hCols, hRows: Integer; y: PShortArray;
		ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
{$ENDIF}
{$IFDEF nsp_UsesIir}
//  nspiirl
procedure nspwIirlInit(IirType: TNSPIirType; Taps: PShortArray; Order: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspsIirlInit(IirType: TNSPIirType; Taps: PFloatArray; Order: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspcIirlInit(IirType: TNSPIirType; Taps: PSCplxArray; Order: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspdIirlInit(IirType: TNSPIirType; Taps: PDoubleArray; Order: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspzIirlInit(IirType: TNSPIirType; Taps: PDCplxArray; Order: Integer; var TapState: TNSPIirTapState); external nspl;

procedure nspwIirlInitBq(IirType: TNSPIirType; Taps: PShortArray; NumQuads: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspsIirlInitBq(IirType: TNSPIirType; Taps: PFloatArray; NumQuads: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspcIirlInitBq(IirType: TNSPIirType; Taps: PSCplxArray; NumQuads: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspdIirlInitBq(IirType: TNSPIirType; Taps: PDoubleArray; NumQuads: Integer; var TapState: TNSPIirTapState); external nspl;
procedure nspzIirlInitBq(IirType: TNSPIirType; Taps: PDCplxArray; NumQuads: Integer; var TapState: TNSPIirTapState); external nspl;

procedure nspwIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PShortArray; var DlyState: TNSPIirDlyState); external nspl;
procedure nspsIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PFloatArray; var DlyState: TNSPIirDlyState); external nspl;
procedure nspcIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PSCplxArray; var DlyState: TNSPIirDlyState); external nspl;
procedure nspdIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PDoubleArray; var DlyState: TNSPIirDlyState); external nspl;
procedure nspzIirlInitDlyl(const TapState: TNSPIirTapState; Dlyl: PDCplxArray; var DlyState: TNSPIirDlyState); external nspl;

function nspwIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Short;
		ScaleMode: Integer; var ScaleFactor: Integer): Short; external nspl;
function nspsIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Float): Float; external nspl;

function _nspcIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; external nspl name 'nspcIirl';
function nspcIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	DlyState
	push	TapState
	call	_nspcIirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Double): Double; external nspl;
function nspzIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TDCplx): TDCplx; external nspl;

function _nspscIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Float): TSCplx; external nspl name 'nspscIirl';
function nspscIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Float): TSCplx; assembler;
asm
	push	Samp
	push	DlyState
	push	TapState
	call	_nspscIirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; external nspl name 'nspcsIirl';
function nspcsIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	DlyState
	push	TapState
	call	_nspcsIirl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: Double): TDCplx; external nspl;
function nspzdIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState; Samp: TDCplx): TDCplx; external nspl;

procedure nspwbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PShortArray; outSamps: PShortArray;
			NumIters, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;
procedure nspsbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PFloatArray; outSamps: PFloatArray;
			NumIters: Integer); external nspl;
procedure nspcbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray;
			NumIters: Integer); external nspl;
procedure nspdbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDoubleArray; outSamps: PDoubleArray;
			NumIters: Integer); external nspl;
procedure nspzbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray;
			NumIters: Integer); external nspl;
procedure nspscbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PFloatArray; outSamps: PSCplxArray;
			NumIters: Integer); external nspl;
procedure nspcsbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray;
			NumIters: Integer); external nspl;
procedure nspdzbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDoubleArray; outSamps: PDCplxArray;
			NumIters: Integer); external nspl;
procedure nspzdbIirl(const TapState: TNSPIirTapState; var DlyState: TNSPIirDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray;
			NumIters: Integer); external nspl;
// nspiirh

procedure nspwIirInit(IirType: TNSPIirType; TapVals: PShortArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspsIirInit(IirType: TNSPIirType; TapVals: PFloatArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspcIirInit(IirType: TNSPIirType; TapVals: PSCplxArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspdIirInit(IirType: TNSPIirType; TapVals: PDoubleArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspzIirInit(IirType: TNSPIirType; TapVals: PDCplxArray; Order: Integer; var State: TNSPIirState); external nspl;

procedure nspcsIirInit(IirType: TNSPIirType; TapVals: PFloatArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspscIirInit(IirType: TNSPIirType; TapVals: PSCplxArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspzdIirInit(IirType: TNSPIirType; TapVals: PDoubleArray; Order: Integer; var State: TNSPIirState); external nspl;
procedure nspdzIirInit(IirType: TNSPIirType; TapVals: PDCplxArray; Order: Integer; var State: TNSPIirState); external nspl;

procedure nspwIirInitBq(IirType: TNSPIirType; TapVals: PShortArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspsIirInitBq(IirType: TNSPIirType; TapVals: PFloatArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspcIirInitBq(IirType: TNSPIirType; TapVals: PSCplxArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspdIirInitBq(IirType: TNSPIirType; TapVals: PDoubleArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspzIirInitBq(IirType: TNSPIirType; TapVals: PDCplxArray; NumQuads: Integer; var State: TNSPIirState); external nspl;

procedure nspcsIirInitBq(IirType: TNSPIirType; TapVals: PFloatArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspscIirInitBq(IirType: TNSPIirType; TapVals: PSCplxArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspzdIirInitBq(IirType: TNSPIirType; TapVals: PDoubleArray; NumQuads: Integer; var State: TNSPIirState); external nspl;
procedure nspdzIirInitBq(IirType: TNSPIirType; TapVals: PDCplxArray; NumQuads: Integer; var State: TNSPIirState); external nspl;

procedure nspIirFree(var State: TNSPIirState); external nspl;

function nspsIir(var State: TNSPIirState; Samp: Float): Float; external nspl;

function _nspcIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; external nspl name 'nspcIir';
function nspcIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcIir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdIir(var State: TNSPIirState; Samp: Double): Double; external nspl;
function nspzIir(var State: TNSPIirState; Samp: TDCplx): TDCplx; external nspl;
function nspwIir(var State: TNSPIirState; Samp: Short; ScaleMode: Integer; var ScaleFactor: Integer): Short; external nspl;

function _nspscIir(var State: TNSPIirState; Samp: Float): TSCplx; external nspl name 'nspscIir';
function nspscIir(var State: TNSPIirState; Samp: Float): TSCplx; assembler;
asm
	push	Samp
	push	State
	call	_nspscIir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; external nspl name 'nspcsIir';
function nspcsIir(var State: TNSPIirState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcsIir
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzIir(var State: TNSPIirState; Samp: Double): TDCplx; external nspl;
function nspzdIir(var State: TNSPIirState; Samp: TDCplx): TDCplx; external nspl;

procedure nspsbIir(var State: TNSPIirState; inSamps: PFloatArray; outSamps: PFloatArray; NumIters: Integer); external nspl;
procedure nspcbIir(var State: TNSPIirState; inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspdbIir(var State: TNSPIirState; inSamps: PDoubleArray; outSamps: PDoubleArray; NumIters: Integer); external nspl;
procedure nspzbIir(var State: TNSPIirState; inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspwbIir(var State: TNSPIirState; inSamps: PShortArray; outSamps: PShortArray; NumIters, ScaleMode: Integer; var ScaleFactor: Integer); external nspl;

procedure nspscbIir(var State: TNSPIirState; inSamps: PFloatArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspcsbIir(var State: TNSPIirState; inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspdzbIir(var State: TNSPIirState; inSamps: PDoubleArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspzdbIir(var State: TNSPIirState; inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
{$ENDIF}
{$IFDEF nsp_UsesLms}
procedure nspsLmslInit(LmsType: TNSPLmsType; Taps: PFloatArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspdLmslInit(LmsType: TNSPLmsType; Taps: PDoubleArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspcLmslInit(LmsType: TNSPLmsType; Taps: PSCplxArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspzLmslInit(LmsType: TNSPLmsType; Taps: PDCplxArray; TapsLen: Integer;
		   Step, Leak: Float; ErrDly: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspsLmslInitMr(LmsType: TNSPLmsType; Taps: PFloatArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspdLmslInitMr(LmsType: TNSPLmsType; Taps: PDoubleArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspcLmslInitMr(LmsType: TNSPLmsType; Taps: PSCplxArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspzLmslInitMr(LmsType: TNSPLmsType; Taps: PDCplxArray; TapsLen: Integer;
			Step, Leak: Float; ErrDly, DownFactor, DownPhase: Integer; var TapState: TNSPLmsTapState); external nspl;
procedure nspsLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PFloatArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspdLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PDoubleArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspcLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PSCplxArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspzLmslInitDlyl(var TapState: TNSPLmsTapState; Dlyl: PDCplxArray; AdaptB: Integer; var DlyState: TNSPLmsDlyState); external nspl;
function nspsLmslGetStep(const State: TNSPLmsTapState): Float; external nspl;
function nspsLmslGetLeak(const State: TNSPLmsTapState): Float; external nspl;
procedure nspsLmslSetStep(Step: Float; var State: TNSPLmsTapState); external nspl;
procedure nspsLmslSetLeak(Leak: Float; var State: TNSPLmsTapState); external nspl;
function nspsLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: Float): Float; external nspl;

function _nspcLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: TSCplx): TSCplx; external nspl name 'nspcLmsl';
function nspcLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	Samp.Im
	push	Samp.Re
	push	DlyState
	push	TapState
	call	_nspcLmsl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: Double): Double; external nspl;
function nspzLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp, Err: TDCplx): TDCplx; external nspl;

function _nspscLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp: Float; Err: TSCplx): TSCplx; external nspl name 'nspscLmsl';
function nspscLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp: Float; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	Samp
	push	DlyState
	push	TapState
	call	_nspscLmsl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzLmsl(var TapState: TNSPLmsTapState; var DlySTate: TNSPLmsDlyState; Samp: Double; Err: TDCplx): TDCplx; external nspl;
function nspsbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PFloatArray; Err: Float): Float; external nspl;

function _nspcbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PSCplxArray; Err: TSCplx): TSCplx; external nspl name 'nspcbLmsl';
function nspcbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PSCplxArray; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	inSamps
	push	DlyState
	push	TapState
	call	_nspcbLmsl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDoubleArray; Err: Double): Double; external nspl;
function nspzbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDCplxArray; Err: TDCplx): TDCplx; external nspl;

function _nspscbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PFloatArray; Err: TSCplx): TSCplx; external nspl name 'nspscbLmsl';
function nspscbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PFloatArray; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	inSamps
	push	DlyState
	push	TapState
	call	_nspscbLmsl
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzbLmsl(var TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
		inSamps: PDoubleArray; Err: TDCplx): TDCplx; external nspl;
function nspsLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Float): Float; external nspl;

function _nspcLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; external nspl name 'nspcLmslNa';
function nspcLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	DlyState
	push	TapState
	call	_nspcLmslNa
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Double): Double; external nspl;
function nspzLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TDCplx): TDCplx; external nspl;

function _nspscLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Float): TSCplx; external nspl name 'nspscLmslNa';
function nspscLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Float): TSCplx; assembler;
asm
	push	Samp
	push	DlyState
	push	TapState
	call	_nspscLmslNa
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function _nspcsLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; external nspl name 'nspcsLmslNa';
function nspcsLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TSCplx): TSCplx; assembler;
asm
	push	Samp.Im
	push	Samp.Re
	push	DlyState
	push	TapState
	call	_nspcsLmslNa
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: Double): TDCplx; external nspl;
function nspzdLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState; Samp: TDCplx): TDCplx; external nspl;
procedure nspsbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PFloatArray; outSamps: PFloatArray; NumIters: Integer); external nspl;
procedure nspcbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspdbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDoubleArray; outSamps: PDoubleArray; NumIters: Integer); external nspl;
procedure nspzbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspscbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PFloatArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspcsbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PSCplxArray; outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspdzbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDoubleArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspzdbLmslNa(const TapState: TNSPLmsTapState; var DlyState: TNSPLmsDlyState;
			inSamps: PDCplxArray; outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspsLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PFloatArray); external nspl;
procedure nspcLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PSCplxArray); external nspl;
procedure nspdLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PDoubleArray); external nspl;
procedure nspzLmslGetTaps(const TapState: TNSPLmsTapState; OutTaps: PDCplxArray); external nspl;
procedure nspsLmslSetTaps(InTaps: PFloatArray; var TapState: TNSPLmsTapState); external nspl;
procedure nspcLmslSetTaps(InTaps: PSCplxArray; var TapState: TNSPLmsTapState); external nspl;
procedure nspdLmslSetTaps(InTaps: PDoubleArray; var TapState: TNSPLmsTapState); external nspl;
procedure nspzLmslSetTaps(InTaps: PDCplxArray; var TapState: TNSPLmsTapState); external nspl;
procedure nspsLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PFloatArray); external nspl;
procedure nspcLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PSCplxArray); external nspl;
procedure nspdLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PDoubleArray); external nspl;
procedure nspzLmslGetDlyl(const TapState: TNSPLmsTapState; const DlyState: TNSPLmsDlyState; OutDlyl: PDCplxArray); external nspl;
procedure nspsLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PFloatArray; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspcLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PSCplxArray; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspdLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PDoubleArray; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspzLmslSetDlyl(const TapState: TNSPLmsTapState; inDlyl: PDCplxArray; var DlyState: TNSPLmsDlyState); external nspl;
procedure nspwLmslInit(LmsType: TNSPLmsType; Taps: PShortArray; TapsLen: Integer;
			Step, Leak: Short; ErrDly: Integer; var TapState: TNSPWLmsTapState); external nspl;
procedure nspwLmslInitDlyl(var TapState: TNSPWLmsTapState; Dlyl: PShortArray;
			AdaptB: Integer; var DlyState: TNSPWLmsDlyState); external nspl;
function nspwLmsl(var TapState: TNSPWLmsTapState; var DlyState: TNSPWLmsDlyState; Samp, Err: Short): Short; external nspl;
procedure nspwLmslSetTaps(InTaps: PShortArray; var TapState: TNSPWLmsTapState); external nspl;
procedure nspwLmslSetDlyl(const TapState: TNSPWLmsTapState; inDlyl: PShortArray; var DlyState: TNSPWLmsDlyState); external nspl;
procedure nspsLmsInit(LmsType: TNSPLmsType;
			TapVals: PFloatArray; TapsLen: Integer;
			DlyVals: PFLoatArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;
procedure nspdLmsInit(LmsType: TNSPLmsType;
			TapVals: PDoubleArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;
procedure nspcLmsInit(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PSCplxArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;
procedure nspzLmsInit(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDCplxArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;
procedure nspscLmsInit(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PFloatArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;
procedure nspdzLmsInit(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly: Integer; var State: TNSPLmsState); external nspl;

procedure nspsLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PFloatArray; TapsLen: Integer;
			DlyVals: PFLoatArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;
procedure nspdLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDoubleArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;
procedure nspcLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PSCplxArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;
procedure nspzLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDCplxArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;
procedure nspscLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PSCplxArray; TapsLen: Integer;
			DlyVals: PFloatArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;
procedure nspdzLmsInitMr(LmsType: TNSPLmsType;
			TapVals: PDCplxArray; TapsLen: Integer;
			DlyVals: PDoubleArray; Step, Leak: Float;
			ErrDly, DownFactor, DownPhase: Integer; var State: TNSPLmsState); external nspl;

procedure nspLmsFree(var State: TNSPLmsState); external nspl;

function nspsLms(var State: TNSPLmsState; Samp, Err: Float): Float; external nspl;
function nspdLms(var State: TNSPLmsState; Samp, Err: Double): Double; external nspl;

function _nspcLms(var State: TNSPLmsState; Samp, Err: TSCplx): TSCplx; external nspl name 'nspcLms';
function nspcLms(var State: TNSPLmsState; Samp, Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcLms
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspzLms(var State: TNSPLmsState; Samp, Err: TDCplx): TDCplx; external nspl;

function _nspscLms(var State: TNSPLmsState; Samp: Float; Err: TSCplx): TSCplx; external nspl;
function nspscLms(var State: TNSPLmsState; Samp: Float; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	Samp
	push	State
	call	_nspscLms
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzLms(var State: TNSPLmsState; Samp: Double; Err: TDCplx): TDCplx; external nspl;

function nspsbLms(var State: TNSPLmsState; inSamps: PFloatArray; Err: Float): Float; external nspl;
function nspdbLms(var State: TNSPLmsState; inSamps: PDoubleArray; Err: Double): Double; external nspl;

function _nspcbLms(var State: TNSPLmsState; inSamps: PSCplxArray; Err: TSCplx): TSCplx; external nspl name 'nspcbLms';
function nspcbLms(var State: TNSPLmsState; inSamps: PSCplxArray; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	inSamps
	push	State
	call	_nspcbLms
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspzbLms(var State: TNSPLmsState; inSamps: PDCplxArray; Err: TDCplx): TDCplx; external nspl;

function _nspscbLms(var State: TNSPLmsState; inSamps: PFloatArray; Err: TSCplx): TSCplx; external nspl name 'nspscbLms';
function nspscbLms(var State: TNSPLmsState; inSamps: PFloatArray; Err: TSCplx): TSCplx; assembler;
asm
	push	Err.Im
	push	Err.Re
	push	inSamps
	push	State
	call	_nspscbLms
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzbLms(var State: TNSPLmsState; inSamps: PDoubleArray; Err: TDCplx): TDCplx; external nspl;

procedure nspsLmsGetTaps(const State: TNSPLmsState; OutTaps: PFloatArray); external nspl;
procedure nspdLmsGetTaps(const State: TNSPLmsState; OutTaps: PDoubleArray); external nspl;
procedure nspcLmsGetTaps(const State: TNSPLmsState; OutTaps: PSCplxArray); external nspl;
procedure nspzLmsGetTaps(const State: TNSPLmsState; OutTaps: PDCplxArray); external nspl;

procedure nspsLmsSetTaps(InTaps: PFloatArray; var State: TNSPLmsState); external nspl;
procedure nspdLmsSetTaps(InTaps: PDoubleArray; var State: TNSPLmsState); external nspl;
procedure nspcLmsSetTaps(InTaps: PSCplxArray; var State: TNSPLmsState); external nspl;
procedure nspzLmsSetTaps(InTaps: PDCplxArray; var State: TNSPLmsState); external nspl;

procedure nspsLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PFloatArray); external nspl;
procedure nspdLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PDoubleArray); external nspl;
procedure nspcLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PSCplxArray); external nspl;
procedure nspzLmsGetDlyl(const State: TNSPLmsState; OutDlyl: PDCplxArray); external nspl;

procedure nspsLmsSetDlyl(inDlyl: PFloatArray; var State: TNSPLmsState); external nspl;
procedure nspdLmsSetDlyl(inDlyl: PDoubleArray; var State: TNSPLmsState); external nspl;
procedure nspcLmsSetDlyl(inDlyl: PSCplxArray; var State: TNSPLmsState); external nspl;
procedure nspzLmsSetDlyl(inDlyl: PDCplxArray; var State: TNSPLmsState); external nspl;

function nspsLmsGetStep(const State: TNSPLmsState): Float; external nspl;
function nspsLmsGetLeak(const State: TNSPLmsState): Float; external nspl;

procedure nspsLmsSetStep(Step: Float; var State: TNSPLmsState); external nspl;
procedure nspsLmsSetLeak(Leak: Float; var State: TNSPLmsState); external nspl;

function nspsLmsDes(var State: TNSPLmsState; Samp: Float; Des: Float): Float; external nspl;
function nspdLmsDes(var State: TNSPLmsState; Samp: Double; Des: Double): Double; external nspl;

function _nspcLmsDes(var State: TNSPLmsState; Samp: TSCplx; Des: TSCplx): TSCplx; external nspl name 'nspcLmsDes';
function nspcLmsDes(var State: TNSPLmsState; Samp: TSCplx; Des: TSCplx): TSCplx; assembler;
asm
	push	Des.Im
	push	Des.Re
	push	Samp.Im
	push	Samp.Re
	push	State
	call	_nspcLmsDes
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspzLmsDes(var State: TNSPLmsState; Samp: TDCplx; Des: TDCplx): TDCplx; external nspl;

function _nspscLmsDes(var State: TNSPLmsState; Samp: Float; Des: TSCplx): TSCplx; external nspl name 'nspscLmsDes';
function nspscLmsDes(var State: TNSPLmsState; Samp: Float; Des: TSCplx): TSCplx; assembler;
asm
	push	Des.Im
	push	Des.Re
	push	Samp
	push	State
	call	_nspscLmsDes
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspdzLmsDes(var State: TNSPLmsState; Samp: Double; Des: TDCplx): TDCplx; external nspl;

procedure nspsbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TFloatArray;
			outSamps: PFloatArray; NumIters: Integer); external nspl;
procedure nspdbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TDoubleArray;
			outSamps: PDoubleArray; NumIters: Integer); external nspl;
procedure nspcbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TSCplxArray;
			outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspzbLmsDes(var State: TNSPLmsState; const InSamps, DesSamps: TDCplxArray;
			outSamps: PDCplxArray; NumIters: Integer); external nspl;
procedure nspscbLmsDes(var State: TNSPLmsState;
			inSamps: PFloatArray;
			DesSamps: PSCplxArray;
			outSamps: PSCplxArray; NumIters: Integer); external nspl;
procedure nspdzbLmsDes(var State: TNSPLmsState;
			inSamps: PDoubleArray;
			DesSamps: PDCplxArray;
			outSamps: PDCplxArray; NumIters: Integer); external nspl;

function nspsLmsGetErrVal(const State: TNSPLmsState): Float; external nspl;
function nspdLmsGetErrVal(const State: TNSPLmsState): Double; external nspl;

function _nspcLmsGetErrVal(const State: TNSPLmsState): TSCplx; external nspl name 'nspcLmsGetErrVal';
function nspcLmsGetErrVal(const State: TNSPLmsState): TSCplx; assembler;
asm
	push	State
	call	_nspcLmsGetErrVal
	mov	esp, ebp
	mov	ecx, @Result
	mov	TSCplx([ecx]).re, eax
	mov	TSCplx([ecx]).im, edx
end;

function nspzLmsGetErrVal(const State: TNSPLmsState): TDCplx; external nspl;

procedure nspsLmsSetErrVal(Err: Float; var State: TNSPLmsState); external nspl;
procedure nspdLmsSetErrVal(Err: Double; var State: TNSPLmsState); external nspl;
procedure nspcLmsSetErrVal(Err: TSCplx; var State: TNSPLmsState); external nspl;
procedure nspzLmsSetErrVal(Err: TDCplx; var State: TNSPLmsState); external nspl;
{$ENDIF}
end.

